\\ Copyright 2015, 2016, 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

\\-----------------------------------------------------------------------------

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------

\\  *----*----*
\\  |    |    |
\\  *----*----*
\\  |    |    |
\\  *----*----*

\\ half size is factor 1/16
\\ mass 1/4, lengths 1/2 each
inertia_half(m) = 1/16*m;
check_exact(1/16, 1/4 * 1/2 * 1/2);
check_exact(1/16, 1/sqrt2^8);
check_exact(geom_inertia_of_rectangle(1/2,1/2), \
            inertia_half(geom_inertia_of_rectangle(1,1)), \
            "half");

\\ quarter size is factor 1/64
\\ mass 1/16, lengths 1/4 each
inertia_quarter(m) = 1/256*m;
check_exact(1/256, 1/16 * 1/4 * 1/4);
check_exact(1/256, 1/sqrt2^16);
check_exact(geom_inertia_of_rectangle(1/4,1/4), \
            inertia_quarter(geom_inertia_of_rectangle(1,1)), \
            "quarter");

\\ 1/sqrt2^3 size is factor 1/sqrt2^9
\\ mass 1/8, lengths 1/sqrt2^3 each = 1/8
inertia_div_sqrt2pow3(m) = 1/64*m;
check_exact(1/64,  (1/sqrt2^3)^4);
check_exact(1/64,  1/8 * 1/8);
check_exact(geom_inertia_of_rectangle(1/sqrt2^3, 1/sqrt2^3), \
            inertia_div_sqrt2pow3(geom_inertia_of_rectangle(1,1)), \
            "1/sqrt2^3 square");

inertia_scale_area(M,f) = M*f^4;
check_exact(geom_inertia_of_rectangle(2/3,2/3), \
            inertia_scale_area(geom_inertia_of_rectangle(2,2), 1/3), \
            "quarter");

\\      -------->
\\  0 *           * 1
\\      <--------

\\ TIf unit length, mass 2, area 1
\\ If  unit length, mass 1, area 1/2

check_exact(If, geom_inertia_rotate_xydir(If, -1)); \\ 180deg

{
  check_exact(If
              + If
              + 2*geom_inertia_of_point(Gf - 1/2),  \\ mass 2 shifted
              TIf);
}

\\ print("diff");
\\ 2*If - TIf
\\ geom_inertia_of_point(Gf - 1/2)

\\        *
\\      ^   ^
\\    /       \
\\  *           *
\\     \      /
\\       v  v

  \\ geom_inertia_rotate_xydir(If/2, b)
  \\ + geom_inertia_rotate_xydir(If/2 conj(b))
  \\
  \\ + geom_inertia_rotate_xydir(If/2, b)
  \\ + geom_inertia_rotate_xydir(If/2 conj(b))

\\-----------------------------------------------------------------------------

print("G ",BlobGSf);

\\-------------------
\\ blob 4a
\\ BlobGSf + g_to_start = BlobStart_limit
G_to_start = BlobStart_limit - BlobGSf;
print("G_to_start ",G_to_start);
start_to_G = - G_to_start;

G4a = BlobStart_limit + start_to_G/4;
print("G4a ",G4a);

G_to_G4a = G_to_start - G_to_start/4;
print("G_to_G4a ",G_to_G4a);
check_exact(G_to_G4a, G4a - BlobGSf);

b4a_end = BlobStart_limit + BlobDelta_limit/4;
print("b4a_end ",b4a_end);

\\-------------------
\\ blob 4b
G_to_end = BlobEnd_limit - BlobGSf;
print("G_to_end ",G_to_end);
end_to_G = - G_to_end;

G4b = BlobEnd_limit + end_to_G/4;
print("G4b ",G4b);

G_to_G4b = G_to_end - G_to_end/4;
print("G_to_G4b ",G_to_G4b);
check_exact(G_to_G4b, G4b - BlobGSf);

b4b_end = BlobEnd_limit - BlobDelta_limit/4;
check_exact(b4b_end - b4a_end,  BlobDelta_limit/2);

\\-------------------
\\ blob 2
b2_start = BlobEnd_limit - BlobDelta_limit/4;
print("b2_start ",b2_start);

G2 = b2_start + (- start_to_G/2); \\ rotate 180
print("G2 ",G2);

\\ BlobGSf + G_to_b2_start = b2_start
G_to_b2_start = b2_start - BlobGSf;
print("G_to_b2_start ",G_to_b2_start);

G_to_G2 = G_to_b2_start - (-G_to_start/2); \\ rotate 180
print("G_to_G2 ",G_to_G2);
check_exact(G_to_G2, G2 - BlobGSf);

\\-------------------
\\ twindragon
BlobPartsTM_limit = 5/8 - 3/8*I;
print("BlobPartsTM_limit ",BlobPartsTM_limit);

\\ BlobGSf + G_to_TM = BlobPartsTM_limit
G_to_TM = BlobPartsTM_limit - BlobGSf;
print("G_to_TM ",G_to_TM);

TIf_axes_minus45 = geom_inertia_rotate_xydir(TIf,conj(b));
check_exact(TIf_axes_minus45, R45 * TIf * R45^-1);
print("TIf_axes_minus45 ",TIf_axes_minus45);
print("TI minimum ",geom_inertia_principal_xyangle(TIf*1.0) * 180/Pi + 90);
print("TI maximum ",geom_inertia_principal_xyangle(TIf*1.0) * 180/Pi);
print("TI rot minimum ",geom_inertia_principal_xyangle(TIf_axes_minus45*1.0) * 180/Pi + 90);
print("TI rot maximum ",geom_inertia_principal_xyangle(TIf_axes_minus45*1.0) * 180/Pi);
31+45

TC  = (1/2 - 1/4*I);
TC1 = (1/2 - 1/2*I);
TC2 = (3/4 - 1/2*I);
TC3 = (3/4 - 1/4*I);

\\ k=9 length 8 start to end
print("twindragon length ",TC3 - TC1,"  ",abs(TC3 - TC1));
1/sqrt(2)^3
check_exact(geom_abs(TC3-TC1), 1/sqrt2^3);

\\-------------------
\\ BI = 2*1/256*BI + 1/16*BI + 1/256 * R * TI * R^-1
check_exact(119/128*BI, \
            BI - (2*inertia_scale_area(BI,1/4) \
                  + inertia_scale_area(BI,1/2)));

{
  BI16 = [621606693901/13106, -101336477040/6553, 0;
          -101336477040/6553, 405271210921/13106, 0;
          0, 0, 513438952411/6553];
  BI16 /= 2^16;  \\ mass
  BI16 /= 2^16;  \\ lengths
  BI16 = geom_rotate_x
  BI16 = geom_inertia_rotate_xydir(BI16, conj(b)); \\ axes -45
  check_exact(geom_inertia_rotate_xydir(BI16,-1), BI16); \\ no change for 180
}
{
  BI17 = [19893997947611/104854, -3243862498278/52427, 0;
          -3243862498278/52427, 12972819870497/104854, 0;
          0, 0, 16433408909054/52427];
  BI17 /= 2^17;  \\ mass
  BI17 /= 2^17;  \\ lengths
  print("BI17 unrot minimum ",geom_inertia_principal_xyangle(BI17*1.0)*180/Pi + 90);
  print("BI17 unrot maximum ",geom_inertia_principal_xyangle(BI17*1.0)*180/Pi);
  \\ print("B17 Ix ",geom_inertia_Ix(BI17)*1.0);
  \\ print("B17 Iy ",geom_inertia_Iy(BI17)*1.0);
  BI17 = geom_inertia_rotate_xydir(BI17, conj(b)); \\ axes -45
}
print("B16 Ix ",geom_inertia_Ix(BI16)*1.0);
print("B17 Ix ",geom_inertia_Ix(BI17)*1.0);
print("B16 Iy ",geom_inertia_Iy(BI16)*1.0);
print("B17 Iy ",geom_inertia_Iy(BI17)*1.0);
bestappr(BI17[1,1],1000)
bestappr(BI17[2,2],1000)
bestappr(BI17[1,2],1000)
bestappr(BI17[3,3],1000)
print("BI17 minimum ",geom_inertia_principal_xyangle(BI17*1.0) * 180/Pi + 90);
print("BI17 maximum ",geom_inertia_principal_xyangle(BI17*1.0) * 180/Pi);

\\ k=8 twindragon k=4 is 4 sides of k=4 is 4*2^(k-4) = 2^(k-2) segments
\\ TIf is scaled to unit length, mass 2, area 1
\\ want scale down to length 1/sqrt2^3 = 0.35355
check_exact(BlobSegments(16), 26212)
check_exact(BlobSegments(18), 104854)
check_exact(BlobSegments(20), 419428)
BlobSegments(20)/2^20
b_mass = BlobSegments_limit;  \\ 2/5
b2_mass = BlobSegments_limit / 4;
b4_mass = BlobSegments_limit / 4^2;
t4_mass = 1/4;
print("BlobSegments_limit ",BlobSegments_limit);
print("b2_mass ",b2_mass);
print("b4_mass ",b4_mass);
print("t4_mass ",t4_mass);
print("remaining mass ", b_mass - (b2_mass + 2*b4_mass));
check_exact(b2_mass + 2*b4_mass + t4_mass,  b_mass, "b_mass total");

{
  print("BI");
  BI = 128/119
      * (
         inertia_scale_area(TIf_axes_minus45, 1/sqrt2^3)
         + geom_inertia_of_point(G_to_G4a) * b4_mass
         + geom_inertia_of_point(G_to_G4b) * b4_mass
         + geom_inertia_of_point(G_to_G2) * b2_mass
         + geom_inertia_of_point(G_to_TM) * t4_mass
        )
}
print("BI Ix ",geom_inertia_Ix(BI)*1.0);
print("BI Iy ",geom_inertia_Iy(BI)*1.0);
print("BI minimum ",geom_inertia_principal_xyangle(BI*1.0) * 180/Pi + 90);
print("BI maximum ",geom_inertia_principal_xyangle(BI*1.0) * 180/Pi);
{
  check_exact(BI,
              2*inertia_scale_area(BI, 1/4)
              + inertia_scale_area(BI, 1/2)
              + inertia_scale_area(TIf_axes_minus45, 1/sqrt2^3)
              + geom_inertia_of_point(G_to_G4a) * b4_mass
              + geom_inertia_of_point(G_to_G4b) * b4_mass
              + geom_inertia_of_point(G_to_G2) * b2_mass
              + geom_inertia_of_point(G_to_TM) * t4_mass
             );
}
print("Ix/Iy ",geom_inertia_Ix(BI) / geom_inertia_Iy(BI));

print("BI to start-end horizontal");
geom_inertia_rotate_xydir(BI, b)


print("BI to principal");
default(format,"f.6");
geom_inertia_rotate_xydir(BI*1.0, cos(14*Pi/180) + sin(14*Pi/180)*I)

print("G4a + G4b");
geom_inertia_of_point(G_to_G4a) + geom_inertia_of_point(G_to_G4b)


inertia_xydir_simplify(p) = p / gcd(geom_point_x(p), geom_point_y(p));

inertia_principal_xydirsquared(M) =
{
  [(geom_inertia_Ix(M) - geom_inertia_Iy(M))/2,
   - geom_inertia_Ixy(M)];
}
BI_principal_xydirsquared = inertia_principal_xydirsquared(BI);
print("BI_principal_xydirsquared ",BI_principal_xydirsquared,"  ",inertia_xydir_simplify(BI_principal_xydirsquared));


inertia_principal_xydir(M) = \
  geom_xydir_half(inertia_principal_xydirsquared(M));

BI_principal_xydir = inertia_principal_xydir(BI);
print("BI_principal_xydir ",BI_principal_xydir, "  ",inertia_xydir_simplify(BI_principal_xydir));

(4+I)*conj(b)


quit

\\-----------------------------------------------------------------------------
print("\ntwo");
\\
\\            *   /
\\              /
\\            v
\\          ^
\\        /
\\      /  *
\\    /

print("G_to_end      = ",G_to_end);
print("(-end_to_G/2) = ",  (-end_to_G/2));
Gt = G_to_end + (-end_to_G/2);
print("sum           = ",Gt);

Gtwo = (0*b_mass + Gt*b2_mass)/(b_mass + b2_mass);
print("Gtwo ",Gtwo);
Gtwo_to_G = 0 - Gtwo;
print("Gtwo_to_G ",Gtwo_to_G);

Gtwo_to_Gt = Gt - Gtwo;
print("Gtwo_to_Gt ",Gtwo_to_Gt);

{
  print("Itwo");
  Itwo = BI + inertia_scale_area(BI,1/2)
        + geom_inertia_of_point(Gtwo_to_G) * b_mass
        + geom_inertia_of_point(Gtwo_to_Gt) * b2_mass
}
print("Itwo minimum ",geom_inertia_principal_xyangle(Itwo*1.0) * 180/Pi + 90);
print("Itwo maximum ",geom_inertia_principal_xyangle(Itwo*1.0) * 180/Pi);


\\-----------------------------------------------------------------------------
print("\n--------------\nsum of blobs");
print("Gf ",Gf);
print("b_mass ",b_mass);
{
  my(k=100);
  Itotal=geom_inertia_of_point(0);
  my(total_mass=0);
  my(GS=0);
  forstep(i=0,-k,-1,
         my(g = BlobGSf*b^i);
         my(mass = b_mass * 2^i);
         total_mass += mass;
         GS += mass*g;
  );
  forstep(i=-2,-k,-1,
         my(g = 1 - I*BlobGSf*b^i);
         my(mass = b_mass * 2^i);
         total_mass += mass;
         GS += mass*g;
  );
  print("total_mass ",total_mass*1.0);
  GS /= total_mass;
  print("GS ",GS*1.0,"  ",GS*1.0);

  forstep(i=0,-k,-1,
         my(Ipart = inertia_scale_area(BI, sqrt2^i));
         my(xydir = b^-i);  \\ i=0 dir=0, i=-1 dir=1, i=-2,dir=2
         Ipart = geom_inertia_rotate_xydir(Ipart,xydir);
          Itotal += Ipart;
         my(g = BlobGSf*b^i);
         my(GS_to_g = g - GS);
         my(alpha_max=geom_inertia_principal_xyangle(Ipart*1.0) * 180/Pi);
         if(i>-10,
            print(" i=",i,"  g=",g,"  GS_to_g=",GS_to_g," alpha_max ",alpha_max));
         my(mass = b_mass * 2^i);
         Itotal += geom_inertia_of_point(GS_to_g) * mass;
         centroid += mass*GS_to_g;
         );
  forstep(i=-2,-k,-1,
         my(Ipart = inertia_scale_area(BI, sqrt2^i));
         my(xydir=b^(-i-2));  \\ i=-2 dir=0, i=-3 dir=1
         Ipart = geom_inertia_rotate_xydir(Ipart,xydir);
          Itotal += Ipart;
         my(g = 1 - I*BlobGSf*b^i);
         my(GS_to_g = g - GS);
         my(alpha_max=geom_inertia_principal_xyangle(Ipart*1.0) * 180/Pi);
         if(i>-10,
            print(" i=",i,"  g=",g,"  GS_to_g=",GS_to_g," alpha_max ",alpha_max);
            );
         my(mass = b_mass * 2^i);
         Itotal += geom_inertia_of_point(GS_to_g) * mass;
         centroid += mass*GS_to_g;
         );
  print("centroid   ",centroid*1.0);
  Itotal*1.0
}
print("Itotal maximum ",geom_inertia_principal_xyangle(Itotal*1.0) * 180/Pi + 90);
print("Itotal maximum ",geom_inertia_principal_xyangle(Itwo*1.0) * 180/Pi);

If*1.0
