\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
read("../tools/memoize.gp");
read("pol-pfrac.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

gf_terms(g,n) = my(x=variable(g)); \
  concat(vector(min(n,valuation(g,x)), i, 0), \
         Vec(g + O(x^n)));

L(k) = if(k<0,error("L() is for k>=0")); \
        if(k<=3,return(2^k)); \
        L(k-1) + 2*L(k-3);
L=memoize(L);
gL(x) = -1 + 2* (1 + x^2) / (1 - x - 2*x^3);

R(k)=1 + sum(j=0,k-1, L(j));
R = memoize(R);
gR(x)= 1 - 2/(1-x) + 2*(1 + x + x^2)/(1-x-2*x^3);

gU(x)=(3 - x^2 - 4*x^3 - 2*x^4) / ((1 - x - 2*x^3)*(1-x))

\\-----------------------------------------------------------------------------
vector_non_zeros(v) = select((x)->x!=0, v);

str_poly_ascending__join_strings(v,sep) =
{
  concat(vector(2*#v-1,i, if(i%2,v[(i+1)\2],sep)));
}

str_poly_ascending(p) =
{
  if(type(p) == "t_VEC",
     return(str_poly_ascending__join_strings(apply(str_poly_ascending,p),
                                             "\n + ")));

  my(d = denominator(p));
  if(poldegree(d) != 0,
     p = numerator(p);
     my(low = polcoeff(d, 0));
     if(low==0, error("Oops, constant term of denominator is 0"));
     d /= low;
     p /= low;
     my(fra=lcm(vector_non_zeros(apply(denominator,Vec(p)))));
     if(fra==0, error("oops fra=",fra));
     p *= fra;
     my(p_single = (poldegree(p) == 0
                    || poldegree(p) == valuation(p,variable(p))));
     my(d_single = (poldegree(d) == 0
                    || poldegree(d) == valuation(d,variable(d))));
     p_single = 0;
     d_single = 0;
     return(concat([if(fra==1, "", Str(1/fra,"*")),
                    if(p_single,"","("),
                    str_poly_ascending(p),
                    if(p_single,"",")"),
                    "/",
                    if(d_single,"","("),
                    str_poly_ascending(d),
                    if(d_single,"",")")
                   ])));
  if(poldegree(p) <= 0, return(Str(p)));
  my(str="");
  for(i=0,poldegree(p),
     my(coeff=polcoeff(p,i));
     if(coeff == 0, next());
     if(str!="",
        if(real(coeff)<0 || (real(coeff)==0 && imag(coeff)<0),
           str=concat(str," - "); coeff = -coeff,
           str=concat(str," + ")));
     str = concat([str,coeff*variable(p)^i]));
  str;
}

\\ str = str_poly_ascending( (x^2 + 1/2*x + 1)/(2*x^3 + x - 1) );
\\ print("str = "str);
\\ 
\\ str = str_poly_ascending( ((-1/4 + 1/4*I)*x^2 + (1/8 + 1/8*I))/(x^3 - 1/4*I*x + (1/8 + 1/8*I)) );
\\ print("str = "str);
\\ 
\\ str = str_poly_ascending( (x^2)/(2*x^3 + x - 1) );
\\ print("str = "str);

\\-----------------------------------------------------------------------------

Rpred_1e      = 1;
Rpred_1o      = 2;
Rpred_2eA     = 3;
Rpred_3oA     = 4;
Rpred_3oB     = 5;
Rpred_2eB_3oC = 6;
Rpred_3eA     = 7;
Rpred_3eB     = 8;
Rpred_3eC     = 9;
Rpred_non     = 10;
Rpred_matrix =
{
  [ 1,  2;
    3,  6;
    1,  4;
    7,  8;
    9,  4;
    5,  6;
    3, 10;
   10, 10;
   10,  6;
   10, 10 ];
}

M = matrix(10,10);
{
  for(s=1,Rpred_non-1,
     for(b=1,2,
        my(to=Rpred_matrix[s,b]);
        if(to != Rpred_non, M[s,to]++)));
}
M

initial = vectorv(10,i, i!=Rpred_non);

\\ M*g*x + initial = g
\\ (1 -M*x)*g = initial
g = matsolve(matid(10)-M*x, initial);

g1e = g[Rpred_1e];
gf_terms(gR(x),20)
gf_terms(g1e,20)

check_exact(g[1],gR(x));
check_exact(g[2],gL(x));
{
for(i=1,#g, 
   my(p = g[i]);
   print(g[i]);
   my(v = pol_partial_fractions(p));
   for(j=1,#v,
      print("  ",str_poly_ascending(v[j]))); \\ ,"         = ",v[j]
   print("");
);
}

got_gU = {
  g[Rpred_3oA]
  + g[Rpred_3oB]
  + g[Rpred_2eB_3oC]
  + g[Rpred_3eA]
  + g[Rpred_3eB]
  + g[Rpred_3eC];
}
got_gU
gU(x)

gf_terms(1/3*(1 + x)/(1 + x + x^2), 10)*3

{
gf_terms(  1/3*(4)/(1 - x)
+  1/3*(-2)/(1 - 2*x + x^2)
+  1/3*(1 + x)/(1 + x + x^2), 20
)
}
vector(15,k,my(k=k-1); - (k - floor(k/3)) + 1)