\\ Copyright 2014, 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


read("../tools/memoize.gp");

b = 1+I;
L(k) = if(k<0,error("L k<0"), k<=3,2^k, L(k-1) + 2*L(k-3));
L=memoize(L);
R(k)=1 + sum(j=0,k-1, L(j));
R = memoize(R);
RQ(k)=1 + sum(j=0,k-1, LQ(j));
RQ = memoize(RQ);
B(k) = R(k+1);
A(k) = 2^(k-1) - B(k)/4;
JA(k) = A(k+1) - 2*A(k);
JEQ(k) = (3/5 + 1/5*I)*b^k - (1/10 - 3/10*I)*(-1)^k;
LQ(k) = if(k<0,error("LQ() is for k>=0")); \
        if(k==0,return(1)); \
        if(k<=3,return(2^(k-1))); \
        LQ(k-1) + 2*LQ(k-3);
LQ = memoize(LQ);
gLQ(x) = (1 + x^2) / (1 - x - 2*x^3);

GLQ(k) = 1/2*b^max(1,k);
GLQN(k) = LQ(k)*1/2*b^max(1,k);

\\-----------------------------------------------------------------------------

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\n%s\n%s",x,y)));
  return(0);
}

check_count = 0;
check_equal(got,want) =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok "check_count));
}

check_exact(got,want) =
{
  check_count++;
  if(got!= want,
     print("wrong\n");
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count));
}

\\-----------------------------------------------------------------------------
JEQ_sum(k) = sum(j=2,k-1,JEQ(j));

\\ JEQ(k) = (3/5 + 1/5*I)*b^k - (1/10 - 3/10*I)*(-1)^k;

JEQ_sum_by_powers(k) =
{
  (3/5 + 1/5*I)*(b^(k)-1)/(b-1)
  - (1/10 - 3/10*I)*((-1)^(k) - 1)/((-1) - 1)
  - JEQ(0)
  - JEQ(1);
}
check_exact(vector(100,k,my(k=k+1); JEQ_sum_by_powers(k)), \
            vector(100,k,my(k=k+1); JEQ_sum(k)))  /* k>=2 */

JEQ_sum_by_powers(k) =
{
  1/20*(   (4 - 12*I)*b^k
         + (1 - 3*I)*(-1)^k
        -25 - 5*I );
}
check_exact(vector(100,k,my(k=k+1); JEQ_sum_by_powers(k)), \
            vector(100,k,my(k=k+1); JEQ_sum(k)))  /* k>=2 */

JEQ_sum_by_powers(k) =
{
  (1/5 - 3/5*I)*b^k
  + (1/20 - 3/20*I)*(-1)^k
  + (-5/4 - 1/4*I);
}
check_exact(vector(100,k,my(k=k+1); JEQ_sum_by_powers(k)), \
            vector(100,k,my(k=k+1); JEQ_sum(k)))  /* k>=2 */


\\-----------------------------------------------------------------------------

GRQN(k) = \
  if(k==0,return(1/2-1/2*I)); \
  (57-21*I)/82 \
  + b^k/82 * ( \
      ( 70 - 15*I) * RQ(k) \
    + (  3 + 14*I) * RQ(k+1) \
    + (-17 - 11*I) * RQ(k+2) \
    );
GRQN = memoize(GRQN);

GJAN(k) = {
    if(k<3,error("GJAN() is for k>=3"));
      1/164*(-21 - 57*I)
    + 1/20 *(  1 - 3*I )*(-1)^k
    + 1/410*( 32 - I   )*b^k
    + 1/82 * b^k
    * (   (57 + 20*I)*JA(k)
        - ( 4 +  5*I)*JA(k+1)
        + ( 9 +    I)*JA(k+2) );
}

GRQN_by_LQ(k) = 1-1/2*I + (1 + 1/2*I) * sum(j=0,k-1, LQ(j)*b^j);
check_exact(vector(100,k,my(k=k+0); GRQN_by_LQ(k)), \
            vector(100,k,my(k=k+0); GRQN(k)))  /* k>=1 */

check_exact(vector(100,k,my(k=k+1); GRQN(k)),  /* k>=1 */  \
            vector(100,k,my(k=k+1); 1-1/2*I + (1 + 1/2*I) * sum(j=0,k-1, LQ(j)*b^j)))

check_exact(vector(100,k,my(k=k+1); GRQN(k) - (1-1/2*I)),  /* k>=1 */  \
            vector(100,k,my(k=k+1); (1 + 1/2*I) * sum(j=0,k-1, LQ(j)*b^j)))

check_exact(vector(100,k,my(k=k+1); (GRQN(k) - (1-1/2*I))/(1 + 1/2*I)),  /* k>=1 */  \
            vector(100,k,my(k=k+1); sum(j=0,k-1, LQ(j)*b^j)))

LQb_sum(k) = sum(j=0,k-1, LQ(j)*b^j);

LQb_sum_by_GRQN(k) = (GRQN(k) - (1-1/2*I)) / (1+1/2*I);
check_exact(vector(100,k,my(k=k+1); LQb_sum(k)), \
            vector(100,k,my(k=k+1); LQb_sum_by_GRQN(k)))  /* k>=1 */

LQb_sum_by_GRQN(k) = (4/5 - 2/5*I)*GRQN(k) + (-3/5 + 4/5*I);
check_exact(vector(100,k,my(k=k+1); LQb_sum(k)), \
            vector(100,k,my(k=k+1); LQb_sum_by_GRQN(k)))  /* k>=2 */

\\-----------------------------------------------------------------------------
GJAN_by_GRQN(k) = JEQ_sum(k) + sum(j=2,k-1, b^(j+1)*JA(j+1) - 1/2*b^j*LQ(j));
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) = JEQ_sum(k) + sum(j=2,k-1, b^(j+1)*(1/4*(2*LQ(j) + 2*LQ(j+1) - LQ(j+2) - 2)) - 1/2*b^j*LQ(j));
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */


GJAN_by_GRQN(k) =
{
  JEQ_sum(k) 
  + sum(j=2,k-1, 1/4*b^(j+1)*(-2 + 2*LQ(j) + 2*LQ(j+1) - LQ(j+2)) - 1/2*b^j*LQ(j));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  JEQ_sum(k) 
  + sum(j=2,k-1, 1/4*b^(j+1)*(-2))
  + sum(j=2,k-1, 1/4*b^(j+1)*(2*LQ(j) + 2*LQ(j+1) - LQ(j+2)) - 1/2*b^j*LQ(j));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  JEQ_sum(k) 
  + -1/2*b*sum(j=2,k-1, b^j)
  + sum(j=2,k-1, 1/4*b^(j+1)*(2*LQ(j) + 2*LQ(j+1) - LQ(j+2)) - 1/2*b^j*LQ(j));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  JEQ_sum(k) 
  + -1/2*b*((b^k-1)/(b-1) - b^0 - b^1)
  + sum(j=2,k-1, 1/4*b^(j+1)*(2*LQ(j) + 2*LQ(j+1) - LQ(j+2)) - 1/2*b^j*LQ(j));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  JEQ_sum(k) 
  + -1/2*b*(-I*(b^k-1) - b^0 - b^1)
  + sum(j=2,k-1, 1/4*b^(j+1)*(2*LQ(j) + 2*LQ(j+1) - LQ(j+2)) - 1/2*b^j*LQ(j));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  (1/5 - 3/5*I)*b^k
  + (1/20 - 3/20*I)*(-1)^k
  + (-5/4 - 1/4*I)
  + -1/2*b*-I*(b^k-1)
  + (1/2 + 3/2*I)
  + sum(j=2,k-1, 1/4*b^(j+1)*(2*LQ(j) + 2*LQ(j+1) - LQ(j+2)) - 1/2*b^j*LQ(j));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  (-3/10 - 1/10*I)*b^k
  + (1/20 - 3/20*I)*(-1)^k
  + (-1/4 + 3/4*I)
  + sum(j=2,k-1, 2*1/4*b^(j+1)*LQ(j) + 2*1/4*b^(j+1)*LQ(j+1) - 1/4*b^(j+1)*LQ(j+2) - 1/2*b^j*LQ(j));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  (-3/10 - 1/10*I)*b^k
  + (1/20 - 3/20*I)*(-1)^k
  + (-1/4 + 3/4*I)
  + sum(j=2,k-1, 1/2*I*b^j*LQ(j) + 1/2*b^(j+1)*LQ(j+1) + (-1/8 + 1/8*I)*b^(j+2)*LQ(j+2));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

\\ LQb_sum(k) = sum(j=0,k-1, LQ(j)*b^j);
\\ LQb_sum_by_GRQN(k) = (4/5 - 2/5*I)*GRQN(k) + (-3/5 + 4/5*I);
GJAN_by_GRQN(k) =
{
  (-3/10 - 1/10*I)*b^k
  + (1/20 - 3/20*I)*(-1)^k
  + (-1/4 + 3/4*I)
  + 1/2*I*(LQb_sum(k) - b^0*LQ(0) - b^1*LQ(1))
  + sum(j=2,k-1, 1/2*b^(j+1)*LQ(j+1) + (-1/8 + 1/8*I)*b^(j+2)*LQ(j+2));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  (-3/10 - 1/10*I)*b^k
  + (1/20 - 3/20*I)*(-1)^k
  + (-1/4 + 3/4*I)
  + 1/2*I*(LQb_sum(k) - b^0*LQ(0) - b^1*LQ(1))
  + 1/2*(LQb_sum(k+1) - b^0*LQ(0) - b^1*LQ(1) - b^2*LQ(2))
  + sum(j=2,k-1, (-1/8 + 1/8*I)*b^(j+2)*LQ(j+2));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  (-3/10 - 1/10*I)*b^k
  + (1/20 - 3/20*I)*(-1)^k
  + (-1/4 + 3/4*I)
  + 1/2*I*(LQb_sum(k) - b^0*LQ(0) - b^1*LQ(1))
  + 1/2*(LQb_sum(k+1) - b^0*LQ(0) - b^1*LQ(1) - b^2*LQ(2))
  + (-1/8 + 1/8*I)*(LQb_sum(k+2) - b^0*LQ(0) - b^1*LQ(1) - b^2*LQ(2) - b^3*LQ(3));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  (-3/10 - 1/10*I)*b^k
  + (1/20 - 3/20*I)*(-1)^k
  + (-1/4 + 3/4*I)
  + 1/2*I*(- b^0*LQ(0) - b^1*LQ(1))
  + 1/2*( - b^0*LQ(0) - b^1*LQ(1) - b^2*LQ(2))
  + (-1/8 + 1/8*I)*( - b^0*LQ(0) - b^1*LQ(1) - b^2*LQ(2) - b^3*LQ(3))
  + 1/2*I*LQb_sum(k)
  + 1/2*LQb_sum(k+1)
  + (-1/8 + 1/8*I)*LQb_sum(k+2);
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  (-3/10 - 1/10*I)*b^k
  + (1/20 - 3/20*I)*(-1)^k
  + (1/8 - 3/8*I)
  + 1/2*I*LQb_sum(k)
  + 1/2*LQb_sum(k+1)
  + (-1/8 + 1/8*I)*LQb_sum(k+2);
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

\\ LQb_sum_by_GRQN(k) = (4/5 - 2/5*I)*GRQN(k) + (-3/5 + 4/5*I);
GJAN_by_GRQN(k) =
{
  (-3/10 - 1/10*I)*b^k
  + (1/20 - 3/20*I)*(-1)^k
  + (1/8 - 3/8*I)
  + 1/2*I*((4/5 - 2/5*I)*GRQN(k) + (-3/5 + 4/5*I))
  + 1/2*((4/5 - 2/5*I)*GRQN(k+1) + (-3/5 + 4/5*I))
  + (-1/8 + 1/8*I)*((4/5 - 2/5*I)*GRQN(k+2) + (-3/5 + 4/5*I));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */


\\ LQb_sum_by_GRQN(k) = (4/5 - 2/5*I)*GRQN(k) + (-3/5 + 4/5*I);
GJAN_by_GRQN(k) =
{
  (-3/10 - 1/10*I)*b^k
  + (1/20 - 3/20*I)*(-1)^k
  + (-3/5 - 9/20*I)
  + (1/5 + 2/5*I)*GRQN(k)
  + (2/5 - 1/5*I)*GRQN(k+1)
  + (-1/20 + 3/20*I)*GRQN(k+2);
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  (1/5 + 2/5*I)
    * (
    -3/2 + 3/4*I
   - 1/2*conj(b) * b^k
       + -1/4*b * (-1)^k
       + GRQN(k)
       - I*GRQN(k+1)
       + 1/4*b*GRQN(k+2));
}
check_exact(vector(100,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(100,k,my(k=k+2); GJAN(k)))  /* k>=3 */

\\-----------------------------------------------------------------------------
\\ roll down indexes

check_exact(vector(10,k,my(k=k+3); GRQN(k)), \
            vector(10,k,my(k=k+3); b*GRQN(k-1) + 2*b^3*GRQN(k-3) + 3/2 - 9/2*I))  /* k>=4 */

GJAN_by_GRQN(k) =
{
  (1/5 + 2/5*I)
    * (
    -3/2 + 3/4*I
   - 1/2*conj(b) * b^k
       + -1/4*b * (-1)^k
       + GRQN(k)
       - I*GRQN(k+1)
       + 1/4*b*(b*GRQN(k+1) + 2*b^3*GRQN(k-1) + 3/2 - 9/2*I));
}
check_exact(vector(10,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(10,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  (1/5 + 2/5*I)
    * (
       - 1/2*conj(b) * b^k
       + -1/4*b * (-1)^k
       - 2*GRQN(k-1)
       + GRQN(k)
       - 1/2*I*GRQN(k+1)
       );
}
check_exact(vector(10,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(10,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
  (1/5 + 2/5*I)
    * (
       -9/4 - 3/4*I
       - 1/2*conj(b) * b^k
       + -1/4*b * (-1)^k
       + (3/2 - 1/2*I) * GRQN(k)
       - 2 * GRQN(k-1)
       + 2*b * GRQN(k-2)
       );
}
check_exact(vector(10,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(10,k,my(k=k+2); GJAN(k)))  /* k>=3 */


GJAN_by_GRQN(k) =
{
   (-1/5 + 1/10*I)
    * (
         b^(k+1)
       - 1/2*conj(b) * (-1)^k
       - GRQN(k+1)
       - 2*I* GRQN(k)
       + 4*I* GRQN(k-1)
       );
}
check_exact(vector(10,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(10,k,my(k=k+2); GJAN(k)))  /* k>=3 */

GJAN_by_GRQN(k) =
{
   (2 - I)/10
    * (
       + GRQN(k+1)
       + 2*I* GRQN(k)
       - 4*I* GRQN(k-1)
       - b^(k+1)
       + 1/2*conj(b) * (-1)^k
       );
}
check_exact(vector(10,k,my(k=k+2); GJAN_by_GRQN(k)), \
            vector(10,k,my(k=k+2); GJAN(k)))  /* k>=3 */

