\\ Copyright 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(strictargs,1);
default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\ngot  %s\nwant %s",x,y)));
  return(0);
}

check_equal(got,want,name="") =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong ",name,"\n",d); quit(1),
     print("ok ",check_count," ",name));
}

\\-----------------------------------------------------------------------------

delta(k) = if(k==0,0, norm(PlusPoint(2^(k-1)-1) - PlusPoint(2^(k-1))));
vector(15,k,k--; delta(k))

gdelta(x) =
{
  + -2
  + (1)/(1 - x)
  + (1)/(1 - 2*x)
  - I/(1 - (1 - I)*x)
  + I/(1 - (1 + I)*x)
  ;
}
check_exact(gf_terms(gdelta(x),100), vector(100,k,k--; delta(k)));

delta_formula(k) =
{
  if(k==0,0,
     1
     + 2^k
     + I*(b)^k
     - I*(conj(b))^k
     );
}
check_exact(vector(100,k,k--; delta_formula(k)), vector(100,k,k--; delta(k)));

delta_formula(k) =
{
  if(k==0,0,
     2^k
     + vector_modulo([0,-2,-2,-2, 0,2,2,2],k) * 2^floor(k/2)
     + 1
     );
}
vector(20,k,k--; (delta_formula(k)-delta(k))/2^floor(k/2))

check_exact(vector(100,k,k--; delta_formula(k)), vector(100,k,k--; delta(k)));

distance(k) = sum(n=1,2^k-1,  \
                  abs(PlusPoint(n-1) - PlusPoint(n)));
for(k=0,10, print(distance(k)/(2)^k));

distance_by_recurrence(k) = if(k==0,0, 2*distance(k-1) + sqrt(delta(k-1)));
for(k=0,10, print(distance_by_recurrence(k)/2^k,"  ",distance(k)/2^k));

distance_scaled(k) =
{
  sum(i=1,k,
        1/sqrt(2)^i
        * sqrt(1
               + vector_modulo([0,-2,-2,-2, 0,2,2,2],i) / 2^ceil(i/2)
               + 1/2^i
              ));
}  
my(k=5); print(distance_scaled(k),"     ",distance(k)/2^k);
print();
print(distance_scaled(500));
print(distance_scaled(1000));
print(distance_scaled(5000));
\\print(distance_scaled(10000));

distance_scaled_by_recurrence(k) = distance_scaled(k-1) + sqrt(delta(k))/2^k;
print(distance_scaled(5));
print(distance_scaled_by_recurrence(5));

\\for(k=1,20,print(core(delta(k))))

\\ limit 1.93299391400660938364121
\\ cf manhattan 2.243 or maxreim 1.803

\\ norm((dx+dy*I))
\\ norm((dx+dy*I)*b)
distance(6)*2
distance(7)