#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyDragon ':all';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
### level: "$level  n_hi=$n_hi"

my $xyxy_is_twindragon_touch = sub {
  my ($x,$y) = @_;
  ### xyxy_is_twindragon_touch(): "$x, $y"
  {
    my $n = $path->xy_to_n($x,$y);
    ### first: $n
    unless ($n && $n < $n_hi) {
      ### bad n: $n_hi
      return 0;
    }
  }
  ($x,$y) = ($x_hi-$x, $y_hi-$y);
  ### reverse: "$x, $y"
  {
    my $n = $path->xy_to_n($x,$y);
    ### second: $n
    unless ($n && $n < $n_hi) {
      ### bad n: $n_hi
      return 0;
    }
  }
  return 1;
};

$Printwrap = 2;
printwrap("\\begin{scope}[shift=(pos-$level)]\n");
{
  $Printwrap = 4;
  printwrap("\\node at (\\MyEnd,0) [my dot,name=start1-$level];\n");
  printwrap("\\node at ($x_hi,$y_hi-\\MyEnd) [my dot,name=end1];\n");
  printwrap("\\draw (start1-$level)");
  $Printwrap = 6;
  foreach my $n ($n_lo+1 .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);

    if ($xyxy_is_twindragon_touch->($x,$y)) {
      my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
      my ($tdx,$tdy) = $path->n_to_dxdy($n);
      my $sx = $x . chamfer(-$sdx);
      my $sy = $y . chamfer(-$sdy);
      my $tx = $x . chamfer($tdx);
      my $ty = $y . chamfer($tdy);
      printwrap("--($sx,$sy)");
      printwrap("--($tx,$ty)");
    } else {
      printwrap("--($x,$y)");
    }
  }
  printwrap("--(end1);\n");
}

{
  $Printwrap = 4;
  printwrap("\\node at ($x_hi-\\MyEnd,$y_hi) [my dot,name=start2];\n");
  printwrap("\\node at (0,\\MyEnd) [my dot,name=end2];\n");
  printwrap("\\draw (start2)");
  $Printwrap = 6;
  foreach my $n ($n_lo+1 .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);
    ($x,$y) = ($x_hi-$x, $y_hi-$y);

    if ($xyxy_is_twindragon_touch->($x,$y)) {
      my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
      my ($tdx,$tdy) = $path->n_to_dxdy($n);
      ($sdx,$sdy) = (-$sdx,-$sdy);
      ($tdx,$tdy) = (-$tdx,-$tdy);
      my $sx = $x . chamfer(-$sdx);
      my $sy = $y . chamfer(-$sdy);
      my $tx = $x . chamfer($tdx);
      my $ty = $y . chamfer($tdy);
      printwrap("--($sx,$sy)");
      printwrap("--($tx,$ty)");
    } else {
      printwrap("--($x,$y)");
    }
  }
  printwrap("--(end2);\n");
}
$Printwrap = 2;
printwrap("\\end{scope}\n");

print "  % end generated\n";
exit 0;
