\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");
read("pol-ascend.gp");
read("pol-pfrac");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}


\\-----------------------------------------------------------------------------

gboff(x) = 1/(1-x^8)*Polrev(boff_vector);
check_exact(gf_terms(gboff(x),100), vector(100,k,my(k=k-1); boff(k)))

gBlobEnd(x) = 1/3*(2* 1/(1-b*x) - I* (x*gboff(x) + boff(7)));
check_exact(gf_terms(gBlobEnd(x),100), \
            vector(100,k,my(k=k-1); BlobEnd(k)))

\\-----------------------------------------------------------------------------

Rnear_z_samples = [ -1/2, -1/2-1/2*I, 0, 0, \
                    0,    -1,        -I, 1-I, \
                    2, 3+2*I,       5*I, -5+5*I, \
                   -10, -11-10*I,  -21*I, 21-21*I ];


Rnear_z(k) = if(k<4, return(Rnear_z_samples[k+1])); \
  BlobEnd(k) - 1/2*b^k \
  + vector_modulo([I,  0, -1, -1-I,  \
                  -I,  0,  1,  1+I ],k);
check_exact(vector(#Rnear_z_samples,k,my(k=k-1); Rnear_z(k)), Rnear_z_samples)
for(k=0,20,print("k="k"  "Rnear_z(k)));


Rnear_z(k) = if(k<4, return(Rnear_z_samples[k+1])); \
  1/6*b^k \
  + 1/3*vector_modulo([-2, 1 - 2*I, -I, 1 - I, 2, -1 + 2*I, I, -1 + I],k);
check_exact(vector(#Rnear_z_samples,k,my(k=k-1); Rnear_z(k)), Rnear_z_samples)

gRnear_z(x) = 1/6* 1/(1-b*x) \
  + 1/3* 1/(1-x^8)*Polrev([-2, 1 - 2*I, -I, 1 - I, 2, -1 + 2*I, I, -1 + I]) \
  - x;
check_exact(gf_terms(gRnear_z(x),100), \
            vector(100,k,my(k=k-1); Rnear_z(k)))

\\-----------------------------------------------------------------------------

Rnear_norm(k) = norm(Rnear_z(k));
\\ vector(200,k,my(k=k+8); Rnear_norm(k))

gRnear_norm(x) =
{
  1/4
  + 1/2*x
  + x^5
  + x^6
  + 2*x^7
  + 4*x^8
  + x^9*(
  1/3*(1)/(1 - x)
  +  1/9*(128)/(1 - 2*x)
  +  1/18*(1)/(1 + x)
  +  1/12*(1 - I)/(1 + I*x)
  +  1/12*(1 + I)/(1 - I*x)
  +  1/3*(-8 - 16*x)/(1 - 2*x^2)
  +  1/9*(8 + 16*x)/(1 + 2*x^2)
)
  ;
}
check_exact(gf_terms(gRnear_norm(x),1000), \
            vector(1000,k,my(k=k-1); Rnear_norm(k)))

pol_ascending_print(pol_partial_fractions(gRnear_norm(x)))

{
  check_exact(
               (1/36)/(1 - 2*x)

              + (-1/6 - 1/6*x)/(1 - 2*x^2)
              + (-1/18 + 1/18*x)/(1 + 2*x^2)

              + (1/3)/(1 - x)
              + (-1/18)/(1 + x)
              + (1/6 + 1/6*x)/(1 + x^2)
              , gRnear_norm(x))
}

{
  my(v = gf_terms(+ (-1/6 - 1/6*x)/(1 - 2*x^2)
                  + (-1/18 + 1/18*x)/(1 + 2*x^2)
                  ,32));
  vector(#v,i,my(k=i-1); v[i]/2^floor(k/2))
}
{
  gf_terms(+ (1/3)/(1 - x)
           + (-1/18)/(1 + x)
           + (1/6 + 1/6*x)/(1 + x^2)
           ,16)
}

Rnear_norm(k) =
{
  if(k<4, return(norm(Rnear_z_samples[k+1])));
  1/36*2^k
  - 1/9*vector_modulo([2, 1, 1, 2],k) * 2^floor(k/2)
  + 1/9*vector_modulo([4, 5, 1, 2],k);
}
check_exact(vector(1000,k,my(k=k-1); Rnear_norm(k)), \
            gf_terms(gRnear_norm(x),1000))

\\ square when k even, location all real or all imag,
\\ and some extras for small k
vector(100,k,my(k=k-1); issquare(Rnear_norm(k)))

Rnear_norm(k) =
{
  if(k<4, return(norm(Rnear_z_samples[k+1])));
  1/36* (
         2^k
         - vector_modulo([8, 4, 4, 8],k) * 2^floor(k/2)
         + vector_modulo([16, 20, 4, 8],k)
         );
}
check_exact(vector(1000,k,my(k=k-1); Rnear_norm(k)), \
            gf_terms(gRnear_norm(x),1000))

Rnear_norm(k) =
{
  if(k<4, return(norm(Rnear_z_samples[k+1])));
  1/36* (
         2^k
         - vector_modulo([4, 2, 2, 4],k) * 2^floor(k/2)
         - vector_modulo([4, 1, 2, 2],k) * 2^ceil(k/2)
         + vector_modulo([16, 20, 4, 8],k)
         );
}
check_exact(vector(1000,k,my(k=k-1); Rnear_norm(k)), \
            gf_terms(gRnear_norm(x),1000))

Rnear_norm(k) =
{
  if(k<4, return(norm(Rnear_z_samples[k+1])));
  1/36* (
         (2^floor(k/2) - vector_modulo([4, 1, 2, 2],k))
         * (2^ceil(k/2) - vector_modulo([4, 2, 2, 4],k))
         - vector_modulo([4, 1, 2, 2],k) * vector_modulo([4, 2, 2, 4],k)
         + vector_modulo([16, 20, 4, 8],k)
         );
}
check_exact(vector(1000,k,my(k=k-1); Rnear_norm(k)), \
            gf_terms(gRnear_norm(x),1000))

Rnear_norm(k) =
{
  if(k<4, return(norm(Rnear_z_samples[k+1])));
  (
    ( 1/6*(2^ceil(k/2) - vector_modulo([4,2,2,4],k)) )^2
    + vector_modulo([0,1,0,0],k)
  )
  * vector_modulo([1, 1/2],k);
}
check_exact(vector(1000,k,my(k=k-1); Rnear_norm(k)), \
            gf_terms(gRnear_norm(x),1000))


Rnear_norm(k) =
{
  if(k<4, return(norm(Rnear_z_samples[k+1])));
  1/36* (
         2^k
         - vector_modulo([8, 4/sqrt2, 4, 8/sqrt2],k) * sqrt2^k
         + vector_modulo([16, 20, 4, 8],k)
         );
}
check_exact(vector(1000,k,my(k=k-1); Rnear_norm(k)), \
            gf_terms(gRnear_norm(x),1000))

Rnear_norm(k) =
{
  if(k<4, return(norm(Rnear_z_samples[k+1])));
  1/36* (
         (sqrt2^k - vector_modulo([4, 2/sqrt2, 2, 4/sqrt2],k))^2
         - vector_modulo([4, 2/sqrt2, 2, 4/sqrt2],k)^2
         + vector_modulo([16, 20, 4, 8],k)
         );
}
check_exact(vector(1000,k,my(k=k-1); Rnear_norm(k)), \
            gf_terms(gRnear_norm(x),1000))

Rnear_norm(k) =
{
  if(k<4, return(norm(Rnear_z_samples[k+1])));
  ( 1/6*vector_modulo([1,sqrt2],k) 
    * (2^floor(k/2) - vector_modulo([4, 1, 2, 2],k))
  )^2
  + vector_modulo([0,1/2,0,0],k)
  ;
}
check_exact(vector(1000,k,my(k=k-1); Rnear_norm(k)), \
            gf_terms(gRnear_norm(x),1000))


Rnear_m(k) = 1/6*(sqrt2^k - vector_modulo([4, sqrt2, 2, 2*sqrt2],k));
Rnear_norm(k) =
{
  if(k<4, return(norm(Rnear_z_samples[k+1])));
    Rnear_m(k)^2 + vector_modulo([0,1/2,0,0],k);
}
check_exact(vector(1000,k,my(k=k-1); Rnear_norm(k)), \
            gf_terms(gRnear_norm(x),1000))

