#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use Carp 'croak';
use File::Spec;
use List::Util 'max','min','sum','pairmap';
use Math::BaseCnv 'cnv';
use Math::Complex 'pi';
use Math::Libm 'hypot';
use Math::Trig 'deg2rad', 'rad2deg';
use POSIX 'floor';

use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()
use Math::PlanePath::Base::Digits
  'bit_split_lowtohigh', 'digit_split_lowtohigh',
  'digit_join_lowtohigh', 'round_down_pow';
use Math::PlanePath::Base::Generic
  'round_nearest';

# use Math::PlanePath;
# *_divrem_mutate = \&Math::PlanePath::_divrem_mutate;
# unshift @INC, '/usr/share/perl5';

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyDragon ':all';
our @ipc;
$|=1;

# uncomment this to run the ### lines
# use Smart::Comments;

my @skyline_folds = (0,0,1,1, 0,1,0,0, 1,1,0,1);
sub skyline_turn {
  my ($n) = @_;
  ### skyline_turn(): "n=$n"
  $n >= 1 or die;
  my $pos = 0;
  until ($n % 2) { $pos++; $n >>= 1; }
  my $bit = ($n >> 1) & 1;
  my $fold = $skyline_folds[$pos % scalar(@skyline_folds)];
  ### at: "shifted n=$n bit=$bit pos=$pos fold=$fold"
  return $bit ^ $skyline_folds[$pos % scalar(@skyline_folds)]
    ? -1 : 1;
}

my @skyline_want_turns = (undef, 1,1,-1,-1, 1,-1,-1,-1, 1,1,-1,1);
foreach my $n (1 .. $#skyline_want_turns) {
  skyline_turn($n) == $skyline_want_turns[$n] or die;
}


{
  # Skyline by turn sequence

  foreach my $n (1 .. 4) {
    print skyline_turn($n),", ";
  }
  print "\n";

  my $k = 16;
  my $step = 4;

  require Language::Logo;
  my $lo = Logo->new(update => 2, port => 8200 + (time % 100),
                     width => 900, height => 900);
  my $draw;
  $lo->command("penup");
  $lo->command("backward 400");
  $lo->command("seth 90");
  $lo->command("backward 400");
  $lo->command("pendown");
  $lo->command("hideturtle");
  my $angle = 0;
  foreach my $n (1 .. 2**$k) {
    my $t = skyline_turn($n);
    my $angle = $t*90;
    $lo->command("forward $step; left $angle");
  }
  $lo->disconnect("Finished...");
  exit 0;
}
