\\ Copyright 2019 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
default(strictargs,1);
read("dragon-defines.gp");

check_count = 0;
check_equal(got,want,name="") =
{
  check_count++;
  if(!(got===want),
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}


\\-----------------------------------------------------------------------------
\\ Helpers

to_ternary(n) = fromdigits(digits(n,3),10);

balanced_ternary_digits(n) =
{
  my(l=List([]));
  while(n, my(d=n%3);if(d==2,d=-1); listput(l,d); n=(n-d)/3);
  Vecrev(l);
}

LowNon0(n) = n>0 || error(); while(n%3==0, n\=3); n%3;

\\ "Choral"
LowNon1(n) = while(n%3==1, n\=3); n%3;
A116178(n) = LowNon1(n)/2;
{
  my(v=OEIS_samples("A116178"));
  check_equal(vector(#v,n,n--; A116178(n)), 
              v,    \\ OFFSET=0
              "A116178 choral = LowNon1/2");
}

\\ A136442     a(3n)=1, a(3n-1)=0 and a(3n+1)=a(n)
\\ choral relative
A136442(n) = while(n%3, if(n%3==2,return(0)); n\=3); 1;
{
  my(v=OEIS_samples("A136442"));
  check_equal(vector(#v,n,n--; A136442((n))),  v,
              "A136442");
}
check_equal(vector(3^5,n, A136442(n)), \
            vector(3^5,n, 1-(LowNon1(n)/2)));
check_equal(vector(3^5,n, A136442(n)), \
            vector(3^5,n, 1-A116178(n)));


LowNon2(n) = n>=0 || error(); while(n%3==2, n\=3); n%3;

Count0s(n) = my(ret=0); while(n,ret+=((n%3)==0); n\=3); ret;
Count1s(n) = my(ret=0); while(n, if(n%3==1,ret++); n\=3); ret;
Count2s(n) = my(ret=0); while(n, if(n%3==2,ret++); n\=3); ret;
\\vector(20,n, Count0s(n) % 2)
{
  \\ A160384 num non-zeros
  my(v=OEIS_samples("A160384"));
  check_equal(vector(#v,n,n--; Count1s(n) + Count2s(n)), 
              v,
              "A160384");
}
\\ vector(20,n, TernaryCountLow0s(n) + TernaryCountLow1s(n))
\\ not in OEIS: 1, 0, 1, 2, 0, 1, 1, 0, 2, 1, 0, 1, 3, 0, 1, 1, 0, 2, 1, 0
\\              1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20

CountLow0s(n) = n>0 || error(); my(ret=0); while(n%3==0, ret++;n\=3); ret;
\\ A007949
check_equal(vector(3^5,n, CountLow0s(n)), \
            vector(3^5,n, valuation(n,3)), \
            "CountLow0s vs valuation");
A007949(n) = CountLow0s(n);
{
  my(v=OEIS_samples("A007949"));
  check_equal(vector(#v,n, A007949(n)), 
              v,  \\ OFFSET=1
              "A007949");
}

CountLow1s(n) = my(ret=0); while(n%3==1, ret++;n\=3); ret;
{
  my(v=OEIS_samples("A253786"));
  check_equal(vector(#v,n,n--; CountLow1s(n-1)), 
              v,  \\ OFFSET=0
              "A253786 samples vs CountLow1s");
}
check_equal(CountLow1s(-1), 0);

\\ A253786  a(n+1) = count low ternary 1-digits
\\ a(0) counts lows of -1
\\ a(1) counts lows of 0
\\ a(2) counts lows of 1
A253786(n) = n>=0 || error(); n--; my(ret=0); while(n%3==1, ret++;n\=3); ret;
{
  my(v=OEIS_samples("A253786"));
  check_equal(vector(#v,n,n--; A253786(n)), 
              v,  \\ OFFSET=0
              "A253786 vs samples");
}
check_equal(vector(3^5,n, A253786(n)), \
            vector(3^5,n, A007949(2*n-1)), \
            "A253786 vs A007949, for n>=1");  /* Cyril Damamme */

CountLow2s(n) = my(ret=0); while(n%3==2, ret++;n\=3); ret;


\\-----------------------------------------------------------------------------
\\ binomial(2*n,n) valuation

\\ A000989  3-adic valuation of binomial(2n,n)
\\ is extras val (2n)! - 2* val n!
check_equal(vector(100,n,n--; valuation((2*n)!,3)), \
            vector(100,n,n--; 2*valuation(n!,3) \
                            + valuation(binomial(2*n,n),3)))
A000989(n) = valuation(binomial(2*n,n),3);
{
  my(v=OEIS_samples("A000989"));
  check_equal(vector(#v,n,n--; A000989(n)), 
              v,  \\ OFFSET=0
              "A000989 vs samples");
}
check_equal(A000989(0), 0);

{
  check_equal(vector(100,n,n++; A000989(n) - A000989(n-1)),
              vector(100,n,n++; CountLow1s(n-1) - CountLow0s(n)),
              "binomial(2*n,n) increment");
}
{
  check_equal(vector(100,n, A000989(n) - A000989(n-1)),
              vector(100,n, A253786(n) - A007949(n)),
              "binomial(2*n,n) increment");
}

\\ vector(19,n,n++; CountLow1s(n-1) - CountLow0s(n))
\\ not in OEIS: 1, -1, 0, 2, -1, 0, 1, -2, 0, 1, -1, 0, 3, -1, 0, 1, -2, 0, 1
\\ ploth(x=1,1000,my(n=floor(x)); CountLow1s(n-1) - CountLow0s(n))
\\ ploth(x=1,1000,my(n=floor(x)); A000989(n))


\\-----------------------------------------------------------------------------
\\ Ternary 1s Above

\\ how many ternary digits have an odd number of 1s above it,
\\ not including itself
num_odd_1s(n) =
{
  my(v=digits(n,3), odd=0, ret=0);
  for(i=1,#v, if(odd,ret++); if(v[i]==1,odd=!odd));
  ret;
}
\\ vector(27,n, num_odd_1s(n))
\\ not in OEIS: 0,0,1,1,1,0,0,0,2,2,2,1,1,1,2,2,2,0,0,0,1,1,1,0,0,0,3

\\ how many ternary digits have an even number of 1s above it,
\\ not including itself
num_even_1s(n) =
{
  my(v=digits(n,3), even=1, ret=0);
  for(i=1,#v, if(even,ret++); if(v[i]==1,even=!even));
  ret;
}
\\ vector(27,n, num_even_1s(n))
\\ not in OEIS: 1,1,1,1,1,2,2,2,1,1,1,2,2,2,1,1,1,3,3,3,2,2,2,3,3,3,1


\\ how many ternary digits have an odd number of 1s above it,
\\ including itself
num_odd_1s_self(n) =
{
  my(v=digits(n,3), odd=0, ret=0);
  for(i=1,#v, if(v[i]==1,odd=!odd); if(odd,ret++));
  ret;
}
 vector(27,n, num_odd_1s_self(n))
\\ not in OEIS: 1,0,2,1,2,0,1,0,3,2,3,1,2,1,3,2,3,0,1,0,2,1,2,0,1,0,4

\\ how many ternary digits have an even number of 1s above it,
\\ including itself
num_even_1s_self(n) =
{
  my(v=digits(n,3), even=1, ret=0);
  for(i=1,#v, if(v[i]==1,even=!even); if(even,ret++));
  ret;
}
 vector(27,n, num_even_1s_self(n))
\\ not in OEIS: 0,1,0,1,0,2,1,2,0,1,0,2,1,2,0,1,0,3,2,3,1,2,1,3,2,3,0


\\-----------------------------------------------------------------------------
\\ n! sans 3s parity

\\ no good:
\\ +1 is low 1 -> 2           different
\\ +1 is low 10000 -> 10001  same
\\ +1 is low 20000 -> 20001  different
\\ +1 changes parity of n
\\ factors of 3 don't change parity mod 2
\\ vector(50,n, [n/3^valuation(n,3), (n+1)/3^valuation(n+1,3)])

\\ vector(10,n, n=3^n; n=n!; n/=3^valuation(n,3); n%3)
\\ vector(10,n, n=2*3^n; n=n!; n/=3^valuation(n,3); n%3)

CountOdd1s(n) = my(ret=0); while(n,n\=3; ret+=((n%3)==1); n\=3); ret;
vector(20,n, CountOdd1s(n) % 2)

LowNon0pn(n) = 3-2*LowNon0(n);
vector(20,n, LowNon0(n))
vector(20,n, LowNon0pn(n))

print();

\\ A136690 lowest non-0 of n!
A136690(n) = n=n!; n/=3^valuation(n,3); (n%3);

mod2_1or2(n) = n%=2; if(n==0,2,1);
check_equal(vector(3^5,n,n--; A136690(n)), \
            vector(3^5,n,n--; prod(i=1,n,LowNon0(i)) % 3));

\\ 0
\\ 1    1                  1
\\ 2   -1                 -1
\\ 3          1           -1
\\ 4    1                 -1
\\ 5    -1                 1
\\ 6         -1           -1
\\ 7    1                 -1
\\ 8    -1                 1
\\ 9          1            1
\\ 1xxx all of xxx

A136690_by_digits(n) = my(ret=0); while(n,ret+=(2<=(n%6)&&(n%6)<5); n\=3); (ret%2)+1;
check_equal(vector(3^5,n,n--; A136690_by_digits(n)), \
            vector(3^5,n,n--; A136690(n)));

vector(20,n, prod(i=1,n, LowNon0pn(i)))
\\ t(n) = my(ret=0); while(n, sum(k=1,n, floor((n/3^k) % 3 != 1)) % 2;
vector(20,n, A136690(n))
vector(20,n, A136690(n)-1)
vector(20,n, 2-A136690(n))

A136690_pn1(n) = 3-2*A136690_by_digits(n);
vector(20,n, A136690_pn1(n))

vector(20,n, A136690(n))
vector(20,n, A136690(3*n))
vector(20,n, A136690(3*n+1))
vector(20,n, A136690(3*n+2))
vector(20,n, A136690(2*n+1))

vector(20,n,n=2*n-1; 1-((A136690(3*n) - A136690(n))+1)/2)

\\ flip(d,x) = d if x even, or 3-d if x odd.
flip(d,x) = if(x%2, 3-d, d);

\\ A136690 recurrence 3n,3n+1,3n+2
\\ 0 or 1 flip as n%2       how many 1s above
\\ 2      flip as !(n%2)
check_equal(vector(3^5,n, A136690(3*n)), \
            vector(3^5,n, flip(A136690(n), n)), \
            "A136690 3n   flip when n odd");
check_equal(vector(3^5,n, A136690(3*n+1)), \
            vector(3^5,n, flip(A136690(n), n)), \
            "A136690 3n+1   flip when n odd");
check_equal(vector(3^5,n, A136690(3*n+1)), \
            vector(3^5,n, A136690(3*n)), \
            "A136690 3n+1  = 3n");
vector(20,n,n=n; A136690(3*n+2) != A136690(n))
check_equal(vector(3^5,n, A136690(3*n+2)), \
            vector(3^5,n, flip(A136690(n), n+1)), \
            "A136690 3n+2   flip when n even");

check_equal(vector(3^5,n, A136690(3*n)), \
            vector(3^5,n, bitxor(A136690(n), 3*(n%2))), \
            "A136690 3n");
check_equal(vector(3^5,n, A136690(3*n+1)), \
            vector(3^5,n, bitxor(A136690(n), 3*(n%2))), \
            "A136690 3n+1");
vector(20,n, A136690(3*n+2) != A136690(n))
check_equal(vector(3^5,n, A136690(3*n+2)), \
            vector(3^5,n, bitxor(A136690(n), 3*!(n%2))), \
            "A136690 3n+2");

check_equal(vector(3^5,n, LowNon0((3*n)!/n!)), \
            vector(3^5,n, (n%2)+1), \
            "LowNon0 of (3n)!/n!");

A136690_by_digits(n) =
{
  my(total=Mod(0,2), r);
  while(n,
        [n,r] = divrem(n,3);
        total += n + (r==2));
  lift(total) + 1;
}
check_equal(vector(3^5,n,n--; A136690_by_digits(n)), \
            vector(3^5,n,n--; A136690(n)));

\\ Count1s is when n odd
A136690_by_digits(n) =
{
  my(ret=0);
  while(n,
        my(d=n%3); n\=3;
        ret += Count1s(n) + (d==2));
  (ret%2) + 1;
}
check_equal(vector(3^5,n,n--; A136690_by_digits(n)), \
            vector(3^5,n,n--; A136690(n)));

\\ odd when odd number of 1s above
A136690_by_digits(n) =
{
  my(v=digits(n,3), odd=0, total=Mod(0,2));
  for(i=1,#v, total+=odd+(v[i]==2); if(v[i]==1, odd=!odd));
  lift(total) + 1;
}
check_equal(vector(3^5,n,n--; A136690_by_digits(n)), \
            vector(3^5,n,n--; A136690(n)));

check_equal(vector(3^5,n,n--; A136690(n)), \
            vector(3^5,n,n--; (num_odd_1s(n) + Count2s(n))%2 + 1));

A136690_by_digits(n) =
{
  my(ret=0);
  while(n,
        ret += ((n\3)%2) + ((n%3)==2);
        n\=3);
  (ret%2) + 1;
}
check_equal(vector(3^5,n,n--; A136690_by_digits(n)), \
            vector(3^5,n,n--; A136690(n)));

\\ A136690(2*n)
\\ not in OEIS: 2,2,2,1,1,2,1,2,2,1,1,1,2,2,1,2,1,2,1,1,1,2,1,2,1,2,2
\\ not in OEIS: 1,1,1,0,0,1,0,1,1,0,0,0,1,1,0,1,0,1,0,0,0,1,0,1,0,1,1
\\ not in OEIS: 0,0,0,1,1,0,1,0,0,1,1,1,0,0,1,0,1,0,1,1,1,0,1,0,1,0,0
\\ for(n=1,3^3, print1(A136690(2*n), ","));
check_equal(vector(3^5,n,n--; A136690(2*n+1)), \
            vector(3^5,n,n--; prod(i=1,2*n+1,LowNon0(i)) % 3), \
            "A136690 2n+1 prod");

\\ A136690(2*n+1)
\\ not in OEIS: 2,1,2,1,2,2,2,1,2,1,2,1,2,1,1,1,2,2,1,2,1,1,2,2,2,1,2
\\ not in OEIS: 1,0,1,0,1,1,1,0,1,0,1,0,1,0,0,0,1,1,0,1,0,0,1,1,1,0,1
\\ not in OEIS: 0,1,0,1,0,0,0,1,0,1,0,1,0,1,1,1,0,0,1,0,1,1,0,0,0,1,0
\\ vector(3^3,n, 2-A136690(2*n+1))
check_equal(vector(3^5,n,n--; A136690(2*n+1)), \
            vector(3^5,n,n--; prod(i=1,2*n+1,LowNon0(i)) % 3), \
            "A136690 2n+1 prod");

\\ LowNon0 of 2*n = flip LowNon0 of n
\\ 1000 <-> 2000
check_equal(vector(3^5,n, LowNon0(2*n)), \
            vector(3^5,n, 3-LowNon0(n)));

\\ x111 -> x222+1 = (x+1)000
\\ for(n=1,27, print(to_ternary(n),"  ",to_ternary(2*n+1)));
check_equal(vector(3^3,n, LowNon0(2*n+1)), \
            vector(3^3,n, if(LowNon1(n)==0,1,2)));

\\ just the odds
\\for(n=1,3^3, print1(prod(i=1,n, my(n=2*i+1); LowNon0(n)) % 3, ","))
\\ not in OEIS: 1,2,2,2,1,1,2,1,1,1,2,2,2,1,1,2,1,1,1,2,2,1,2,2,1,2,2,
\\for(n=1,3^3, print1((prod(i=1,n, my(n=2*i+1); LowNon0(n)) % 3)==1, ","))
\\ not in OEIS: 1,0,0,0,1,1,0,1,1,1,0,0,0,1,1,0,1,1,1,0,0,1,0,0,1,0,0,
for(n=1,3^3, print1((prod(i=1,n, my(n=2*i+1); LowNon0(n)) % 3)==2, ","))
\\ not in OEIS: 0,1,1,1,0,0,1,0,0,0,1,1,1,0,0,1,0,0,0,1,1,0,1,1,0,1,1,

\\for(n=1,3^3, print1(A136690(n) != A136690(2*n+1), ","));
\\ not in OEIS: 1,1,0,1,1,0,0,0,1,0,0,1,0,0,1,1,1,0,1,1,0,0,0,1,1,1,0

\\ for(n=1,3^3, print1(A136690(n) == A136690(2*n+1), ","));
\\ not in OEIS: 0,0,1,0,0,1,1,1,0,1,1,0,1,1,0,0,0,1,0,0,1,1,1,0,0,0,1

\\ for(n=1,3^3, print1(A136690(3*n) == A136690(2*n+1), ","));
\\ not in OEIS: 1,0,0,0,1,1,0,1,1,1,0,0,0,1,1,0,1,1,1,0,0,1,0,0,1,0,0
\\ for(n=1,3^3, print1(A136690(3*n) != A136690(2*n+1), ","));
\\ not in OEIS: 0,1,1,1,0,0,1,0,0,0,1,1,1,0,0,1,0,0,0,1,1,0,1,1,0,1,1

\\ for(n=1,3^3, print1(A136690(3*n) == A136690(2*n), ","));
\\ not in OEIS: 1,1,0,0,0,1,1,0,1,1,1,0,0,0,1,1,0,1,1,1,0,0,1,0,0,1,0,
\\ for(n=1,3^3, print1(A136690(3*n) != A136690(2*n), ","));
\\ not in OEIS: 0,0,1,1,1,0,0,1,0,0,0,1,1,1,0,0,1,0,0,0,1,1,0,1,1,0,1,


\\-----------------------------------------------------------------------------
\\ A136690(2*n) = LowNon0 of (2n)! by Digits of n

\\ 3n   -> n same
\\ 3n+1 -> n flip
\\ 3n+2 -> n flip on LowNon1(n)==0
check_equal(vector(3^3,n, A136690(2*(3*n)) == A136690(2*n)), \
            vector(3^3,n, 1), \
            "A136690 2n of 3n from n");
check_equal(vector(3^3,n, A136690(2*(3*n+1)) == A136690(2*n)), \
            vector(3^3,n, 0), \
            "A136690 2n of 3n+1 from n");
check_equal(vector(3^3,n, A136690(2*(3*n+2)) == A136690(2*n)), \
            vector(3^3,n, LowNon1(n)==2), \
            "A136690 2n of 3n+2 from n");

A136690_of_2n(n) = A136690_by_digits(2*n);
check_equal(vector(3^5,n, A136690_of_2n(3*n) == A136690_of_2n(n)), \
            vector(3^5,n, 1), \
            "A136690 2n of 3n+1 vs n");
check_equal(vector(3^5,n, A136690_of_2n(3*n+1) == A136690_of_2n(n)), \
            vector(3^5,n, 0), \
            "A136690 2n of 3n+1 vs n");
check_equal(vector(3^5,n, A136690_of_2n(3*n+2) == A136690_of_2n(n)), \
            vector(3^5,n, LowNon1(n)==2), \
            "A136690 2n of 3n+2 vs n");

check_equal(A136690_of_2n(0), 1);
check_equal(A136690_of_2n(1), 2);
check_equal(A136690_of_2n(2), 2);
check_equal(LowNon1(0), 0);

\\ all 1s
\\ runs 011112
A136690_of_2n_by_digits(n) =
{
  my(non1=0, ret=0);
  while(n,
        if(n%3==2, ret+=(non1==0));
        if(n%3==1, ret++, non1=n%3);
        n\=3);
  (ret%2)+1;
}
check_equal(vector(3^5,n,n--; A136690_of_2n_by_digits(n)), \
            vector(3^5,n,n--; A136690_of_2n(n)), \
            "A136690_of_2n_by_digits()");

\\ A080846 low non-2
\\ A137893  
A137893(n) = LowNon0(n)==1;
{
  my(v=OEIS_samples("A137893"));
  check_equal(vector(#v,n, A137893(n)), 
              v,  \\ OFFSET=1
              "A137893 vs samples");
}

check_equal(vector(3^6,n, A136690_of_2n(3*n-1) == A136690_of_2n(n)), \
            vector(3^6,n, LowNon0(n)==1), \
            "A136690 2n of 3n-1 from n");
check_equal(vector(3^6,n, A136690_of_2n(3*n-1) == A136690_of_2n(n)), \
            vector(3^6,n, A137893(n)), \
            "A136690 2n of 3n-1 from n");
\\ balanced ternary
\\ all 1s
\\ -1 000 -1

A136690_of_2n_by_digits(n) =
{
  my(v=balanced_ternary_digits(n),
     ret=0);
  for(i=1,#v,
     if(v[i]==1, ret++);
     if(v[i]==-1,
        for(j=i+1,#v,
           if(v[j]!=0,
              ret += (v[j]==-1);
              break))));
  (ret%2)+1;
}
check_equal(vector(3^6,n,n--; A136690_of_2n_by_digits(n)), \
            vector(3^6,n,n--; A136690_of_2n(n)), \
            "A136690_of_2n_by_digits()");

\\-----------------------------------------------------------------------------
\\ A136690(2*n+1) = LowNon0 of (2n+1)! by Digits of n

check_equal(vector(3^5,n, A136690(2*(3*n+1)+1) == A136690(2*(3*n)+1)), \
            vector(3^5,n, LowNon1(n)==2), \
            "A136690 2n+1 of 3n or 3n+1");
check_equal(vector(3^5,n, A136690(2*(3*n+2)+1) != A136690(2*(3*n)+1)), \
            vector(3^5,n, LowNon1(n)==2), \
            "A136690 2n+1 of 3n or 3n+2");
check_equal(vector(3^5,n, A136690(2*(3*n+2)+1)), \
            vector(3^5,n, 3-A136690(2*(3*n+1)+1)), \
            "A136690 2n+1 of 3n+1 opp= 3n+2");

check_equal(vector(100,n, n=ternary_1s(n); A136690_by_digits(n)), \
            vector(100,n, ((n\2)%2)+1), \
            "A136690 of 11111");

A136690_of_2nplus1(n) = A136690_by_digits(2*n+1);
check_equal(vector(3^5,n, A136690_of_2nplus1(3*n) == A136690_of_2nplus1(n)), \
            vector(3^5,n, 1 - LowNon1(n)/2), \
            "A136690 2n+1 of 3n+1 vs n");
check_equal(vector(3^5,n, A136690_of_2nplus1(3*n+1) == A136690_of_2nplus1(n)), \
            vector(3^5,n, 0), \
            "A136690 2n+1 of 3n+1 vs n");
check_equal(vector(3^5,n, A136690_of_2nplus1(3*n+2) == A136690_of_2nplus1(n)), \
            vector(3^5,n, 1), \
            "A136690 2n+1 of 3n+2 vs n");

check_equal(vector(3^5,n, A136690_of_2nplus1(3*n) == A136690_of_2nplus1(n)), \
            vector(3^5,n, LowNon1(n)==0), \
            "A136690 2n+1 of 3n+1 vs n");
\\
\\ 3n   -> n xor LowNon1(n)==0
\\ 3n+1 -> n flip
\\ 3n+2 -> n same

\\ all 1s
\\ runs 211110   0 with 211..11 above it
A136690_of_2nplus1_by_digits(n) =
{
  my(ret=0);
  while(n,
        if(n%3==0, ret+=(LowNon1(n\3)==2),
           n%3==1, ret++);
        n\=3);
  (ret%2)+1;
}
check_equal(vector(3^5,n,n--; A136690_of_2nplus1_by_digits(n)), \
            vector(3^5,n,n--; A136690_of_2nplus1(n)));

A136690_of_2nplus1_by_digits(n) =
{
  my(non1=-1, ret=0);
  while(n,
        if(n%3==2, ret+=(non1==0));
        if(n%3==1, ret++, non1=n%3);
        n\=3);
  (ret%2)+1;
}
check_equal(vector(3^5,n,n--; A136690_of_2nplus1_by_digits(n)), \
            vector(3^5,n,n--; A136690_of_2nplus1(n)));

count_runs_211110(n) =
{
  my(non1=-1, ret=0);
  while(n,
        if(n%3==2, ret+=(non1==0));
        if(n%3!=1, non1=n%3);
        n\=3);
  ret;
}
for(n=6,80, print1(count_runs_211110(n), ","));
\\ not in OEIS: 1,0,0,0,0,0,0,0,0,1,0,0,1,1,1,1,0,0,1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0,1,1,1,1,0,0,1,0,0,1,1,1,1,1,1,2,1,1,1,1,1,1,0,0,1,0,0,1,1,1,1,0,0,1,0,0

check_equal(vector(3^5,n,n--; (count_runs_211110(n) + Count1s(n))%2+1), \
            vector(3^5,n,n--; A136690_of_2nplus1(n)));

\\ balanced ternary

\\ what is this one?
\\ check_equal(vector(3^3,n, A136690_of_2nplus1(3*n-1) != A136690_of_2nplus1(n)), \
\\             vector(3^3,n, LowNon1(n)==1), \
\\             "A136690 2n+1 of 3n-1 vs n");

\\ not right:
\\ A136690_of_2nplus1_by_digits(n) =
\\ {
\\   my(v=balanced_ternary_digits(n),
\\      ret=0);
\\   for(i=1,#v,
\\      if(v[i]==1, ret++);
\\      if(v[i]==-1,
\\         for(j=i+1,#v,
\\            if(v[j]!=0,
\\               ret += (v[j]==-1);
\\               break))));
\\   (ret%2)+1;
\\ }
\\ check_equal(vector(3^6,n,n--; A136690_of_2nplus1_by_digits(n)), \
\\             vector(3^6,n,n--; A136690_of_2nplus1(n)), \
\\             "A136690_of_2nplus1_by_digits()");


\\-----------------------------------------------------------------------------
\\ valuation (n)!

{
  my(v=OEIS_samples("A054861"));
  check_equal(vector(#v,n,n--; valuation((n)!,3)),
              v,
              "A054861");
}
check_equal(vector(100,n,n--; valuation(n!,3)), \
            vector(100,n,n--; sum(i=1,n, floor(n/3^i))));
check_equal(vector(100,n,n--; valuation(n!,3)), \
            vector(100,n,n--; sum(i=1,n, valuation(i,3))));
check_equal(vector(100,n,n--; valuation(n!,3)), \
            vector(100,n,n--; sum(i=1,n, TernaryCountLow0s(i))));

\\-----------------------------------------------------------------------------
\\ valuation (2*n)!

check_equal(vector(100,n,n--; valuation((2*n)!,3)), \
            vector(100,n,n--;  sum(i=1,n, floor(2*n/3^i))));
check_equal(vector(100,n,     valuation(2*n,3)), \
            vector(100,n,      TernaryCountLow0s(n)));
check_equal(vector(100,n,n--; valuation((2*n)!,3)), \
            vector(100,n,n--; sum(i=1,n, \
                             TernaryCountLow1s(i-1) + TernaryCountLow0s(i))));

\\ vector(30,n,n++; (n-1) - valuation((2*n)!,3))
\\ not in OEIS: 0,0,1,0,0,1,1,0,1,1,1,2,0,0,1,1,0,1,1,1,2,1,1,2,2,0,1,1,1,2

\\ A073216 (two powers of 3) / 2, and can be same so 2*3^k / 2
{
  \\ where valuation (2*n)! == n-1
  my(v=OEIS_samples("A073216"));
  check_equal(select_first_n(n-> valuation((2*n)!,3) == n-1, #v),  v,
             "A073216");
}

\\ A055235 two powers of 3
\\ A038464 two distinct powers of 3    cf A038444 decimal
two_pow3s_vector(len,distinct=0) = {
  my(ret=vector(len),upto=0);
  for(i=0,len,
    for(j=0,i-distinct,
      if(upto>=len,break(2));
      ret[upto++] = 3^i + 3^j));
  ret;
}
{
  my(v=OEIS_samples("A055235"));
  check_equal(two_pow3s_vector(#v),  v,
              "A055235");
}
{
  my(v=OEIS_samples("A038464"));
  check_equal(two_pow3s_vector(#v,1), v, "A038464 distinct");
}
{
  my(v=OEIS_samples("A073216"));
  check_equal(two_pow3s_vector(#v)/2, v, "A073216");
}

\\ 
\\ vector(15,n,n++; valuation((2*n)!,3))
\\ not in OEIS: 1, 2, 2, 4, 5, 5, 6, 8, 8, 9, 10, 10, 13, 14, 14
\\        n-1 = 1, 2, 3, 4, 5, 6, 7, 8, 9,10, 11, 12, 13, 14, 15
\\              ^  ^  ^  ^  ^        ^                 ^   ^
check_equal(vector(1000,n, valuation((2*n)!,3) > n-1), \
            vector(1000,n, 0));
\\ vector(30,n,n++; valuation((2*n)!,3) == n-1)
\\ vector(30,n,n++; valuation((2*n)!,3) != n-1)
\\ not in OEIS: 1,1,0,1,1,0,0,1,0,0,0,0,1,1,0,0,1,0,0,0,0,0,0,0,0,1,0,0,0,0
\\ not in OEIS: 0,0,1,0,0,1,1,0,1,1,1,1,0,0,1,1,0,1,1,1,1,1,1,1,1,0,1,1,1,1
\\ select_first_n(n-> valuation((2*n)!,3) != n-1, 15)
\\ not in OEIS: 4, 7, 8, 10, 11, 12, 13, 16, 17, 19, 20, 21, 22, 23


\\-----------------------------------------------------------------------------
\\ valuation (2*n+1)!

\\ vector(15,n, valuation((2*n+1)!,3))
\\ not in OEIS: 1, 1, 2, 4, 4, 5, 6, 6, 8, 9, 9, 10, 13, 13, 14
\\          n = 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11, 12, 13, 14, 15
\\                       ^                           ^^
\\ ploth(x=1,1000,my(n=floor(x)); n - sum(i=1,2*n+1, TernaryCountLow0s(i)))
\\ vector(20,n, n - valuation((2*n+1)!,3))
\\ not in OEIS: 0, 1, 1, 0, 1, 1, 1, 2, 1, 1, 2, 2, 0, 1, 1, 1, 2, 1, 1, 2

check_equal(vector(100,n,n--; valuation(2*n+1,3)), \
            vector(100,n,n--;  TernaryCountLow1s(n)));
check_equal(vector(100,n,     valuation(2*n-1,3)), \
            vector(100,n,      TernaryCountLow1s(n-1)));

\\ new low 0,  depends on n
check_equal(vector(50,n, n=3*n; n - valuation((2*n+1)!,3)), \
            vector(50,n, n - valuation((2*n+1)!,3) \
                   + TernaryCountLow0s(2*n+1)));
check_equal(2*(3*'n)+1,  3*(2*'n+1) - 2,  "new low 0");
\\ 
\\ new low 1,  unchanged   2*(3*n+1)+1  == 3*(2*n+1)
check_equal(vector(50,n, n=3*n+1; n - valuation((2*n+1)!,3)), \
            vector(50,n, n - valuation((2*n+1)!,3)));
check_equal(2*(3*'n+1)+1,  3*(2*'n+1),  "new low 1");
\\ 
\\ new low 2,  +1
check_equal(vector(50,n, n=3*n+2; n - valuation((2*n+1)!,3)), \
            vector(50,n, n - valuation((2*n+1)!,3)  + 1 ));
check_equal(2*(3*'n+2)+1,  3*(2*'n+1) + 2,  "new low 2");


\\-----------------------------------------------------------------------------
\\ valuation (3*n)!

{
  my(v=OEIS_samples("A004128"));
  check_equal(vector(#v,n,n--; valuation((3*n)!,3)),  v,
              "A004128");
}
check_equal(vector(100,n,n--; valuation((3*n)!,3)), \
            vector(100,n,n--; sum(i=1,n, floor(3*n/3^i))));
check_equal(vector(100,n,n--; valuation((3*n)!,3)), \
            vector(100,n,n--; sum(i=1,n, 1+floor(n/3^i))));
check_equal(vector(100,n,n--; valuation((3*n)!,3)), \
            vector(100,n,n--; n + sum(i=1,n, TernaryCountLow0s(i))));
check_equal(vector(100,n,n--; valuation((3*n)!,3)), \
            vector(100,n,n--; n + valuation(n!,3)));  /* extra n many */


\\-----------------------------------------------------------------------------
\\ binomial(3*n,n)/(2*n+1)

\\ A001764 = binomial(3*n,n)/(2*n+1) various combinatorial
\\   num complete ternary trees of n internal, 3n external
\\   num rooted ordered trees 2n edges, 2n-1 vertices,
\\     every vertex out-degree 2 so "even" trees
\\   num Kreweras intervals of size n
\\   num paths steps 2n across n up staying below y=1/2x
\\   num non-crossing partitions of 2n with all blocks even size
\\     a(2)=3 is 12-34, 14-23, 1234

{
  my(v=OEIS_samples("A001764"));
  check_equal(vector(#v,n,n--; binomial(3*n,n)/(2*n+1)), 
              v,
              "A001764");
}

check_equal(vector(100,n,n--; binomial(3*n,n)/(2*n+1)), \
            vector(100,n,n--; (3*n)! / (2*n+1)! / n! ), \
            "A001764 factorials");
check_equal(3*'n-1, 2*'n+1-1 + 'n-1);

\\ 2 to 3n
\\ 2 to 2n+1
\\ 2 to n
check_equal(vector(10,n,n--; (2*n+1)%3), \
            [1, 0, 2, 1, 0, 2, 1, 0, 2, 1]);

\\ (3n)!/(2n!)/(2n+1)
\\ (3n)*(3n-1)*(3n-2)/(2n)/(2n-1)/n
\\   0     2      1    -n   -n-1  n
{
  check_equal(vector(100,n, n++; binomial(3*n,n)/(2*n+1)), 
              vector(100,n, binomial(3*n,n)/(2*n+1) 
                     * (3*n+2) * (3*n+1) * (3*n) / (2*n) / (2*n+3) / (n+1) ),
              "binomial factor");
};
\\ not an integer
\\ vector(10,n, (3*n+2) * (3*n+1) * (3*n) / (2*n) / (2*n+3) / (n+1) )


\\-----------------------------------------------------------------------------
\\ binomial(3*n,n)/(2*n+1) valuation

val_bin(n) = valuation(binomial(3*n,n)/(2*n+1),3);
vector(20,n,n++; val_bin(n))
\\ not in OEIS: 1, 1, 0, 1, 1, 1, 2, 1, 1, 2, 2, 0, 1, 1, 1, 2, 1, 1, 2, 2

check_equal(vector(10,n,n--; valuation(binomial(3*n,n)/(2*n+1), 3)), \
            [0,0,1, 1,0,1, 1,1,2, 1]);

check_equal(vector(50,n, valuation(binomial(3*n,n),3) \
                   - valuation(binomial(3*n,n)/(2*n+1),3)), \
            vector(50,n, TernaryCountLow1s(n)));

vector(30,n,n++; (binomial(3*n,n)/(2*n+1) / 3^val_bin(n)) % 3)
\\ not in OEIS: 1,1,1,1,2,1,1,1,1,1,1,1,1,2,1,1,2,2,2,2,1,1,2,1,1,1,1,1,1,1
\\ not A026492 Thue-Morse, middle match only

\\ vector(30,n,n+=5; ((binomial(3*n,n)/(2*n+1) / 3^val_bin(n)) % 3) - 1)
\\ not in OEIS: 1,0,0,0,0,0,0,0,0,1,0,0,1,1,1,1,0,0,1,0,0,0,0,0,0,0,0,1,0,0

\\ select_first_n(n-> ((binomial(3*n,n)/(2*n+1) / 3^val_bin(n)) % 3) == 2, 15)
\\ select_first_n(n-> ((binomial(3*n,n)/(2*n+1) / 3^val_bin(n)) % 3) == 1, 20)
\\ not in OEIS: 6, 15, 18, 19, 20, 21, 24, 33, 42, 45, 46, 47, 48, 51, 54
\\ not in OEIS: 1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 22, 23, 25, 26

\\ apply(to_ternary, select_first_n(n-> ((binomial(3*n,n)/(2*n+1) / 3^val_bin(n)) % 3) == 2, 15))
\\ not in OEIS: 20, 120, 200, 201, 202, 210, 220, 1020, 1120, 1200, 1201, 1202, 1210, 1220, 2000

\\ WRONG:
\\ val_bin_rem_by_digits(n) = while(n, if(n%9==4,return(1)); n\=3); 0;
\\ check_equal(vector(30,n,n+=5; val_bin_rem_by_digits(n)), \
\\             select_first_n(val_bin_rem_by_digits, 15), \
\\             "val_bin_rem_by_digits()");

\\ Right:
val_bin_by_digits(n) =
{
  my(p=2,ret=0);
  while(n,
        if(n%3==1, ret+=(p==0),
           ret+=((p=n%3)==2));
        n\=3);
  ret;
}
check_equal(vector(1000,n,n--; val_bin_by_digits(n)), \
            vector(1000,n,n--; val_bin(n)), \
            "val_bin_by_digits()");

\\ Return the number of ternary 1s in n which are in a run 11110,
\\ ie. have a 0-digit below their run of 1s.
\\ The least significant digit does not have a 0 below so never counts.
Count1sAbove0(n) = {
  my(prev=2, ret=0);
  while(n,
    \\ print("n "n%3"  prev "prev);
    if(n%3==0, prev=0,
       n%3==1, if(prev==0,ret++),
       n%3==2, prev=2);
    n\=3);
  ret;
}
\\ vector(20,n,n+=2; Count1sAbove0(n))
\\ not in OEIS: 1, 0, 0, 0, 0, 0, 1, 1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0

check_equal(vector(1000,n,n--; Count2s(n) + Count1sAbove0(n)), \
            vector(1000,n,n--; val_bin(n)));
\\ vector(20,n,n++; Count2s(n))
\\ vector(20,n,n++; Count2s(5*n+3))
\\ vector(20,n,n++; Count1sAbove0(n))

check_equal(Count1sAbove0(from_ternary(11110)),  4);
check_equal(Count1sAbove0(from_ternary(1111)),  0);
check_equal(to_ternary(2*(3*from_ternary(1111)+2)),  100001);
check_equal(to_ternary(2*(3*from_ternary(11110)+2)),  222211);
check_equal(to_ternary(2*(3*from_ternary(2)+2)),  121);
check_equal(to_ternary(2*from_ternary(2222)),  12221);
check_equal(to_ternary(2*from_ternary(11112)),  100001);
check_equal(to_ternary(2*from_ternary(11110)),  22220);

\\ vector(15,n, sum(i=1,2*n+1, TernaryCountLow1s(i)))
\\ not in OEIS: 1, 3, 4, 4, 5, 8, 8, 9, 10, 10, 12, 13, 13, 14, 16


\\ new low 0, depends on n
check_equal(vector(50,n, val_bin(3*n)), \
            vector(50,n, val_bin(n) + TernaryCountLow1s(n)));
check_equal(vector(50,n, val_bin(3*n)), \
            vector(50,n, val_bin(n) + TernaryCountLow0s(2*n+1)));
\\ 
\\ new low 1, unchanged
check_equal(vector(50,n, val_bin(3*n+1)), \
            vector(50,n, val_bin(n)));
\\ 
\\ new low 2, +1
check_equal(vector(50,n, val_bin(3*n+2)), \
            vector(50,n, val_bin(n) + 1));
\\ 
my(n=from_ternary(2)); check_equal(valuation(binomial(3*n,n)/(2*n+1), 3),  1);
my(n=from_ternary(20)); check_equal(valuation(binomial(3*n,n)/(2*n+1), 3),  1);
my(n=from_ternary(200)); check_equal(valuation(binomial(3*n,n)/(2*n+1), 3),  1);
my(n=from_ternary(220)); check_equal(valuation(binomial(3*n,n)/(2*n+1), 3),  2);
my(n=from_ternary(2220)); check_equal(valuation(binomial(3*n,n)/(2*n+1), 3),  3);

{
  check_equal(vector(50,n, valuation(binomial(3*n,n)/(2*n+1),3)), 
              vector(50,n, n - sum(i=1,n, 
                               TernaryCountLow1s(i) + TernaryCountLow0s(i))),
              "val_bin by sum");
}


\\-----------------------------------------------------------------------------
\\ Other: binomial(3*n,n)

{
  my(v=OEIS_samples("A005809"));
  check_equal(vector(#v,n,n--; binomial(3*n,n)),
              v,
              "A005809 binomial");
}
check_equal(vector(50,n, binomial(3*n,n)), \
            vector(50,n, (3*n)!/(2*n)!/n!));

\\ A089643 valuation of binomial(3*n,n)
{
  my(v=OEIS_samples("A089643"));
  check_equal(vector(#v,n,n--; valuation(binomial(3*n,n),3)),
              v,
              "A089643 valuation");
}
check_equal(vector(50,n, valuation(binomial(3*n,n),3)), \
            vector(50,n, valuation((3*n)!,3) - valuation((2*n)!,3) - valuation((n)!,3)));

check_equal(vector(50,n, valuation(binomial(3*n,n),3)), \
            vector(50,n, n + sum(i=1,n, \
                           - TernaryCountLow0s(i) - TernaryCountLow1s(i-1) )));
check_equal(vector(50,n, valuation(binomial(3*n,n),3)), \
            vector(50,n, n - valuation((2*n)!,3)));

\\-----------------------------------------------------------------------------
\\ sumh, sumh'

sumh_C(n,k) = if(n<=1,n, sumh_C(ceil((n-1)/k),k) + n);
sumhdash_C(n,k) = sumh_C(n,k) - n;

\\ A127427 = v_3( (6n)! ) - v_3( (3n)! ), where v_3(N) is the 3-adic valuation
\\      cumulative TernaryCountLow1s
{
  my(v=OEIS_samples("A127427"));
  check_equal(vector(#v,n,n--; sumh_C(n,3)), v, "A127427");
}
check_equal(vector(100,n, sumh_C(n,3)), \
            vector(100,n, valuation((6*n)!,3) - valuation((3*n)!,3)));
check_equal(vector(100,n, sumh_C(n,3)), \
            vector(100,n, sumh_C(floor((n+1)/3),3) + n));
check_equal(vector(100,n, sumh_C(n+1,3)), \
            vector(100,n, sumh_C(n,3) + TernaryCountLow1s(n) + 1));
check_equal(vector(100,n, sumh_C(n+1,3)), \
            vector(100,n, sumh_C(n,3) + TernaryCountLow1s(3*n+1)));
check_equal(vector(20,n, sumh_C(3*n,3)), \
            vector(20,n, sumh_C(n,3) + 3*n));
check_equal(vector(20,n, sumh_C(3*n+1,3)), \
            vector(20,n, sumh_C(n,3) + 3*n + 1));
\\ vector(20,n, sumh_C(3*n+2,3)) - \
\\ vector(20,n, sumh_C(n,3) + 3*n)
\\ vector(20,n, TernaryCountLow0s(n))
\\ vector(20,n, sumh_C(n,3))
\\ vector(20,n, sumh_C(n+1,3) - sumh_C(n,3))
\\ vector(20,n, sumh_C(n+1,3) - sumh_C(n,3))
\\ 
\\ for(n=1,20, print(sumh_C(n,3),"  ",balanced_ternary_digits(n)))

\\ A220845 complete 3-ary trees sum of exclusive heights  sumh'(C_3(n))
A220845(n) = my(ret=0); while(n, n=ceil((n-1)/3); ret+=n); ret;
{
  my(v=OEIS_samples("A220845"));
  check_equal(vector(#v,n, A220845(n)), v, "A220845");
}
\\ vector(10,n, sumhdash_C(n,3))
check_equal(vector(6,n,n--; ceil((n-1)/3)), [0, 0, 1, 1, 1, 2]);
check_equal(vector(100,n, A220845(n)), \
            vector(100,n, sumhdash_C(n,3)));
check_equal(vector(100,n, A220845(n)), \
            vector(100,n, sumh_C(n,3) - n));
check_equal(vector(100,n, A220845(n)), \
            vector(100,n, sumh_C(ceil((n-1)/3),3)));
check_equal(vector(100,n, A220845(n)), \
            vector(100,n, A220845(ceil((n-1)/3)) + ceil((n-1)/3)));
check_equal(vector(100,n, A220845(n)), \
            vector(100,n, valuation((2*n)!,3) - valuation((n)!,3)));
check_equal(vector(100,n, sumhdash_C(n,3)), \
            vector(100,n, sum(i=0,n-1, TernaryCountLow1s(i))));
check_equal(vector(500,n, A220845(n)), \
            vector(500,n, sum(i=1,n, A253786(i))));
\\ cumulative TernaryCountLow1s()
check_equal(vector(100,n, sumhdash_C(n+1,3)), \
            vector(100,n, sumhdash_C(n,3) + TernaryCountLow1s(n)));

\\ vector(10,n, binomial(3*n,n) - binomial(3*(n-1),n-1))
\\ not in OEIS: 2, 12, 69, 411, 2508, 15561, 97716, 619191, 3951354, 25358190
check_equal(vector(1000,n, (binomial(3*n,n) - binomial(3*(n-1),n-1)) % 3), \
            vector(1000,n, if(n==1,2,0)));
\\ 
\\  vector(50,n, binomial(3*n,n) - binomial(3*(n-1),n-1))
\\  vector(50,n, (3*n)!/(2*n)!/n!)


\\-----------------------------------------------------------------------------
print("end");