\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

\\ Similar: rationals devel/A142151-bit-masks.gp

default(recover,0);
default(strictargs,1);

check_count = 0;
check_equal(got,want,name="") =
{
  check_count++;
  if(got != want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

gf_terms(g,n) = {
  if(g==0,return(vector(n)));
  my(x = variable(g),
     zeros = min(n,valuation(g,x)),
     v = Vec(g + O(x^n)));
  if(zeros>=0, concat(vector(zeros,i,0), v),
               v[-zeros+1 .. #v]);
}


\\-----------------------------------------------------------------------------
\\ A006519 - lowest 1 bit only

lowest_1_or_zero(n) =
{
  n>=0 || error();
  bitand(n,-n);
}
lowest_1(n) =
{
  n>=1 || error();
  1<<valuation(n,2);
}
A006519=lowest_1;  \\ OFFSET=1
{
  my(want=OEIS_samples("A006519")); \\ OFFSET=1
  check_equal(vector(#want,n, A006519(n)),
              want,
              "A006519 samples");

  check_equal(vector(1024,n, lowest_1(n)),
              vector(1024,n, lowest_1_or_zero(n)),
              "lowest_1 vs bitand");
  check_equal(lowest_1_or_zero(0), 0,
              "lowest_1_or_zero(0)");
}

\\ below lowest 1
\\ vector(24,n, lowest_1(n)\2)
\\ not in OEIS: 0, 1, 0, 2, 0, 1, 0, 4, 0, 1, 0, 2, 0, 1, 0, 8, 0, 1, 0, 2


\\-----------------------------------------------------------------------------
\\ A057168 - next same hamming

\\ Cf
\\ Joerg fxtbook 1.24.1 with loop for moving down bits by Doug Moore and
\\ Glenn Rhoads, and ref to Gosper's.
\\ Hacker's Delight, section 2-1, A Novel Application.

\\ Charles in A057168
nextham(n) =
{
  n>=1 || error();
  my(u=bitand(n,-n), v=u+n); (bitxor(v,n)/u)>>2 + v;
}
A057168 = nextham;

{
  my(want=OEIS_samples("A057168")); \\ OFFSET=1
  check_equal(vector(#want,n, A057168(n)),
              want,
              "A057168 samples");

  check_equal(vector(2^10,n, my(h=hammingweight(n), t=n+1);
                     while(hammingweight(t)!=h, t++); t),
              vector(2^10,n, A057168(n)),
              "nextham() by search");

  \\ MFH in A057168
  check_equal(vector(2^12,n, n + bitxor(n, n+n=bitand(n, -n))\n\4 + n),
              vector(2^12,n, A057168(n)),
              "nextham() vs modify n");

  \\ Gosper HAKMEM item 175 hacks.html
  my(G(A) = my(n=A);
     my(B,C,D);
     B = A;                /* MOVE B,A */
     C = -B;               /* MOVN C,B */
     C = bitand(C,B);      /* AND C,B  */  C==lowest_1(n) || error();
     A += C;               /* ADD A,C  */
     D = A;                /* MOVE D,A */
     D = bitxor(D,B);      /* XOR D,B  */  D%C==0 || error();
     \\ print(to_binary(D)"  "to_binary(C));
     D >>= 2;              /* LSH D,-2 */
     [C,D] = divrem(D,C);  /* IDIVM D,C  low end 0s needed */
     bitor(A,C);           /* IOR A,C   */
    );
  \\ G(from_binary(11100011100111100));

  check_equal(vector(24,A, G(A)),
              vector(24,n, A057168(n)),
              "nextham() vs Gosper HAKMEM");
}

\\-----------------------------------------------------------------------------
\\ A241816  lowest 10 -> 01, smallest change by swap two bits

lowest_10_flip(n) =
{
  \\ locate 10, then sub lowest_1 of that
  my(t=bitnegimply(n>>1,n)); n - bitand(t,-t);
}
A241816=lowest_10_flip;
{
  my(want=OEIS_samples("A241816")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A241816(n)),
              want,
              "A241816 samples, lowest_10_flip");

  check_equal(lowest_10_flip(31), 31, "lowest_10_flip() of no 10");

  check_equal(vector(2^10,n,n--;
                     my(t=bitnegimply(n>>1,n)); n - bitand(t,-t)),
              vector(2^10,n,n--; lowest_10_flip(n)),
              "lowest_10_flip() minus bitnegimply");

  \\----------
  check_equal(vector(2^10,n,n--; my(t=bitand(n,n+1));    \\ clear trailing 1s
                     n - bitand(t,-t)>>1),
              vector(2^10,n,n--; lowest_10_flip(n)),
              "lowest_10_flip() by clear trailing 1s");

  check_equal(vector(2^10,n,n--;
                     my(t=bitnegimply(n>>1,n)); n - bitnegimply(t,t-1)),
              vector(2^10,n,n--; lowest_10_flip(n)),
              "lowest_10_flip() minus bitnegimply");

  check_equal(vector(2^10,n,n--;
                     my(t=bitand(n>>1, bitneg(n)));    \\ clear trailing 1s
                     t = bitor(bitneg(t), t-1) + 1;
                     n + t),
              vector(2^10,n,n--; lowest_10_flip(n)),
              "lowest_10_flip() expand");

  check_equal(vector(2^10,n,n--;
                     my(t = bitor(bitor(n,              \\ 01 and 00 -> 01
                                        bitneg(n>>1)),
                                  bitand(n>>1, bitneg(n))-1)
                           + 1);
                     n + t),
              vector(2^10,n,n--; lowest_10_flip(n)),
              "lowest_10_flip() expand 2");
}
{
  \\ some parts
  my(n=from_binary(                  1110001100011),
     to=bitor(n, bitneg(n>>1)),
     x=bitand(n>>1, bitneg(n)));
  check_equal(to_binary(x),             1000010000);
  check_equal(to_binary(to%2^16), 1111110111101111);
  check_equal(to_binary(to),           -1000010001);
}

\\ amount subtracted
\\ vector(15,n,n++; n - lowest_10_flip(n))
\\ not in OEIS: 1, 0, 2, 2, 1, 0, 4, 4, 1, 4, 2, 2, 1, 0, 8
\\
\\ bit pos of lowest 10
\\ vector(15,n,n++; my(d=n-lowest_10_flip(n)); if(d,valuation(d,2),-1))
\\ not in OEIS: 0, 0, 1, 1, 0, 0, 2, 2, 0, 2, 1, 1, 0, 0, 3
\\ not in OEIS: 0, -1, 1, 1, 0, -1, 2, 2, 0, 2, 1, 1, 0, -1, 3

{
  if(0,
     my(len=64,
        inv=vector(len,i,[]));
     for(n=0,32*len,
        my(f=lowest_10_flip(n));
        if(f>=1 && f<=#inv,
           inv[f] = concat(inv[f],n)));
     for(f=1,#inv,
        print(f" "to_binary(f)"  "apply(to_binary,inv[f])));

     \\ how many inverses lowest_10_flip(something) = n
     \\ vector(#inv,i,#inv[i])
     \\ not in OEIS: 2, 1, 2, 0, 2, 1, 2, 1, 2, 0, 1, 0, 2, 1, 2, 1, 2, 1, 2, 0, 2, 0, 1, 1, 1, 0, 1

     \\ numbers not occurring, no inverses
     \\ A246590  even numbers with odd part 4k+3    ...1100..00
     \\ select(v->#v==1, inv, 1)

     select(v->#v==2, inv, 1)
    );
}

\\ A055941  sum 0 bits below each 1 bit (crossreffed in A241816)
lowest_10_flip_steps(n) =
{
  my(ret=0, t);
  while((t=lowest_10_flip(n))!=n, n=t; ret++);
  ret;
}
\\ vector(20,n,n--; lowest_10_flip_steps(n))


\\-----------------------------------------------------------------------------
\\ lowest 01 -> 10
lowest_01_flip(n) =
{
  my(t=bitnegimply(n,n>>1));     \\ locate 01
  n + lowest_1_or_zero(t);
}
\\ vector(15,n,n++; lowest_01_flip(n))
\\ not in OEIS: 4, 5, 8, 6, 10, 11, 16, 10, 12, 13, 20, 14, 22, 23, 32


\\-----------------------------------------------------------------------------
\\ A246590  even numbers with odd part 4k+3    ...1100..00
\\ even terms from A091067

lowest_01_pos(n) = n=bitnegimply(n,n>>1); if(n,valuation(n,2), 'none);
lowest_10_pos(n) = n=bitnegimply(n>>1,n); if(n,valuation(n,2), 'none);
isA246590(n) =
{
  my(a,b);
  (a=lowest_01_pos(n))!='none && (b=lowest_10_pos(n))!='none && a > b+1;
}
{
  my(want=OEIS_samples("A246590")); \\ OFFSET=0
  check_equal(select(isA246590, [0..want[#want]]),
              want,
              "A246590 samples");

  check_equal(vector(2^10,n,n--; bitand(n, lowest_1_or_zero(n)<<1) >= 4),
              vector(2^10,n,n--; isA246590(n)),
              "isA246590() by lowest_1");

  check_equal(vector(2^10,n,n--; bitand(n, bitand(n,-n)<<1) >= 4),
              vector(2^10,n,n--; isA246590(n)),
              "isA246590() by lowest_1");

  check_equal(vector(2^10,n,n--; bitand(n>>1, bitand(n,-n)) >= 2),
              vector(2^10,n,n--; isA246590(n)),
              "isA246590() by lowest_1");
}


\\-----------------------------------------------------------------------------
\\ A243109  prev same hamming or n if no such

\\ Joerg fxtbook 1.24.1 prev_colex_comb() by nextham of bitneg.

prevham_by_search(n) =
{
  n>=0 || error();
  my(want=hammingweight(n));
  forstep(p=n-1,0,-1,
     if(hammingweight(p)==want, return(p)));
  n;
}
A243109=prevham_by_search;
{
  my(want=OEIS_samples("A243109")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A243109(n)),
              want,
              "A243109 samples");

  my(n=from_binary(          10111000011 ),
     p=prevham_by_search(n));
  check_equal(to_binary(p),  10110111000 );
  check_equal(n,             1475, "example n");
  check_equal(p,             1464, "example p");
}

\\ Joerg fxtbook 1.24.1 prev_colex_comb() by nextham of bitneg.
prevham_by_bitneg(n) =
{
  my(k=if(n,logint(n,2))+8,
     p=nextham(bitneg(n,k)));
  if(p>=2^k,n, bitneg(p,k));
}
\\ [10111111111, 
\\   1111111111, 0]

{
  check_equal(vector(8,k,k--; prevham_by_bitneg(2^k-1)),
              vector(8,k,k--; 2^k-1),
              "prevham_by_bitneg() of 2^k-1");

  check_equal(vector(2^12,n,n--; prevham_by_bitneg(n)),
              vector(2^12,n,n--; prevham_by_search(n)),
              "prevham_by_bitneg() vs prevham_by_search()");
}

\\ at odd
\\ vector(24,n,n*=2;n++; prevham_by_search(n))
\\ not in OEIS: 3, 3, 7, 6, 7, 11, 15, 12, 14, 19, 15, 22, 23, 27, 31, 24

\\ 0 when no previous
\\ vector(20,n,n++; my(p=prevham_by_search(n)); if(p==n,0, p))
\\ not in OEIS: 1,0,2,3,5,0,4,6,9,7,10,11,13,0,8,12,17,14,18,19
\\
\\ -1 when no previous
\\ vector(20,n,n++; my(p=prevham_by_search(n)); if(p==n,-1, p))
\\ not in OEIS: 1,-1,2,3,5,-1,4,6,9,7,10,11,13,-1,8,12,17,14,18,19


\\-----------------------------------------------------------------------------
\\ A038712 - 1 bit mask of the lowest 1-bit and below

mask_to_lowest_1(n) =
{
  n>=1 || error();
  bitxor(n,n-1);
}
check_equal(mask_to_lowest_1(11), 1);
check_equal(mask_to_lowest_1(12), 7);
A038712=mask_to_lowest_1;
{
  my(want=OEIS_samples("A038712")); \\ OFFSET=1
  check_equal(vector(#want,n, A038712(n)),
              want,
              "A038712 samples");
}

\\-----------------------------------------------------------------------------
\\ A135481 - extract low 1s (if any)

low1s(n) = bitxor(n,n+1)>>1;
A135481=low1s;
{
  my(want=OEIS_samples("A135481")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A135481(n)),
              want,
              "A135481 samples");
}
check_equal(low1s(11), 3);
check_equal(low1s(12), 0);
check_equal(low1s(13), 1);

\\-----------------------------------------------------------------------------
\\ A007814 - count low 0s

count_low_0s(n) =
{
  n>=1 || error("count_low_0s() is for n>=1");
  valuation(n,2);
}
A007814=count_low_0s;
{
  my(want=OEIS_samples("A007814")); \\ OFFSET=1
  check_equal(vector(#want,n, A007814(n)),
              want,
              "A007814 samples");
}


\\-----------------------------------------------------------------------------
\\ A087116 - num runs of 0s

\\ by Gheorghe Coserea
A087116(n) = if(n==0, 1, hammingweight(bitxor(n, n>>1)) >> 1);
{
  my(want=OEIS_samples("A087116")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A087116(n)),
              want,
              "A087116 samples");

  \\ A087116 wrongly takes 0 as a run of 0s, so exception at n=0
  check_equal(vector(10,n,n--; my(ret=0);
                     if(n==0,ret=1);
                     while(1,
                           while(n%2,n>>=1); if(n==0,break);
                           ret++; while(n%2==0,n>>=1));
                     ret),
              vector(10,n,n--; A087116(n)),
              "A087116 by bit scan");
}

\\-----------------------------------------------------------------------------
\\ A229762 - 1-bit below each runs of 1s, so each 10 -> 01 and all else 0s

\\ my line of code
A229762(n) =
{
  n>=0 || error();
  bitnegimply(n>>1,n);
}
{
  my(want=OEIS_samples("A229762")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A229762(n)),
              want,
              "A229762 samples");
  want=OEIS_bfile_samples("A229762"); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A229762(n)),
              want,
              "A229762 samples");
  print("b229762 size ",#want);

  \\ my example
  my(n=from_binary(                 1110001100011));
  check_equal(to_binary(A229762(n)),  01000010000);
  check_equal(n,                    7267);
  check_equal(A229762(n),           528);
}
{
  check_equal(vector(1024,n,n--; if(n==0,0, A087116(n))),
              vector(1024,n,n--; hammingweight(A229762(n))),
              "A229762 hammingweight not quite A087116");
}

\\-----------------------------------------------------------------------------
\\ A229763  lowest 1-bit of each each run of 1s

A229763(n) =
{
  n>=0 || error();
  bitnegimply(n,n<<1);
}
{
  my(want=OEIS_samples("A229763")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A229763(n)),
              want,
              "A229763 samples");
  want=OEIS_bfile_samples("A229763"); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A229763(n)),
              want,
              "A229763 samples");
  print("b229763 size ",#want);

  check_equal(to_binary(A229763(from_binary(1100))),
                                             100);

  check_equal(vector(2^10,n,n--; A229763(n)),
              vector(2^10,n,n--; bitand(bitxor(2*n,n), n)),
              "A229763 sequence definition");

  \\ Ralf Stephan in A229763, evens bring out factor 2
  check_equal(vector(2^10,n,n--; A229763(2*n)),
              vector(2^10,n,n--; 2*A229763(n)));

  \\ Ralf Stephan in A229763, bit below runs in 2n
  check_equal(vector(2^10,n,n--; A229763(n)),
              vector(2^10,n,n--; A229762(2*n)));
}

\\ A229763 by 1 and 10xxxx
gA229763_term(x,k) = sum(i=2*2^k,3*2^k-1, 2*2^k*x^i) / (1-x^(4*2^k));
gA229763(x,k_limit) = x/(1-x^2) + sum(k=0,k_limit, gA229763_term(x,k));
{
  check_equal(gA229763_term(x,0), 2*x^2 / (1 - x^4) );
  check_equal(gf_terms(gA229763_term(x,0),1024),
              vector(1024,n,n--; if(n%4==2, 2)));
  check_equal(gf_terms(gA229763_term(x,1),1024),
              vector(1024,n,n--; if(bitand(n,6)==4, 4)));
  check_equal(vector(8,k,k--; gA229763_term(x,k)),
              vector(8,k,k--; 2^(k+1) * x^(2^(k+1))
                         / ((1-x) * (1 + x^(2^k)) * (1 + x^(2*2^k))) ));

  check_equal(gf_terms(gA229763(x,10), 2^10),
              vector(2^10,n,n--; A229763(n)));
}
\\ my(k=3); factor(1 + x^(2*2^k))
\\ my(k=3); factor(1 + x^(2^k))
\\ my(k=3); (1-x) * (1 + x^(2*2^k)) * (1 + x^(2^k))

\\-----------------------------------------------------------------------------
\\ Lowest 0 as a single 1

lowz(n) =
{
  n>=0 || error();
  n++; bitand(n,-n);
}
{
  check_equal(vector(1024,n,n--; lowz(n)),
              vector(1024,n,n--; A006519(n+1)),
              "lowz vs A006519");

  check_equal(vector(1024,n,n--; bitand(bitneg(n),n+1)),
              vector(1024,n,n--; lowz(n)),
              "lowz by bitand");
  check_equal(vector(1024,n,n--; bitand(-n-1,n+1)),
              vector(1024,n,n--; lowz(n)),
              "lowz by bitand");
  check_equal(vector(1024,n,n--; n++; bitand(n,-n)),
              vector(1024,n,n--; lowz(n)),
              "lowz by bitand");

  check_equal(vector(1024,n,n--; bitnegimply(n+1,n)),
              vector(1024,n,n--; lowz(n)),
              "lowz by bitnegimply");

  check_equal(vector(1024,n,n--; bitnegimply(n,n+1)+1),
              vector(1024,n,n--; lowz(n)),
              "lowz by bitnegimply+1");
}

\\-----------------------------------------------------------------------------
\\ A129760 - Clear Lowest 1

clear_lowest_1(n) = n - lowest_1(n);
A129760=clear_lowest_1;
{
  my(want=OEIS_samples("A129760")); \\ OFFSET=1
  check_equal(vector(#want,n, A129760(n)),
              want,
              "A129760 samples");

  check_equal(vector(1024,n, bitand(n,n-1)),   \\ borrow from lowest 1
              vector(1024,n, clear_lowest_1(n)),
              "clear_lowest_1 by bitand");
}

clear_trailing_1s(n) =
{
  n>=0 || error();
  clear_lowest_1(n+1);
}
{
  check_equal(clear_trailing_1s(from_binary(1100111)),
              from_binary(                  1100000));
  check_equal(clear_trailing_1s(from_binary(1100000)),
              from_binary(                  1100000));
  check_equal(clear_trailing_1s(from_binary(1010101)),
              from_binary(                  1010100));
}


\\-----------------------------------------------------------------------------
\\ A086799 - change low 0s to 1s

low_0s_to_1s(n) = bitor(n,n-1);
A086799=low_0s_to_1s;
{
  my(want=OEIS_samples("A086799")); \\ OFFSET=1
  check_equal(vector(#want,n, A086799(n)),
              want,
              "A086799 samples");
}

\\-----------------------------------------------------------------------------
\\ Lowest 1 Move Down to Lowest

lowest_1_to_lowest(n) =
{
  n>=0 || error();
  if(n==0,0, clear_lowest_1(n) + 1);
}
\\ vector(24,n, lowest_1_to_lowest(n))
\\ not in OEIS: 1, 1, 3, 1, 5, 5, 7, 1, 9, 9, 11, 9, 13, 13


\\-----------------------------------------------------------------------------
\\ Lowest 1 Down One

lowest_1_down_one(n) =
{
  n>=0 || error();
  if(n, n - lowest_1(n)\2, 0);
}
\\ vector(24,n, lowest_1_down_one(n))
\\ not in OEIS: 1, 1, 3, 2, 5, 5, 7, 4, 9, 9, 11, 10, 13

{
  check_equal(vector(1024,n,n*=2; lowest_1_down_one(n)),
              vector(1024,n,n*=2; prevham_by_search(n)),
              "prevham n even = lowest_1_down_one");
}


\\-----------------------------------------------------------------------------
\\ A066884  hammingweight n array (by diags)

hamarray(n,k) =
{
  n>=1 || error();
  k>=1 || error();
  my(p=n,C=1,ret=0);
  until(k<=C, p++;C*=p;C/=p-n);
  C==binomial(p,n) || error();
  while(p>n, C/=p;C*=p-n;p--;
        C==binomial(p,n) || error();
        if(k>C, ret+=1<<p; k-=C; C*=n;C/=p-n+1;n--));
  ret + bitneg(0,n);
}
{
  my(want=OEIS_samples("A066884"));
  check_equal(antidiagonals_of_func(#want,hamarray,1,1),
              want,
              "hamarray() A066884 samples");
  want=OEIS_bfile_samples("A066884"); \\ OFFSET=0
  print("b066884 size ",#want);
  check_equal(antidiagonals_of_func(#want,hamarray,1,1),
              want,
              "hamarray() b066884.txt samples");
}
{
  my(want=OEIS_samples("A067576"));
  check_equal(antidiagonals_of_func(#want,hamarray,1,1,
                                    1),  \\ downward
              want,
              "hamarray() A067576 samples");
}
{
  my(bad=0,count=0);
  for(n=1,80,
     my(want=bitneg(0,n));
     for(k=1,400,
        count++;
        hamarray(n,k)==want || bad++;
        want=nextham(want)));
  check_equal(bad, 0,
              Str("hamarray() vs nextham() ("count" checks)"));
}

\\ cf antidiagonals_of_func(30,binomial,0,0) \\ A052553
my_binomial(n,k) =
{
  n>=0 || error();
  if(n==k-1, 0, 
     (0 <= k && k <= n) || error();
     binomial(n,k));
}

hamarray_p(n,k) =
{
  for(p=n-1,oo,
     if(my_binomial(p,n) < k && k <= my_binomial(p+1,n), return(p)));
}
\\ vector(6,k, hamarray_p(3,k))
{
  my(T=(n,k)-> if(n==0&&k==1, 0, hamarray(n,k)),
     C=my_binomial);
  check_equal(matrix(30,150,n,k,
                     my(p=hamarray_p(n,k));
                     (C(p,n) < k && k <= C(p+1,n)) || error();
                     2^p + T(n-1,k-C(p,n))),
              matrix(30,150,n,k, T(n,k)),
              "hammarray() recurrence");
}
\\ my(n=3); vector(6,k, my(p=TtoP(n,k)); T(n-1,k-binomial(p,n)) + 2^p)
\\ my(n=3); vector(6,k, T(3,k))


\\ hamarray() compact
T(n,k) = { my(p=n,C=1,ret=0);  \\ C==binomial(p,n)
  until(k<=C, p++;C*=p;C/=p-n);
  while(p>n, C/=p;C*=p-n;p--;
    if(k>C, ret+=1<<p; k-=C; C*=n;n--;C/=p-n;));
  ret + bitneg(0,n); }
{
  check_equal(matrix(50,50,n,k, T(n,k)),
              matrix(50,50,n,k, hamarray(n,k)),
              "hamarray() compact");
}
kill(T);


\\ by search
\\ T(n,k) = k>=1 || error(); \
\\   for(i=0,oo, if(hammingweight(i)==n, if(k--<=0, return(i))));

hamarray_checked(n,k) =
{
  my(debug=0); k>=1 || error(); 
  my(i=n,C=1,ret=0); 
  while(i++; C*=i; C/=i-n; C==binomial(i,n)||error(); 
        k>binomial(i,n), ); 
  if(debug,print("start C="C)); 
  while(i>n, 
    C/=i;C*=i-n; 
    i--; 
    if(debug,print("i="i" n="n"  k="k" cmp c="binomial(i,n)" C="C)); 
    C==binomial(i,n)||error(); 
    my(c=binomial(i,n)); 
    if(k>c, 
      if(debug,print("  take 1")); 
      ret+=1<<i; k-=c; 
      C*=n; C/=i-n+1; n--)); 
  \\ [n,ret];
  ret + bitneg(0,n);
}
\\ TT(3,2)
\\ vector(6,k,TT(3,k))
\\ vector(6,k,T(3,k))
{
  check_equal(matrix(30,120,n,k,hamarray_checked(n,k)),
              matrix(30,120,n,k,hamarray(n,k)),
              "hamarray_checked() vs hamarray()");
}


\\-----------------------------------------------------------------------------
\\ A339597 - when 2*n+1 first appears in A086799
\\ 1100 1     1101 1   1011 1
\\            1101 0   1100 0

A339597_by_search(n) =
{
  n>=0 || error();
  for(i=0,oo, if(A086799(i)==2*n+1, return(i)));
}
{
  my(want=OEIS_samples("A339597")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A339597_by_search(n)),
              want,
              "A339597 samples");

  check_equal(vector(128,n,n=2*n-2; 2*n+1),
              vector(128,n,n=2*n-2; A339597_by_search(n)),
              "A339597 of even is 2n+1");
  check_equal(vector(128,n, 2*A339597_by_search(n)),
              vector(128,n, A339597_by_search(2*n+1)),
              "A339597 of odd is 2*sequence itself");

  check_equal(vector(128,n,n--; lowest_1_down_one(2*n+2)),
              vector(128,n,n--; A339597_by_search(n)),
              "A339597 = lowest_1_down_one(2n+2)");
}
check_equal(to_binary(A339597_by_search(8)),  10001);
check_equal(to_binary(A339597_by_search(11)), 10100);
check_equal(to_binary(A339597_by_search(12)), 11001, "12");
check_equal(to_binary(A339597_by_search(13)), 11010, "13");
check_equal(to_binary(A339597_by_search(14)), 11101);
check_equal(to_binary(A339597_by_search(15)), 10000);
{
  my(n=from_binary(                            11100011111));
  check_equal(to_binary(2*n+1),                111000111111);
  check_equal(to_binary(A339597_by_search(n)), 111000100000);
}
{
  my(n=from_binary(                            11100111110000));
  check_equal(to_binary(2*n+1),                111001111100001);
  check_equal(to_binary(A339597_by_search(n)), 111001111100001);
}
\\ vector(20,n,n--; A339597_by_search(n) - (2*n+1))

A339597(n) = 2*n + 1 - low1s(n);
{
  check_equal(vector(256,n,n--; A339597(n)),
              vector(256,n,n--; A339597_by_search(n)),
              "A339597() vs A339597_by_search()");

  want=OEIS_bfile_samples("A339597"); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A339597(n)),
              want,
              "A339597 samples");

  check_equal(vector(1024,n,n--; 2*n + 1 - bitxor(n,n+1)>>1),
              vector(1024,n,n--; A339597(n)),
              "A339597() vs bitxor");

  check_equal(vector(1024,n,n--; bitnegimply(2*n+1, low1s(n))),
              vector(1024,n,n--; A339597(n)),
              "A339597() vs bitnegimply");

  check_equal(vector(24,n,n--; bitand(2*n+1, bitneg(bitxor(n,n+1)>>1))),
              vector(24,n,n--; A339597(n)),
              "A339597() vs bitand");

  check_equal(vector(1024,n,n--; n++; 2*n - lowest_1(n)),
              vector(1024,n,n--; A339597(n)),
              "A339597() vs bit add");
  check_equal(vector(1024,n,n--; n++; 2*n - bitand(n,-n)),
              vector(1024,n,n--; A339597(n)),
              "A339597() vs bit add");
  check_equal(vector(1024,n,n--; 2*(n+1) - 1<<valuation(n+1,2)),
              vector(1024,n,n--; A339597(n)),
              "A339597() vs bit add");

  check_equal(vector(1024,n,n--; n++; 2*n - 1<<valuation(n,2)),
              vector(1024,n,n--; A339597(n)),
              "A339597() vs bit add");
}

\\-----------------------------------------------------------------------------
\\ Previous Hamming of Even

{
  check_equal(vector(1024,n,n*=2; lowest_1_down_one(n)),
              vector(1024,n,n*=2; prevham_by_search(n)),
              "prevham n even = lowest_1_down_one");

  check_equal(vector(1024,n, A339597(n-1)),
              vector(1024,n, prevham_by_search(2*n)),
              "prevham n even = A339597");
}


\\-----------------------------------------------------------------------------
\\ Previous Hamming Bit Twiddle

prevham(n) =
{
  n>=0 || error();
  my(t=bitnegimply(n>>1,n));       \\ A229762
  if(t==0,n,
     my(z=bitnegimply(n+1,n));     \\ lowz(n), A006519(n+1)
     n-z+1                         \\ A129760(n+1)
     - bitand(t,-t)/z);
}
{
  check_equal(vector(65536,n,n--; prevham(n)),
              vector(65536,n,n--; prevham_by_search(n)),
              "prevham() vs prevham_by_search()");

  \\ prevham formula
  check_equal(vector(2^16,n,n--; my(t = A129760(n+1));   
                     t == clear_trailing_1s(n) || error();  \\ clear 001..1
                     if(t==0,n, t - 2^(A007814(t) - A007814(n+1) - 1))),
              vector(2^16,n,n--; prevham(n)),
              "prevham() formula by 2^valuation");

  \\ prevham compact by bit shift
  check_equal(vector(1024,n,n--;    \\ t = strip low 1s
my(s=n+1,t=bitand(n,s)); if(t==0,n, t - 1<<(valuation(t,2)-valuation(s,2)-1));
                    ),
              vector(1024,n,n--; prevham(n)),
              "prevham() compact by valuation");

  \\----------

  check_equal(vector(1024,n,n--;
                     my(t=bitnegimply(n>>1,n));       \\ A229762
                     if(t==0,n,
                        n++; my(z=bitand(n,-n));     \\ lowest_1
                        n - z - bitand(t,-t)/z);
                    ),
              vector(1024,n,n--; prevham(n)),
              "prevham() with two bitand");

  check_equal(vector(1024,n,n--;
                     my(t=bitnegimply(n>>1,n));       \\ A229762
                     if(t==0,n,
                        my(h=valuation(n++,2));
                        n - 1<<h - 1<<(valuation(t,2)-h));
                    ),
              vector(1024,n,n--; prevham(n)),
              "prevham() by valuation");

  check_equal(vector(1024,n,n--;
                     my(t=bitnegimply(n>>1,n));       \\ A229762
                     if(t==0,n,
                        my(h=valuation(n++,2));
                        n - 1<<h - bitand(t,-t)>>h);
                    ),
              vector(1024,n,n--; prevham(n)),
              "prevham() by valuation and shift lowest_1");

  check_equal(vector(1024,n,n--;
                     my(w=bitnegimply(n>>1,n));
                     if(w==0,n, w=bitxor(w,w-1);
        bitnegimply(n,w) - (1<<(hammingweight(bitnegimply(w,n))-1))) ),
              vector(1024,n,n--; prevham(n)),
              "prevham() by hammingweight");

  \\----
  \\ parts
  check_equal(vector(1024,n,n--;
                     my(t=bitnegimply(n>>1,n), z=bitnegimply(n+1,n));
                     t),
              vector(1024,n,n--; A229762(n)),
              "prevham() t");

  check_equal(vector(1024,n,n--;
                     my(t=bitnegimply(n>>1,n), z=bitnegimply(n+1,n));
                     z),
              vector(1024,n,n--; A006519(n+1)),
              "prevham() z");

  check_equal(vector(1024,n,
                     my(t=bitnegimply(n>>1,n), z=bitnegimply(n+1,n));
                     n-z+1),
              vector(1024,n, A129760(n+1)),
              "prevham() n-z+1");

  \\---------
  \\ formula

  check_equal(vector(2^16,n,n--; my(t = A229762(n));
                     if(t==0,n, A129760(n+1) - A006519(t)/A006519(n+1))),
              vector(2^16,n,n--; prevham(n)),
              "prevham() formula by division");

  check_equal(vector(2^16,n,n--; my(t = A129760(n+1));   \\ strip 001..1
                     if(t==0,n, t - (1/2)*A006519(t)/A006519(n+1))),
              vector(2^16,n,n--; prevham(n)),
              "prevham() formula by 1/2");

  \\---------

  check_equal(vector(2^12,n,n*=2; n - lowest_1(n/2)),
              vector(2^12,n,n*=2; prevham(n)),
              "prevham() formula for even n");

  check_equal(vector(2^12,n,n--; my(f=bitnegimply(n>>1,n),
                                    t=valuation(n+1,2),
                                    s=if(f,valuation(f,2)+1,t+1),
                                    x=n>>s);
                     s >= t+1 || error("n="to_binary(n)" s="s" t="t);
                     x*2^s + 2^t-1 == n || error("n="to_binary(n)" s="s" t="t" x="x);
                     if(x==0, n,  n - 2^(s-t-1) - 2^t + 1) ),
              vector(2^12,n,n--; prevham_by_search(n)),
              "prevham() formula by x*2^s");

  check_equal(vector(2^12,n,n--; my(f=bitnegimply(n>>1,n),
                                    t=valuation(n+1,2),
                                    s=if(f,valuation(f,2)+1,t+1),
                                    x=n>>s);
                     s >= t+1 || error("n="to_binary(n)" s="s" t="t);
                     x*2^s + 2^t-1 == n || error("n="to_binary(n)" s="s" t="t" x="x);
                     if(x==0, n,  x*2^s - 2^(s-t-1) ) ),
              vector(2^12,n,n--; prevham_by_search(n)),
              "prevham() formula by x*2^s");
}


\\-----------------------------------------------------------------------------
check_equal(n,'n);
check_equal(C,'C);
check_equal(p,'p);
check_equal(k,'k);
check_equal(i,'i);
check_equal(T,'T);
print("end");
