\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\ngot  %s\nwant %s",x,y)));
  return(0);
}

check_equal(got,want,name="") =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong ",name,"\n",d); quit(1),
     print("ok ",check_count," ",name));
}

\\-----------------------------------------------------------------------------

points_rotate(v, angle) = v * exp(angle*I);
points_translate(v, offset) = apply(z->z+offset, v);

twindragon_midpoints(k) =
{
  my(v = twindragon_points(k));
  v = concat(v, [v[1]]);
  vector_midpoints(v);
}
check_exact(twindragon_midpoints(0), \
            [ 1/2, 1+1/2*I, 1/2+I, 1/2*I ], \
            "twindragon_midpoints(0)");

twindragon_centroid(k) = 1/2*b^(k+1);

check_exact(twindragon_centroid(0), \
            1/2 + 1/2*I, \
            "twindragon_centroid(0)");
check_exact(vector(10,k,k--; twindragon_centroid(k)), \
            vector(10,k,k--; vector_mean(twindragon_midpoints(k))), \
            "twindragon_centroid");

cis(a) = cos(a) + I*sin(a);

\\-----------------------------------------------------------------------------

TI_ka_ref(k,a) =
{
  if(k<0,error("TI_ka_ref() is for k>=0"));
  my(v=twindragon_midpoints(k));
  my(centroid = twindragon_centroid(k));
  my(rotate = 1/((b/sqrt(2))^(k+1) * cis(a)));
  v = apply(z->(z-centroid)*rotate, v);
  sum(i=1,#v, imag(v[i])^2);
}
TI_ka_ref = memoize(TI_ka_ref);

\\ imag(p/cis(a))^2
\\ = imag((x+I*y)*(cos(a) - I*sin(a)))^2

TI_x_ref(k) =
{
  if(k<0,error("TI_x_ref() is for k>=0"));
  my(v = twindragon_midpoints(k));
  my(centroid = twindragon_centroid(k));
  my(rotate = 1/b^(k+1));
  v = apply(z->(z-centroid)*rotate, v);
  sum(i=1,#v, imag(v[i])^2) * 2^(k+1);
}
TI_x_ref = memoize(TI_x_ref);
check_equal(vector(10,k,my(k=k-1); TI_x_ref(k)), \
            vector(10,k,my(k=k-1); TI_ka_ref(k,0)), \
            "TI_x_ref")

check_exact(TI_x_ref(0), \
            4 * (1/2*1/sqrt2)^2, \
            "k=0  TI_x 4 half diagonals");

TI_y_ref(k) =
{
  if(k<0,error("TI_y_ref() is for k>=0"));
  my(v=twindragon_midpoints(k));
  my(centroid = twindragon_centroid(k));
  my(rotate = 1/b^(k+1));
  v = apply(z->(z-centroid)*rotate, v);
  sum(i=1,#v, real(v[i])^2) * 2^(k+1);
}
TI_y_ref = memoize(TI_y_ref);
check_equal(vector(10,k,my(k=k-1); TI_y_ref(k)), \
            vector(10,k,my(k=k-1); TI_ka_ref(k,Pi/2)), \
            "TI_y_ref")

TI_xy_ref(k) =
{
  if(k<0,error("TI_xy_ref() is for k>=0"));
  my(v=twindragon_midpoints(k));
  my(centroid = twindragon_centroid(k));
  my(rotate = 1/b^(k+1));
  v = apply(z->(z-centroid)*rotate, v);
  -2 * sum(i=1,#v, imag(v[i])*real(v[i])) * 2^(k+1);
}
TI_xy_ref = memoize(TI_xy_ref);
{
  check_equal(vector(40,a,my(a=2*Pi/a);
              vector(10,k,my(k=k-1);
         TI_x_ref(k)*cos(a)^2 + TI_xy_ref(k)*cos(a)*sin(a) + TI_y_ref(k)*sin(a)^2
              )),
    vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); TI_ka_ref(k,a))),
    "TI_xy_ref")
}

TI_z_ref(k) =
{
  if(k<0,error("TI_z_ref() is for k>=0"));
  my(v=twindragon_midpoints(k));
  my(centroid = twindragon_centroid(k));
  v = apply(z->(z-centroid), v);
  sum(i=1,#v, norm(v[i]));
}
TI_z_ref = memoize(TI_z_ref);
check_equal(vector(10,k,my(k=k-1); TI_z_ref(k)), \
            vector(10,k,my(k=k-1); TI_x_ref(k) + TI_y_ref(k)))

TI_z_ref = memoize(TI_z_ref);
check_exact(TI_z_ref(0), \
            4 * 1/2^2, \
            "k=0  4 half squares");

\\-----------------------------------------------------------------------------
\\ cos(a+45) = cos(a)*1/sqrt(2) - sin(a)*1/sqrt(2)

{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + Pi/4),
                                1/sqrt(2)*cos(a) - 1/sqrt(2)*sin(a))),
            vector(100,k, 1))
}
{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + Pi/4)^2,
                                1/2*cos(a)^2 - cos(a)*sin(a) + 1/2*sin(a)^2)),
            vector(100,k, 1))
}
{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + Pi/4)^2,
                                1/2 - cos(a)*sin(a))),
            vector(100,k, 1))
}

{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + 3*Pi/4),
                                - 1/sqrt(2)*cos(a) - 1/sqrt(2)*sin(a))),
            vector(100,k, 1))
}
{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + 3*Pi/4)^2,
                                1/2 + cos(a)*sin(a))),
            vector(100,k, 1))
}

\\-----------------------------------------------------------------------------
\\ sin(a+45) = cos(a)*1/sqrt(2) + sin(a)*1/sqrt(2)

{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(sin(a + Pi/4),
                                1/sqrt(2)*cos(a) + 1/sqrt(2)*sin(a))),
            vector(100,k, 1))
}
{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(sin(a + Pi/4)^2,
                                1/2 + cos(a)*sin(a))),
            vector(100,k, 1))
}

{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(sin(a + 3*Pi/4),
                                1/sqrt(2)*cos(a) - 1/sqrt(2)*sin(a))),
            vector(100,k, 1))
}
{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(sin(a + 3*Pi/4)^2,
                                1/2 - cos(a)*sin(a))),
            vector(100,k, 1))
}

\\-----------------------------------------------------------------------------
\\ cos(a+45)*sin(a+45) 

{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + Pi/4)*sin(a + Pi/4),
                                1/2*cos(a)^2 - 1/2*sin(a)^2)),
            vector(100,k, 1))
}
{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + 3*Pi/4)*sin(a + 3*Pi/4),
                                - 1/2*cos(a)^2 + 1/2*sin(a)^2)),
            vector(100,k, 1))
}


\\-----------------------------------------------------------------------------
\\ TI_ka()

\\            *-------*
\\            |       |
\\            |   +   |
\\            |       |
\\    *-------*-------*  --> X
\\    |       |
\\    |   +   |
\\    |       |
\\    *-------+

\\ sin(a - arg(I_GS_diff))^2
\\ = (sin(a)*cos(diff) - cos(a)*sin(diff))^2
\\ r^2*( sin(a)^2*cos(diff)^2 - 2*cos(a)*sin(diff)*sin(a)*cos(diff) + cos(a)^2*sin(diff)^2 )
\\ = sin(a)^2*real(diff)^2 - 2*cos(a)*sin(a)*imag(diff)*real(diff) + cos(a)^2*imag(diff)^2
\\
\\ r^2*cos(arg(diff))^2 = real(diff)^2
\\
\\ (     imag(I_GS_diff(k))^2                  * cos(a)^2
\\                  - 2* imag(I_GS_diff(k))*real(I_GS_diff(k)) * cos(a)*sin(a)
\\                  +    real(I_GS_diff(k))^2                  * sin(a)^2
\\                  )

TI_ka(k,a) =
{
  if(k==0, return(1/2));

  2*  TI_ka_ref(k-1, a + Pi/4)
  + 2* 4*2^(k-1) * ( abs(1/2*b^k)*sin(a-degrees(45)) )^2;
  ;
}
check_equal(TI_ka_ref(0,0), 1/2)
check_equal(TI_ka(0,0), 1/2)
check_equal(TI_ka(0,Pi/2), 1/2)
check_equal(TI_ka_ref(1,0), 3)
check_equal(TI_ka(1,Pi/2), 3)

check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); TI_ka(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); TI_ka_ref(k,a))))

TI_ka(k,a) =
{
  if(k==0, return(1/2));

  2*  TI_ka_ref(k-1, a + Pi/4)
  + 4^k * sin(a-degrees(45))^2;
  ;
}
check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); TI_ka(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); TI_ka_ref(k,a))))

TI_ka(k,a) =
{
  if(k==0, return(1/2));

  + 2*TI_x_ref(k-1)   *( 1/2*cos(a)^2 - cos(a)*sin(a) + 1/2*sin(a)^2)
  + 2*TI_xy_ref(k-1)  *( 1/2*cos(a)^2                 - 1/2*sin(a)^2)
  + 2*TI_y_ref(k-1)   *( 1/2*cos(a)^2 + cos(a)*sin(a) + 1/2*sin(a)^2)

  + 4^k * ( 1/2*cos(a)^2 - cos(a)*sin(a) + 1/2*sin(a)^2)
  ;
}
check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); TI_ka(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); TI_ka_ref(k,a))))

TI_x(k) =
{
  if(k==0, return(1/2));
  TI_x_ref(k-1) + TI_xy_ref(k-1) + TI_y_ref(k-1) + 1/2*4^k
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_x(k)), \
            vector(10,k,my(k=k-1); TI_x_ref(k)), "TI_x")

TI_y(k) =
{
  if(k==0, return(1/2));
  TI_x_ref(k-1) - TI_xy_ref(k-1) + TI_y_ref(k-1) + 1/2*4^k
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_y(k)), \
            vector(10,k,my(k=k-1); TI_y_ref(k)), "TI_y")

TI_xy(k) =
{
  if(k==0, return(0));
  -2*TI_x_ref(k-1) + 2*TI_y_ref(k-1) - 4^k
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_xy(k)), \
            vector(10,k,my(k=k-1); TI_xy_ref(k)), "TI_xy")


\\-----------------------------------------------------------------------------
\\ TI_xy

TI_xy(k) =
{
  if(k==0, return(0));
  if(k==1, return(-4));
  -4*TI_xy_ref(k-2) - 4^k
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_xy(k)), \
            vector(10,k,my(k=k-1); TI_xy_ref(k)), "TI_xy")

\\ gTI_xy(x) = -4*x^2*gTI_xy(x) - 1/(1-4*x) + 1
\\ gTI_xy(x)*(1+4*x^2) = - 1/(1-4*x) - 4*x
gTI_xy(x) = 1/(1+4*x^2) * (-1/(1-4*x) + 1);
check_equal(gf_terms(gTI_xy(x), 10), \
            vector(10,k,my(k=k-1); TI_xy_ref(k)), "gTI_xy")

pol_ascending_print(pol_partial_fractions(gTI_xy(x)))

gTI_xy(x) = (-4/5)/(1 - 4*x)  + (4/5 - 4/5*x)/(1 + 4*x^2);
check_equal(gf_terms(gTI_xy(x), 10), \
            vector(10,k,my(k=k-1); TI_xy_ref(k)), "gTI_xy")

TI_xy(k) =
{
  -4/5*4^k + vector_modulo([4/5, -4/5],k)*(-4)^floor(k/2)
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_xy(k)), \
            vector(10,k,my(k=k-1); TI_xy_ref(k)), "TI_xy")

TI_xy(k) =
{
  -4/5*4^k + vector_modulo([4/5, -2/5, -4/5, 2/5],k)*2^k
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_xy(k)), \
            vector(10,k,my(k=k-1); TI_xy_ref(k)), "TI_xy")

\\ print(vector(10,k,k-=1; TI_xy_ref(k)))
\\ print(vector(10,k,k-=1; TI_xy(k)))

\\-----------------------------------------------------------------------------
\\ TI_z

TI_z(k) =
{
  if(k==0, return(1));
  2*TI_z_ref(k-1) + 4^k
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_z(k)), \
            vector(10,k,my(k=k-1); TI_z_ref(k)), "TI_z")
TI_z(k) =
{
  sum(j=0,k, 2^(k-j)*4^j );
}
check_equal(vector(10,k,my(k=k-1); TI_z(k)), \
            vector(10,k,my(k=k-1); TI_z_ref(k)), "TI_z")

TI_z(k) =
{
  2^k*sum(j=0,k, (4/2)^j );
}
check_equal(vector(10,k,my(k=k-1); TI_z(k)), \
            vector(10,k,my(k=k-1); TI_z_ref(k)), "TI_z")

TI_z(k) =
{
  2^k*(2^(k+1) - 1);
}
check_equal(vector(10,k,my(k=k-1); TI_z(k)), \
            vector(10,k,my(k=k-1); TI_z_ref(k)), "TI_z")

TI_z(k) =
{
  2*4^k - 2^k;
}
check_equal(vector(10,k,my(k=k-1); TI_z(k)), \
            vector(10,k,my(k=k-1); TI_z_ref(k)), "TI_z")

\\-----------------------------------------------------------------------------
\\ TI_x

TI_x(k) =
{
  if(k==0, return(1/2));
  TI_xy_ref(k-1) + TI_z_ref(k-1) + 1/2*4^k
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_x(k)), \
            vector(10,k,my(k=k-1); TI_x_ref(k)), "TI_x")

TI_x(k) =
{
  if(k==0, return(1/2));
  TI_xy_ref(k-1) + 2*4^(k-1) - 2^(k-1) + 1/2*4^k
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_x(k)), \
            vector(10,k,my(k=k-1); TI_x_ref(k)), "TI_x")

TI_x(k) =
{
  if(k==0, return(1/2));
  TI_xy_ref(k-1) + 4^k - 1/2*2^k
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_x(k)), \
            vector(10,k,my(k=k-1); TI_x_ref(k)), "TI_x")

TI_x(k) =
{
  if(k==0, return(1/2));
 -4/5*4^(k-1) + vector_modulo([4/5, -2/5, -4/5, 2/5],(k-1))*2^(k-1)
  + 4^k - 1/2*2^k
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_x(k)), \
            vector(10,k,my(k=k-1); TI_x_ref(k)), "TI_x powers")

TI_x(k) =
{
  4/5*4^k - vector_modulo([3/10, 1/10, 7/10, 9/10],k)*2^k
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_x(k)), \
            vector(10,k,my(k=k-1); TI_x_ref(k)), "TI_x powers")

\\-----------------------------------------------------------------------------
\\ TI_y

TI_y(k) =
{
  6/5*4^k - vector_modulo([7/10, 9/10, 3/10, 1/10],k)*2^k
  ;
}
check_equal(vector(10,k,my(k=k-1); TI_y(k)), \
            vector(10,k,my(k=k-1); TI_y_ref(k)), "TI_y")

\\ print(vector(50,k,my(k=k-1); TI_y(k)))


\\-----------------------------------------------------------------------------

TI_ka(k,a) =
{
  TI_x(k)    *cos(a)^2
  + TI_xy(k) *cos(a)*sin(a)
  + TI_y(k)  *cos(a)^2;
}
ploth(alpha=0,360, alpha = alpha*Pi/180; TI_ka(100,alpha))

quit

TI_x(k) =
{
  my(v = dragon_midpoints(k+1));
  my(centroid = twindragon_centroid(k));
  my(rotate = 1/b^(k+1));
  v = apply(z->(z-centroid)*rotate, v);
  v = concat(v, -v);

  sum(i=1,#v, imag(v[i])^2) * 2^(k+1);
}
check_equal(vector(8,k,my(k=k-1); TI_x(k)), \
            vector(8,k,my(k=k-1); TI_x_ref(k)), "TI_x by two dragons")

\\ I_x = sum x^2
\\ TI_x = (sum (x+a)^2) + (sum (-x-a)^2)
\\      = 2*( sum (x+a)^2 )
\\      = 2*( sum x^2 + 2*xa + a^2 )
\\      = 2*( sum(x^2) + 2^k*a^2 )         since sum(x)==0
\\      = 2*I_x + 2*2^(k+1)*a^2

\\ I_xy = -2*sum xy
\\ TI_x = -2*(sum (x+a)*(y+b) + sum (-x-a)*(-y-b) )
\\      = 2*-2*( sum (x+a)*(y+b) )
\\      = 2*-2*( sum x*y + x*b + y*a + a*b )
\\      = 2*-2*( sum x*y ) + 2*-2*(sum a*b )     since sum(x)==sum(y)==0
\\      = 2*I_xy + 2*-2*2^(k+1)*a*b
\\      = 2*I_xy + -4*2^(k+1)*a*b


d(k) = (twindragon_centroid(k) - GS(k+1)) / b^(k+1);
print("Ix");
print(vector(10,k,k--; TI_x(k) - 2*I_x(k+1)));
print(vector(10,k,k--; 2 * 2^(k+1) * imag(d(k))^2 * 2^(k+1)));
print("Iy");
print(vector(10,k,k--; TI_y(k) - 2*I_y(k+1)));
print(vector(10,k,k--; 2 * 2^(k+1) * real(d(k))^2 * 2^(k+1)));
print("Ixy");
print(vector(10,k,k--; TI_xy(k) - 2*I_xy(k+1)));
print(vector(10,k,k--; -4 * 2^(k+1) * imag(d(k))*real(d(k)) * 2^(k+1)));

\\ dd(k) = ((1/2*b^k - GS(k)) / b^k)^2 * 2^k;
\\ print(vector(10,k,k--; 2*real(dd(k))^2));
\\ print(vector(10,k,k--; 2*imag(dd(k))^2));

quit
\\-----------------------------------------------------------------------------
print("inertia by twindragon_midpoints()");
{
  my(k=10);
  points = twindragon_midpoints(k) / b^(k+1);
  points = points_translate(points, -vector_mean(points));
  check_exact(vector_mean(points), 0);
  m = geom_inertia_of_points(points) * 2^k;
  geom_inertia__print(m);
}

d = geom_inertia_principal_xydir(m);
d=d[1]+d[2]*I;
print("d " d);
print("d slope " imag(d)/real(d), " = ", imag(d)/real(d)*1.0, "   cf 3/4");
print("d sqrt " sqrt(d));
alpha_max = geom_inertia_principal_xyangle(m);
alpha_min = alpha_max + Pi/2;
print("degrees min ",alpha_min*180/Pi,"   max ",alpha_max*180/Pi);
print("radians min ",alpha_min,"   max ",alpha_max);
print("I       min ",geom_inertia_I_at_xyangle(m,alpha_min))
print("I       max ",geom_inertia_I_at_xyangle(m,alpha_max))
\\ forstep(alpha=degrees(-10),degrees(60), degrees(2), \
\\        print(geom_inertia_I_at_xyangle(m,alpha),"   at alpha=", alpha*180/Pi));

{
  for(k=0,10,
     points = twindragon_midpoints(k) / b^(k+1);
     points = points_translate(points, -vector_mean(points));
     if(vector_mean(points) != 0, error());
     m = geom_inertia_of_points(points) * 2^k;
     print1(4*geom_inertia_TI_x(m),",");
  );
print();

  for(k=0,10,
     points = twindragon_midpoints(k) / b^(k+1);
     points = points_translate(points, -vector_mean(points));
     if(vector_mean(points) != 0, error());
     m = geom_inertia_of_points(points) * 2^k;
     print1(4*geom_inertia_TI_y(m),",");
  );
print();
}

TI_z(k) = 4^k - 1/2*2^k;
