\\ Copyright 2016, 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

arg_0_to_2pi(z) = my(a=arg(z)); if(a<0,a+=2*Pi); a;
lex_by_abs(a,b) = \
  if(abs(a) == abs(b), lex(arg_0_to_2pi(a),arg_0_to_2pi(b)), lex(abs(a),abs(b)));
lex_by_abs_descending(a,b) = - lex_by_abs(a,b);


\\-----------------------------------------------------------------------------

n_other_high_common(n) =
{
  my(other=n_to_other(n));
  if(other == -1, return(""));
  n = binary(n);
  other = binary(other);
  if(#n != #other, return(0));
  for(i=1,#n, if(n[i] != other[i], return(i-1)));
  #n;
}

n_to_other(n) =
{
  my(z = dragon_point(n),
     an1 = dragon_unpoint(z,1),
     an2 = dragon_unpoint(z,-1));
  if(! ((an1[1] == 0 && an1[2] == n)
        || (an2[1] == 0 && an2[2] == n)),
     error("oops, ",an1," ",an2));
  if(an1[1] == 0 && an1[2] != n,  an1[2],
     an2[1] == 0 && an2[2] != n,  an2[2],
     -1);
}
vector(16+1,n,n--; n_to_other(n))
{
  for(n=0,50,
     my(other = n_to_other(n));
     if(other > n, printf("%3d %3d %10s %10s     %s\n",
                          n, other, to_binary(n), to_binary(other),
                          n_other_high_common(n)));
  );
}

\\-----------------------------------------------------------------------------

binary_to_b_powers(v) =
{
  my(s=1);
  for(i=2,#v, v[i]*=s; if(v[i]&&!v[i-1], s = -s));
  v;
}

\\ 7,11 15,19
v = concat([0,0],binary(11));
v = binary_to_b_powers(v);
printf("%40s\n", v);
subst(Pol(v),'x,b)

v = concat([0,0,0,0,0,0],binary(7));
v = binary_to_b_powers(v);
printf("%40s\n", v);
subst(Pol(v),'x,b)

{
  v = concat([0,0,0],binary(11));
  vabs = vector(#v);
  pos = #v;
  old_s = 1;
  new_s = -1;
  delta = 0;
  while(pos > 1,
        print();
        if(v[pos] && !v[pos-1],
           old_s = -old_s;
           print("  old_s change to ",old_s));
        old_bit_signed = v[pos] * old_s;
        print("at pos="pos" "v" delta "delta"  old_s="old_s"  new_s="new_s);

        print("  old n bits  "v[pos-1]," "old_bit_signed);
        my(z = v[pos-1]*b + old_bit_signed);
        my(z_plus_delta = z+delta);
        print("  z = ",z,"   z+delta = "z_plus_delta"  mod b^2 = "z_mod_bsquared_bits(z_plus_delta));
        if(z_mod_bsquared_bits(z_plus_delta) == 1,
           new_s = -new_s;
           print("  new_s change to ",new_s));

        new_bit = z_mod_b(z_plus_delta);
        new_bit_signed = new_bit * new_s;
        more_delta = old_bit_signed - new_bit_signed;
        print("  new_bit="new_bit" "new_bit_signed"  more delta "more_delta);
        delta += more_delta;
        vabs[pos] = new_bit;
        v[pos] = new_bit_signed;

        pos--;
        delta /= b;
       );
}
print();
print("end "v" delta "delta" old_s="old_s" new_s="new_s"   n="from_binary_vector(vabs));
print("  v    "v);
print("  vabs "vabs);
subst(Pol(v),'x,b)
print();

delta_seen = Map();
more_delta_seen = Map();

n_to_other_by_delta(n) =
{
  my(v = concat([0,0],binary(n)));
  my(pos = #v);
  my(old_s = 1);
  my(new_s = -1);
  my(delta = 0);
  while(pos > 1,
        mapput(delta_seen,delta,1);

        \\ old_bit == old_bit_signed mod b^2
        z = v[pos-1]*b + v[pos];
        if(z_mod_bsquared_bits(z) == 1, old_s = -old_s);

        z += delta;
        if(z_mod_bsquared_bits(z) == 1, new_s = -new_s);

        \\ if(v[pos] && !v[pos-1], old_s = -old_s);
        my(old_bit = v[pos]);

        \\ more_delta (1,0,-1) - (1,0,-1) = 2,1,0,-1,-2
        new_bit = z_mod_b(z);
        my(more_delta = old_bit*old_s - new_bit*new_s);
        mapput(more_delta_seen,more_delta,1);
        delta += more_delta;

        v[pos] = new_bit;
        pos--;
        delta /= b;
       );
  \\ print("end "v" delta "delta" old_s="old_s" new_s="new_s"   n="from_binary_vector(v));
  if(delta || old_s != new_s, -1, from_binary_vector(v));
}
{
  my(count=0);
  for(n=1,1000,
     my(want = n_to_other(n));
     \\ if(want == -1, next());
     my(got  = n_to_other_by_delta(n));
     if(got != want,
        print("n="n);
        print("want="want);
        print("got ="got);
        quit(1);
        );
     count++;
  );
  print(count" ok");
}

print("delta seen");
v = Mat(delta_seen)[,1];
v = vecsort(v,lex_by_abs);
{
  for(i=1,#v,
     printf("%6s   %d\n", v[i], z_mod_bsquared_bits(v[i]));
  );
}

print("more delta seen");
v = Mat(more_delta_seen)[,1];
v = vecsort(v,lex_by_abs)

n_to_other_iterate(state,bit_above) =
{
  my(old_s = state[1],
     new_s = state[2],
     delta = state[3],
     bit   = state[4]);

  if(bit && !bit_above, old_s = -old_s);
  my(old_bit = bit);

  z = bit_above*b + old_bit + delta;
  if(z_mod_bsquared_bits(z) == 1, new_s = -new_s);

  new_bit = z_mod_b(z);
  delta += old_bit*old_s - new_bit*new_s;
  \\ v[pos] = new_bit;

  delta /= b;
  [old_s, new_s, delta, bit_above];
}

print();
{
  my(l = List([]),
     seen = Map());
  listput(l, [1,-1,0, 0]);
  listput(l, [1,-1,0, 1]);
  while(#l,
        my(state = l[1]);
        listpop(l,1);
        if(mapisdefined(seen,state), next());
        mapput(seen,state,1);

        print(state);
        for(bit_above=0,1,
           my(new_state = n_to_other_iterate(state,bit_above));
           print("  "bit_above" "bit"  "new_state);
           listput(l, new_state);
           
           my(new_delta=new_state[3]);
           if(! mapisdefined(delta_seen,new_delta),
              print("unexpected delta "new_delta);
              quit(1));
        );
       );
  print("seen states  ",#seen);
}