#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.

use strict;
use 5.006;
use Carp 'croak';
use Math::PlanePath::DragonCurve;
use Test::More;

use lib 'tools';
use MyXYConvexHull ':all';

use MyTestHelpers;
BEGIN { MyTestHelpers::nowarnings(); }

plan tests => 12;


#------------------------------------------------------------------------------
# num_points()
# num_points_even()

{
  my $h = MyXYConvexHull->new_from_points([0,0]);
  is($h->num_points, 1);
  is($h->num_points_even, 1);
  is($h->num_points_odd, 0);
}
{
  my $h = MyXYConvexHull->new_from_points([1,0]);
  is($h->num_points, 1);
  is($h->num_points_even, 0);
  is($h->num_points_odd, 1);
}

{
  my $h = MyXYConvexHull->new_from_points([0,0], [1,1], [2,0]);
  is($h->num_points, 4);
  is($h->num_points_even, 3);
  is($h->num_points_odd, 1);
}
{
  my $h = MyXYConvexHull->new_from_points([1,1],[1,-1], [-1,1],[-1,-1]);
  is($h->num_points, 9);
  is($h->num_points_even, 5);
  is($h->num_points_odd, 4);
}

#------------------------------------------------------------------------------
exit 0
