#!/usr/bin/perl -w

# Copyright 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::BigRat;
use Math::Geometry::Planar;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon;
use MyPlanar ':all';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


# should be $level == 2 mod 4
my $type = $ARGV[0] || '135';

print "  % generated by tools/$FindBin::Script $type\n";

my $points = $MyDragon::hull6_points;
$points = MyPlanar::points_scale($points,3);  # to factor 18
my $scale = 6 * 3;
print "  \\begin{scope}[scale=1/$scale]\n";

{
  print "% GP-Test  18*hull_fractal == [";
  my $join = '';
  foreach my $point (@$points) {
    print "$join$point->[0]+I*$point->[1]";
    $join = ',';
  }
  print "]\n";
}
scalar(@$points) == 10 or die "oops";
# print MyDragon::points_str($points),"\n";
{
  require Math::Geometry::Planar;
  my $planar = Math::Geometry::Planar->new;
  $planar->points($points);
  $planar = $planar->convexhull2;
  $points = $planar->points;
}
# print MyDragon::points_str($points),"\n";

my %surrounding
  = (135 => [ [0,0, 2],
              [0,0, 3],
              [1,-1, 2],
              [1,-1, 1],
              [1,1, 3],
              [2,0, 2],
            ],
     246 => [ [0,0, 3],
              [0,0, 2],
              [-1,1, 0],
              [0,2, 3],
              [1,1, 2],
              [1,1, 1],
              [1,1, 0],
              [2,0, 2],
              [1,-1, 1],
            ],
     268 => [ [1,1, 3],
              [2,0, 2],
              [1,-1, 0],
              [1,-1, 1],
              [1,-1, 3],
              [0,-2, 1],
              [-1,-1, 0],
              [0,0, 3],
              [0,0, 2],
            ],
    );
my $surroundings = $surrounding{$type};
{
  my $gpc;
  foreach my $i (0 .. $#$surroundings) {
    my $surrounding = $surroundings->[$i];
    my ($x,$y, $dir) = @$surrounding;
    my $new_points = MyPlanar::points_rotate_plus90($points, $dir);
    $new_points = MyPlanar::points_move($new_points, $x*$scale, $y*$scale);
    ### $union
    ### $new

    my $new_gpc = MyPlanar::points_to_gpc($new_points);

    if (defined $gpc) {
      $gpc = Math::Geometry::Planar::GpcClip('UNION',$gpc,$new_gpc);
      # print "union to ",scalar(@$union),"  ",MyDragon::points_str($union),"\n";
    } else {
      $gpc = $new_gpc;
    }
  }

  my @contours = Math::Geometry::Planar::Gpc2Polygons($gpc);
  my $total_holes = 0;
  foreach my $contour (@contours) {  # Math::Geometry::Planar contour
    ### $contour
    # Second and subsequent polygon of the contour are holes.
    # Think their points already cycle the opposite way as required by pgf
    # default \fill[nonzero rule], but using \fill[even odd rule] works either
    # way.
    $Printwrap = 2;
    printwrap(" \\fill[my grey,even odd rule]");
    $Printwrap += 2;
    my $polygons = $contour->polygons;  # arrayref of points arrayrefs
    $total_holes += scalar(@$polygons)-1;
    my $separator = '';
    foreach my $points (@$polygons) {
      ### $points
      printwrap($separator);
      $separator = "\n";
      printwrap_poly($points);
    }
    printwrap(";\n");
  }


  # $Printwrap = 2;
  # printwrap("\\fill[my grey] ");
  # my $join = '';
  # $Printwrap += 2;
  # foreach my $i (0 .. $#$union) {
  #   printwrap("$join($union->[$i]->[0],$union->[$i]->[1])");
  #   $join = '--';
  # }
  # printwrap("--cycle;\n");
}
print "  \\end{scope}\n";

{
  $Printwrap = 2;
  foreach my $surrounding (@{$surrounding{$type}}) {
    my ($x,$y, $dir) = @$surrounding;
    $dir *=90;
    printwrap("\\pic at ($x,$y) [transform shape,rotate=$dir] {my hull pic};\n");
  }
}

print "  % end generated\n";
exit 0;
