#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'BlobN';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 6;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::DragonCurve->new;

my $n_blob_start = BlobN($level);
my $n_blob_end = BlobN($level+1)-3;

{
  my ($x,$y) = $path->n_to_xy($n_blob_start);
  print "  \\coordinate (start) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_blob_end);
  print "  \\coordinate (end) at ($x,$y) [my dot];\n";
}

my @n_list = ($n_blob_start - .5,
               ($n_blob_start .. $n_blob_end),
              $n_blob_end + .5);
{
  my ($x,$y) = $path->n_to_xy($n_list[0]);
  print "  \\coordinate (line-start) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_list[-1]);
  print "  \\coordinate (line-end) at ($x,$y) [my dot];\n";
}

my $segments = MyLineSegments->new;
foreach my $i (0 .. $#n_list-1)  {
  my ($x1,$y1) = $path->n_to_xy($n_list[$i]);
  my ($x2,$y2) = $path->n_to_xy($n_list[$i+1]);
  $segments->add(2*$x1, 2*$y1, 2*$x2, 2*$y2);
}

$Printwrap = 2;
printwrap("\\draw");
$Printwrap += 2;
$segments->printwrap (transform => sub {
                        my ($x,$y) = @_;
                        return ($x/2, $y/2);
                      });
printwrap(";\n");

print "  % end generated\n";
exit 0;
