#!/usr/bin/perl -w

# Copyright 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'transform_to_horizontal','xy_rotate_plus90','xy_rotate_minus90',
  'BlobN','P3_xy','P8_xy','BP1_xy','BP6_xy';
use MyPrintwrap ':all';
use MyLineSegments;


my $level = 7;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::DragonCurve->new;

sub transform {
  my ($x,$y) = @_;
  ($x,$y) = transform_to_horizontal($x,$y, $level+($level&1));
  return ($x, $y);
}

my $n_lo = BlobN($level) - 1;
my $n_hi = BlobN($level+1) - 2;
my ($bp1_x,$bp1_y) = BP1_xy($level);
my ($bp6_x,$bp6_y) = BP6_xy($level);
my $bp1_n = $path->xy_to_n($bp1_x,$bp1_y);
my $bp6_n = $path->xy_to_n($bp6_x,$bp6_y);

my %xy_to_name;
{
  my ($x,$y) = transform($path->n_to_xy($n_lo));
  print "  \\coordinate (start) at ($x,$y);\n";
}
{
  my ($x,$y) = transform($path->n_to_xy($n_hi));
  print "  \\coordinate (end) at ($x,$y);\n";
}
{
  my ($x,$y) = transform($bp1_x,$bp1_y);
  print "  \\node (BP1) at ($x,$y) [my dot];\n";
  $xy_to_name{"$x,$y"} = 'BP1';
}
{
  my ($x,$y) = transform($bp6_x,$bp6_y);
  print "  \\node (BP6) at ($x,$y) [my dot];\n";
  $xy_to_name{"$x,$y"} = 'BP6';
}

sub want_arrow {
  my ($n) = @_;
  return (($bp1_n - 1 <= $n && $n <= $bp1_n)
          || ($bp6_n - 1 <= $n && $n <= $bp6_n));
}

my $segments = MyLineSegments->new;

$Printwrap = 2;
my $count_arrows = 0;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  if (want_arrow($n)) {
    my $arrow = ($n&1 ? '<-' : '->');
    ($x1,$y1) = transform($x1,$y1);
    ($x2,$y2) = transform($x2,$y2);
    my $xy1 = $xy_to_name{"$x1,$y1"} // "$x1,$y1";
    my $xy2 = $xy_to_name{"$x2,$y2"} // "$x2,$y2";
    printwrap(" \\draw[$arrow] ($xy1)--($xy2);");
    $count_arrows++;
  } else {
    $segments->add($x1,$y1, $x2,$y2);
  }
}
printwrap(" % $count_arrows arrows\n");

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);
printwrap(";\n");

print "  % end generated\n";
exit 0;
