#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'RQ','xy_rotate_minus90';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

# right boundary squares centroid by path


my $max_level = 10;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";

my $path = Math::PlanePath::DragonCurve->new;

sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}

print "% GP-DEFINE  GRQtotal_samples_by_path = {1/2*[\n";

foreach my $k (0 .. $max_level) {
  ### $k
  my $n_start = $path->n_start;
  (my $n_lo, my $n_hi) = $path->level_to_n_range($k);

  my $count = 0;
  my %seen;
  my $x_total = 0;
  my $y_total = 0;
 N: foreach my $n ($n_lo .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my $mx = $x;
    my $my = $y;

    my $untraversed = 0;
    foreach my $i (1 .. 4) {
      ### at: "$i     $x,$y and $dx,$dy"
      if ($x <= $mx && $y <= $my) {
        $mx = $x;
        $my = $y;
      }
      if (! path_xyxy_is_traversed_either($path, $x,$y, $x+$dx,$y+$dy, $n_hi)) {
        $untraversed = 1;
      }
      $x += $dx;
      $y += $dy;
      ($dx,$dy) = xy_rotate_minus90($dx,$dy);
    }
    next unless $untraversed;

    ### key: "$mx,$my"
    next if $seen{"$mx,$my"}++;
    $count++;

    $x_total += $mx + .5;
    $y_total += $my + .5;
  }
  $count == RQ($k) or die "count=$count want RQ($k)=",RQ($k);

  my $gx = $x_total / $count;
  my $gy = $y_total / $count;
  $x_total *= 2;
  $y_total *= 2;
  my $comma = ($k == $max_level ? '' : ',');
  print "% GP-DEFINE    $x_total + I*$y_total$comma   /* k=$k */\n";
}
print "% GP-DEFINE  ]};\n";

print "% end generated\n";
exit 0;
