#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_minus90','xy_rotate_plus90','transform_to_horizontal';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 14;

$level % 2 == 0 or die "level must be even";
print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
my $scale = sqrt($x_hi**2 + $y_hi**2);

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

sub path_n_is_right_boundary {
  my ($path, $n, $n_limit) = @_;
  ### path_n_is_right_boundary(): $n
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}
sub path_n_is_left_boundary {
  my ($path, $n, $n_limit) = @_;
  ### path_n_is_left_boundary(): $n
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}
sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}


$Printwrap = 2;
printwrap("\\begin{scope}[scale=1/$scale]\n");
my $count_segments = 0;
my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  next unless path_n_is_right_boundary($path,$n, $n_hi);
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  ($x1,$y1) = transform($x1,$y1);
  ($x2,$y2) = transform($x2,$y2);
  $segments->add($x1,$y1, $x2,$y2);
  $count_segments++;
}

{
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

$Printwrap = 2;
printwrap("\\end{scope}\n");

my $draw_count = $segments->lines_joined_count_points;

print "  % total boundary segments $count_segments drawn with $draw_count points\n";
print "% GP-Test  R($level) == $count_segments\n";
print "  % end generated\n";
exit 0;







# $Printwrap = 2;
# printwrap("\\begin{scope}[scale=1/$scale]\n");
# my $count = 0;
# foreach my $func (\&path_n_is_right_boundary,
#                   # \&path_n_is_left_boundary,
#                  ) {
#   $Printwrap = 2;
#   printwrap("\\draw ");
#   $Printwrap = 4;
#   my $join = '';
#   foreach my $n ($n_lo .. $n_hi) {
#     next unless $func->($path,$n, $n_hi);
#
#     my ($x,$y) = $path->n_to_xy($n);
#     ($x,$y) = transform($x,$y);
#     printwrap("$join($x,$y)");
#     $join = '--';
#     $count++;
#   }
#   printwrap(";\n");
# }
# $Printwrap = 2;
# printwrap("\\end{scope}\n");
#
# print "  % total points $count\n";
# print "  % end generated\n";
# exit 0;
