#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Lingua::EN::Numbers;
use Math::PlanePath::ComplexMinus 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 5;
if (@ARGV) {
  $level = shift @ARGV;
}
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::ComplexMinus->new;

sub level_to_boundary_segments {
  my ($k) = @_;
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);
  my $segments = MyLineSegments->new;
  my $tx = 0;
  my $ty = 0;
  my $tcount = 0;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    if ($n >= $n_hi/2) {
      $tx += $x;
      $ty += $y;
      $tcount++;
    }

    my ($dx,$dy) = (1,0);
    foreach (0 .. 3) {
      my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
      my $n2 = $path->xy_to_n($x+$dx,$y+$dy);
      if ($n2 > $n_hi) {
        $segments->add(2*$x+$dx -$lx, 2*$y+$dy -$ly,
                       2*$x+$dx +$lx, 2*$y+$dy +$ly);
      }
      ($dx,$dy) = ($lx,$ly);
    }
  }
  my $mx = 2*$tx / $tcount;
  my $my = 2*$ty / $tcount;
  return ($segments, $mx,$my);
}

my ($level_segments) = level_to_boundary_segments($level);
{
  $Printwrap = 2;
  printwrap("\\fill[my level] ");
  $Printwrap += 2;
  $level_segments->printwrap;
  printwrap(";\n");
}

{
  my $segments = MyLineSegments->new;
  foreach my $offset (1 .. 5) {
    my ($bigger_segments,$mx,$my) = level_to_boundary_segments($level+$offset);
    $segments->add_segments($bigger_segments);
    my $name = Lingua::EN::Numbers::num2en($offset);
    print "  \\newcommand\\MyMiddle\u${name}{$mx,$my}\n";
  }
  $segments->remove_segments($level_segments);

  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

print "  % end generated\n";
exit 0;
