#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'sum';

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'RQ';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 6;

print "  % generated by tools/$FindBin::Script\n";

my @RQ_types = ('1e');

my %RQ_types_expand = ('1e' => ['1e','1o'],
                       '1o' => ['2'],
                       '2'  => ['1e','3o'],
                       '3o' => ['3e','3o'],
                       '3e' => ['2']);
foreach my $type (sort keys %RQ_types_expand) {
  print "% GP-Test  RQ_type_expand_vector[RQ_type_$type] == [",
    join(',',map {"RQ_type_$_"} @{$RQ_types_expand{$type}}), "]\n";
}

my @vertex_names;
my %vertex_level;
my %vertex_children;
my %vertex_type;
my %child_to_parent;

$Printwrap = 2;
foreach my $level (0 .. $max_level) {
  my @new_RQ_types;
  foreach my $i (0 .. $#RQ_types) {
    my $name = "k$level-$i";
    push @vertex_names, $name;
    my $type = $RQ_types[$i];
    $vertex_type{$name} = $type;
    $vertex_level{$name} = $level;
    my @child_names;
    if ($level < $max_level) {
      foreach my $child_type (@{$RQ_types_expand{$type}}) {
        my $child_pos = scalar(@new_RQ_types);
        my $child_name = 'k'.($level+1).'-'.$child_pos;
        $child_to_parent{$child_name} = $name;
        push @child_names, $child_name;
        push @new_RQ_types, $child_type;
      }
      $vertex_children{$name} = \@child_names;
    }
  }
  @RQ_types = @new_RQ_types;
  if ($level < $max_level) {
    print "% GP-Test  RQ_type_list(",$level+1,") == [",
      join(',',map {"RQ_type_$_"} @RQ_types),
      "]\n";
  }
}

my %vertex_pos;
{
  # positions of bottom row $RQ_pos decrementing,
  # others midway between their children
  my $RQ_pos = RQ($max_level) - 1;
  foreach my $i (reverse 0 .. $#vertex_names) {
    my $name = $vertex_names[$i];
    my $pos;
    if (my $aref = $vertex_children{$name}) {
      my @child_poses = map {$vertex_pos{$_}} @$aref;
      ### children: @$aref
      ### @child_poses
      $pos = sum(@child_poses) / scalar(@child_poses);  # midway

      # single child bias to the left
      my $right_name = $vertex_names[$i+1];
      my $right_aref = $vertex_children{$right_name};
      if ($right_aref && @$right_aref == 1
          && $vertex_level{$right_name} < $max_level-2
          && $child_to_parent{$right_name} eq $child_to_parent{$name}) {
        $vertex_pos{$right_name} = $pos + ($vertex_pos{$right_name} - $pos) * .85;
      }

    } else {
      $pos = $RQ_pos--;
    }
    ### $name
    ### $pos
    $vertex_pos{$name} = $pos;
  }
}

foreach my $name (@vertex_names) {
  my $pos = "$vertex_pos{$name}";
  $pos =~ s/(\.\d\d\d\d)(\d+)/$1/;
  printwrap(" \\node ($name) at ($pos, -$vertex_level{$name}) {$vertex_type{$name}};");
}
printwrap_newline();

printwrap(" \\draw");
$Printwrap += 2;
foreach my $name (@vertex_names) {
  foreach my $child_name (@{$vertex_children{$name}}) {
    printwrap(" ($name)--($child_name)");
  }
}
printwrap(";\n");

$Printwrap = 2;
foreach my $level (0 .. $max_level) {
  printwrap(" \\node at (column |- 0,-$level) [left] {\$k{=}$level\$};");
}
printwrap_newline();

{
  my @RQ_vpar;
  my %vertex_to_v = map {$vertex_names[$_] => $_+1} 0 .. $#vertex_names;
  foreach my $i (0 .. $#vertex_names) {
    my $name = $vertex_names[$i];
    my $comma = ($i < $#vertex_names ? ',' : '');
    my $parent = $child_to_parent{$name};
    my $p = (defined $parent ? $vertex_to_v{$parent} : 0);
    push @RQ_vpar, $p;
  }

  my $w = 0;
  print "% GP-Test  RQ_type_vpar(",$max_level+1,") == [ \\\n% GP-Test    ";
  foreach my $i (0 .. $#RQ_vpar) {
    if ($w > 20) {
      print " \\\n% GP-Test    ";
      $w = 0;
    }
    print $RQ_vpar[$i], $i==$#RQ_vpar ? '' : ',';
    $w++;
  }
  print "]\n";
}

print "  % end generated\n";
exit 0;
