#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'mul','div','chamfer','xy_rotate_plus90';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
### level: "$level  n_hi=$n_hi"

sub xyxy_is_twindragon_touch {
  my ($x,$y) = @_;
  ### xyxy_is_twindragon_touch(): "$x, $y"

  my @n_list = ($path->xy_to_n_list($x,$y),
                $path->xy_to_n_list($x_hi-$x,$y_hi-$y));
  @n_list = grep {$_ < $n_hi} @n_list;
  return @n_list > 1;
}

# twindragon drawing
{
  $Printwrap = 2;
  printwrap("\\draw[my dragon] ");
  my $join = '';
  $Printwrap = 4;
  foreach my $rev (0,1) {
    foreach my $n ($n_lo .. $n_hi-1) {

      my ($x,$y) = $path->n_to_xy($n);
      if ($rev) {
        ($x,$y) = ($x_hi-$x, $y_hi-$y);
      }

      if (xyxy_is_twindragon_touch($x,$y)) {
        my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
        my ($tdx,$tdy) = $path->n_to_dxdy($n);
        if ($rev) {
          ($sdx,$sdy) = (-$sdx,-$sdy); # rotate 180
          ($tdx,$tdy) = (-$tdx,-$tdy); # rotate 180
        }
        my $sx = $x . chamfer(-$sdx);
        my $sy = $y . chamfer(-$sdy);
        my $tx = $x . chamfer($tdx);
        my $ty = $y . chamfer($tdy);
        printwrap("$join($sx,$sy)");
        printwrap("--($tx,$ty)");
      } else {
        printwrap("$join($x,$y)");
      }
      $join = '--';
    }
  }
  printwrap("--cycle;\n");
}

$Printwrap = 2;
{
  my ($x,$y) = $path->n_to_xy($n_lo);
  printwrap("\\node (S) at ($x,$y) [my dot];\n");
}
{
  my ($x,$y) = $path->n_to_xy($n_hi*3/4);
  printwrap("\\node (M) at ($x,$y) [my dot];\n");
}
{
  my ($x,$y) = $path->n_to_xy($n_hi/2);
  ($x,$y) = ($x_hi-$x, $y_hi-$y);
  printwrap("\\node (C) at ($x,$y) [my dot];\n");
}
{
  my ($x,$y) = $path->n_to_xy($n_hi*7/8);
  printwrap("\\node (L) at ($x,$y) [my dot];\n");
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  printwrap("\\node (E) at ($x,$y) [my dot];\n");
}

print "  % end generated\n";
exit 0;
