#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'BlobN','transform_to_horizontal',
  'points_to_hull','blob_hull_points_sort',
  'BP1_xy','BP2_xy','BP3_xy','BP4dash_xy','BP4_xy',
  'BP5_xy','BP6_xy','BP7_xy','BP8dash_xy','BP8_xy',
  'BP9_xy','BP10_xy';
use MyLineSegments;
use MyPrintwrap ':all';


my $k = 4;
if (@ARGV) { $k = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $k\n";

my $path = Math::PlanePath::DragonCurve->new;

my @BP_funcs = (['BP1', \&BP1_xy ],
                ['BP2', \&BP2_xy ],
                ['BP3', \&BP3_xy ],
                ['BP4dash', \&BP4dash_xy ],
                ['BP4', \&BP4_xy ],
                ['BP5', \&BP5_xy ],
                ['BP6', \&BP6_xy ],
                ['BP7', \&BP7_xy ],
                ['BP8dash', \&BP8dash_xy ],
                ['BP8', \&BP8_xy ],
                ['BP9', \&BP9_xy ],
                ['BP10', \&BP10_xy ]);

my $transform = sub {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $k);
};

my $blob_n_lo = BlobN($k);
my $blob_n_hi = BlobN($k+1)-3;

my $segments = MyLineSegments->new;
my $points = [ ];
foreach my $n ($blob_n_lo .. $blob_n_hi-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $segments->add($x1,$y1, $x2,$y2);
  push @$points, [$x2,$y2];
}
my $scale = ($k % 2 == 0 ? '' : ',scale={1/sqrt(2)},');
$Printwrap = 2;
printwrap("\\begin{scope}[shift=(pos-$k)$scale]\n");
$Printwrap += 2;
printwrap("\\draw[my grey] ");
$Printwrap += 2;
$segments->printwrap (transform => $transform);
printwrap(";\n");

$points = points_to_hull($points);
$points = blob_hull_points_sort($points, $k);

my @BP_points = map {[$BP_funcs[$_]->[1]->($k)]} 0 .. $#BP_funcs;
my @BP_seen;

$Printwrap -= 2;
my $num = 1;
foreach my $p (@$points) {
  my ($x,$y) = @$p;

  my @BP_names;
  foreach my $i (0 .. $#BP_funcs) {
    my ($fx,$fy) = @{$BP_points[$i]};
    if ($x == $fx && $y == $fy) {
      push @BP_names, $BP_funcs[$i]->[0];
      $BP_seen[$i] = 1;
    }
  }

  my $name = join('-', @BP_names);
  if ($name) {
    $name = " ($name-$k)";
  }
  my ($tx,$ty) = $transform->($x,$y);
  printwrap("\\node$name at ($tx,$ty) [my dot];\n");
  $num++;
}
foreach my $i (0 .. $#BP_funcs) {
  my $BP_name = $BP_funcs[$i]->[0];
  my ($fx,$fy) = @{$BP_points[$i]};
  my $seen = ($BP_seen[$i] ? '' : '  /* not */');
    print "% GP-Test  $BP_name($k) == $fx + I*$fy$seen\n";
  }

  {
    my ($x,$y) = $path->n_to_xy($blob_n_lo);
    ($x,$y) = $transform->($x,$y);
    print "  \\coordinate (start-$k) at ($x,$y);\n";
  }
{
  my ($x,$y) = $path->n_to_xy($blob_n_hi);
  ($x,$y) = $transform->($x,$y);
  print "  \\coordinate (end-$k) at ($x,$y);\n";
}
print "  \\end{scope}\n";

print "  % end generated\n";
exit 0;
