#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'BlobN','BlobSegments','MidSN','MidEN';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 15;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;

my $n_blob_start = BlobN($level);
my $n_blob_end   = BlobN($level+1) - 3;
my $n_lo = $n_blob_start - 1;  # one segment extra each end
my $n_hi = $n_blob_end   + 1;
my $MidSN = MidSN($level);
my $MidEN = MidEN($level);
my $first_hi  = $MidSN + 1;
my $second_lo = $MidEN - 1;

{
  my ($x,$y) = $path->n_to_xy($n_lo);
  print "  \\coordinate (start) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($first_hi);
  print "  \\coordinate (middle-1) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($second_lo);
  print "  \\coordinate (middle-2) at ($x,$y);\n";
}

{
  my $segments = MyLineSegments->new;
  my $len = 0;
  for (my $k = $level-4; $k >= 4; $k -= 2) {
    $len += BlobSegments($k) + 3;
  }
  # print "  % lo $n_lo hi $n_hi len $len\n";

  foreach my $n ($n_lo .. $first_hi-1) {
    $segments->add($path->n_to_xy($n),
                   $path->n_to_xy($n+1));
  }
  foreach my $n ($second_lo .. $n_hi-1) {
    $segments->add($path->n_to_xy($n),
                   $path->n_to_xy($n+1));
  }

  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

print "% GP-Test-Last  MidSN($level) == $MidSN\n";
print "% GP-Test-Last  MidEN($level) == $MidEN\n";

print "  % end generated\n";
exit 0;
