#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min','max';
use Lingua::EN::Numbers;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 7;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my $n_lo = BlobN($level);
my $n_hi = BlobN($level+1) - 3;

my @count;
foreach my $want_count (1,2) {
  my %seen;
  my $x_min = 0;
  my $x_max = 0;
  my $count = 0;
  print "  \\begin{scope}[shift=(pos-$want_count)]\n";

  $Printwrap = 4;
  printwrap("\\fill foreach \\p in {");

  $Printwrap = 6;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    next if $seen{"$x,$y"}++;

    my @n_list = $path->xy_to_n_list($x,$y);
    @n_list = grep {$_ <= $n_hi} @n_list;
    my $c = scalar(@n_list);
    $c == 1 || $c == 2 || die;

    if ($c == $want_count) {
      printwrap("($x,$y),");
      $count++;
    }
    $x_max = max($x_max, $x);
    $x_min = min($x_min, $x);
  }
  printwrap("}");
  printwrap(" { \\p circle[] };\n");

  print "    \\coordinate (x-min-$want_count) at ($x_min,0);\n";
  print "    \\coordinate (x-max-$want_count) at ($x_max,0);\n";
  print "  \\end{scope}\n";

  my $name = Lingua::EN::Numbers::num2en($want_count);
  print "  \\newcommand{\\MyCount\u$name}{$count}\n";
  $count[$want_count] = $count;
}
print "% GP-Test-Last  BlobS($level)==$count[1]\n";
print "% GP-Test-Last  BlobD($level)==$count[2]\n";

print "  % end generated\n";
exit 0;
