#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 6;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $x_total = 0;
my $y_total = 0;
my $weight_total;

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

{
  my ($x,$y) = $path->n_to_xy($n_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
  # my $str = xy_to_tex_str($x, $y);
  # print "  \\newcommand{\\MyEnd}{$str}\n";
}

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $segments->add($x1,$y1, $x2,$y2);
  $x_total += $x1 + $x2;
  $y_total += $y1 + $y2;
  $weight_total += 2;
}

{
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";");
  printwrap("  % " . $segments->lines_joined_count_points . " points\n");
}

while ($weight_total
       && ($weight_total % 2) == 0
       && ($x_total % 2) == 0
       && ($y_total % 2) == 0) {
  $x_total /= 2;
  $y_total /= 2;
  $weight_total /= 2;
}
print "  \\node (GS) at ($x_total/$weight_total,$y_total/$weight_total) [my dot];\n";
{
  my $str = xy_to_tex_str($x_total, $y_total);
  print "  \\newcommand{\\MyGS}{\\dfrac{$str}{$weight_total}}\n";
}


print "% GP-Test-Last  GS($level) == $x_total/$weight_total+$y_total/$weight_total*I\n";
print "  % end generated\n";
exit 0;
