#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::ComplexMinus 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyLineSegments;
use MyDragon 'xy_rotate_minus90','transform_to_horizontal',
  '@dir4_to_dx','@dir4_to_dy';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


# prefer an odd level here so squares are horizontal when curve rotated to
# endpoints horizontal
my $level = 14;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  my complex minus fractal pic/.pic={\n";

my $path = Math::PlanePath::ComplexMinus->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
my $scale = 2**($level/2+1); # +1 for *2 on coordinates
### $n_lo
### $n_hi

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, 3*$level);   # 3* for I-1 at 135deg
}

$Printwrap = 2;
printwrap("\\begin{scope}[scale=1/$scale]\n");
my $count_boundary = 0;
my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi) {
  ### $n
  my ($x,$y) = $path->n_to_xy($n);
  foreach my $dir4 (0 .. 3) {
    my $dx = $dir4_to_dx[$dir4];
    my $dy = $dir4_to_dy[$dir4];
    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
    my $n_adjacent = $path->xy_to_n($x+$dx,$y+$dy);
    ### side: "dir=$dir4  $dx,$dy  $rx,$ry  adjacent $n_adjacent"
    if ($n_adjacent <= $n_hi) {
      ### non-boundary side ...
      next;
    }
    my $x1 = 2*$x + $dx - $rx;
    my $y1 = 2*$y + $dy - $ry;
    my $x2 = 2*$x + $dx + $rx;
    my $y2 = 2*$y + $dy + $ry;
    $segments->add ($x1,$y1, $x2,$y2);
    $count_boundary++;
  }
}

$Printwrap = 2;
printwrap("\\path[pic actions] ");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);
printwrap(";\n");

$Printwrap = 2;
printwrap("\\end{scope}\n");

my $draw_count = $segments->lines_joined_count_points;

print "  % total boundary segments $count_boundary drawn with $draw_count points\n";
print "% GP-Test-Last  ComplexBoundary($level) == $count_boundary\n";
print "  },\n";
print "  % end generated\n";
exit 0;
