#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::Geometry::Planar;
use Math::PlanePath::ComplexPlus;

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 14;

print "% generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::ComplexPlus->new;

my $var = $FindBin::Script;
$var =~ s/\.pl$//;
$var =~ tr/-/_/;
print "% GP-DEFINE  $var = {[\n";

sub xy_to_expression {
  my ($x,$y) = @_;

  if ($x == 0 && $y == 0) {
    return '0';
  }
  my $str = '';
  if ($x) {
    $str = $x;
  }
  if ($y) {
    if ($y > 0 && length($str)) { $str .= '+'; }
    if ($y < 0) { $str .= '-'; $y = -$y; }
    if ($y != 1) { $str .= "$y*"; }
    $str .= 'I';
  }
  return $str;
}

my @area;
foreach my $k (0 .. $max_level) {
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  my $points = [];
  foreach my $n ($n_lo .. $n_hi) {
    push @$points, [$path->n_to_xy($n)];
  }
  my $area = 0;
  if (@$points >= 3) {
    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    if (@$points >= 5) {
      $planar = $planar->convexhull2;
      $points = $planar->points;
      ### $points
    } else {
      # k=2 sort points so crossing sides don't cancel out area
      $points = hull_points_sort_by_angle ($points, $x_hi,$y_hi);
      $planar->points($points);
    }
    $area = $planar->area;
  }
  push @area, $area;
  $points = hull_points_sort_by_angle ($points, $x_hi,$y_hi);
  push @$points, shift @$points;

  print "% GP-DEFINE    /* $k */ [ ";
  my $join = '';
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    print $join, xy_to_expression($x,$y);
    $join = ',';
  }
  print " ]", ($k<$max_level ? "," : ""), "\n";
}

print "% GP-DEFINE  ]};\n";
print "% GP-DEFINE  ${var}_area = [ ",join(', ',@area)," ];\n";

print "% end generated\n";
exit 0;
