#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'TurnLpred_flat', 'TurnRpred_flat',
  '@dTurnLeft_state_table', '@dTurnLeft_state_to_name',
  '@dTurnLeft_state_to_d', '@dTurnLeft_state_to_decr',
  'transmute_initial_0s_to_1s',
  '@dTurnRight_state_table', '@dTurnRight_state_to_name',
  '@dTurnRight_state_to_d', '@dTurnRight_state_to_incr',
  'transmute_lowest_1_to_0',

  # 'TurnLpred_flat','TurnRpred_flat',
  # '@dNextL_state_table',
  # '@dNextL_state_to_name', '@dNextL_state_to_d',
  # '@dNextL_LtoH_state_table',
  # '@dNextL_LtoH_state_to_name', '@dNextL_LtoH_state_to_d',
  # '@dNextLsub1_state_table',
  # '@dNextLsub1_state_to_name', '@dNextLsub1_state_to_d',
  # '@dNextLsub1_M_state_table',
  # '@dNextLsub1_M_state_to_name', '@dNextLsub1_M_state_to_d',
  # '@dNextR_state_table',
  # '@dNextR_state_to_name','@dNextR_state_to_d','$dNextR_starting'
  ;
use MyPrintwrap ':all';
use MyFLAT;
$|=1;

print "% generated by tools/$FindBin::Script\n";

{
  foreach my $i (0 .. $#dTurnLeft_state_to_name) {
    print "% GP-DEFINE  dTurnLeft_$dTurnLeft_state_to_name[$i] = ",$i+1,";\n";
  }
  my $prefix = "dTurnLeft_state_table = {[";
  foreach my $state (0 .. $#dTurnLeft_state_table) {
    my $semi = $state == $#dTurnLeft_state_table ? ' ]};' : ';   ';
    printf "%% GP-DEFINE  %s %2d,%2d%s  \\\\ %2d %s\n",
      $prefix,
      $dTurnLeft_state_table[$state]->[0] + 1,
      $dTurnLeft_state_table[$state]->[1] + 1,
      $semi,
      $state + 1,
      $dTurnLeft_state_to_name[$state];
    $prefix = ' ' x length($prefix);
  }

  printwrap_indent("% GP-DEFINE  ");
  printwrap_gp_vector([map {defined ? $_ : "'none"} @dTurnLeft_state_to_d],
                      "dTurnLeft_state_to_d");
  printwrap_gp_vector(\@dTurnLeft_state_to_decr, "dTurnLeft_state_to_decr");


  # $T is all of TurnLpred strings but transmuted per dTurnLeft nseen
  my $T = TurnLpred_flat();
  my $T_even = $T->intersect(FLAT::Regex->new('(0|1)* 0 | []')->as_dfa);
  $T_even = $T_even->MyFLAT::transmute (func => \&transmute_initial_0s_to_1s,
                                        direction => 'reverse');
  my $T_odd = $T->intersect(FLAT::Regex->new('(0|1)* 1')->as_dfa);
  $T_odd = $T_odd->MyFLAT::digits_increment(add => -1)
    ->MyFLAT::optional_leading_0s;
  $T = $T_even->union($T_odd)->as_dfa;

  my @is_final;
  foreach my $state (0 .. $#dTurnLeft_state_table) {
    my $f = MyFLAT::aref_to_FLAT_DFA
      (\@dTurnLeft_state_table,
       name => "dTurnLeft state $state",
       accepting => $state);
    $is_final[$state] = $f->intersect($T)->is_empty ? 0 : 1;
  }
  printwrap_gp_vector(\@is_final, "dTurnLeft_is_final");

  print "%\n";
}
{
  foreach my $i (0 .. $#dTurnRight_state_to_name) {
    print "% GP-DEFINE  dTurnRight_$dTurnRight_state_to_name[$i] = ",$i+1,";\n";
  }
  my $prefix = "dTurnRight_state_table = {[";
  foreach my $state (0 .. $#dTurnRight_state_table) {
    my $semi = $state == $#dTurnRight_state_table ? ' ]};' : ';   ';
    printf "%% GP-DEFINE  %s %d,%d%s  \\\\ %d %s\n",
      $prefix,
      $dTurnRight_state_table[$state]->[0] + 1,
      $dTurnRight_state_table[$state]->[1] + 1,
      $semi,
      $state + 1,
      $dTurnRight_state_to_name[$state];
    $prefix = ' ' x length($prefix);
  }

  printwrap_indent("% GP-DEFINE  ");
  printwrap_gp_vector([map {defined ? $_ : "'none"} @dTurnRight_state_to_d],
                      "dTurnRight_state_to_d");
  printwrap_gp_vector(\@dTurnRight_state_to_incr, "dTurnRight_state_to_incr");


  # $T is all of TurnRpred strings but transmuted per dTurnRight nseen
  my $T = TurnRpred_flat();
  my $T_even = $T->intersect(FLAT::Regex->new('(0|1)* 0 | []')->as_dfa);
  $T_even = $T_even->MyFLAT::transmute  (func => \&transmute_lowest_1_to_0,
                                         direction => 'reverse')
    ->MyFLAT::optional_leading_0s;
  my $T_odd = $T->intersect(FLAT::Regex->new('(0|1)* 1')->as_dfa);
  $T = $T_even->union($T_odd)->as_dfa;

  my @is_final;
  foreach my $state (0 .. $#dTurnRight_state_table) {
    my $f = MyFLAT::aref_to_FLAT_DFA
      (\@dTurnRight_state_table,
       name => "dTurnRight state $state",
       accepting => $state);
    $is_final[$state] = $f->intersect($T)->is_empty ? 0 : 1;
  }
  printwrap_gp_vector(\@is_final, "dTurnRight_is_final");
}

print "% end generated\n";
exit 0;

__END__

#-----------------------------------------------------------------------------
# {
#   foreach my $i (0 .. $#dNextL_state_to_name) {
#     print "% GP-DEFINE  dNextL_$dNextL_state_to_name[$i] = ",$i+1,";\n";
#   }
#   printwrap_indent("% GP-DEFINE  ");
#   printwrap_gp_matrix([map{ [map {$_+1} @$_] } @dNextL_state_table],
#                       "dNextL_state_table");
# 
#   printwrap_gp_vector(\@dNextL_state_to_d, "dNextL_state_to_d");
# 
#   scalar(@dNextL_state_to_d) == scalar(@dNextL_state_table) or die;
#   scalar(@dNextL_state_to_name) == scalar(@dNextL_state_table) or die;
# }

# {
#   foreach my $i (0 .. $#dNextL_LtoH_state_to_name) {
#     print "% GP-DEFINE  dNextL_LtoH_$dNextL_LtoH_state_to_name[$i] = ",$i+1,";\n";
#   }
#   printwrap_indent("% GP-DEFINE  ");
#   printwrap_gp_matrix([map{ [map {$_+1} @$_] } @dNextL_LtoH_state_table],
#                       "dNextL_LtoH_state_table");
# 
#   printwrap_gp_vector(\@dNextL_LtoH_state_to_d, "dNextL_LtoH_state_to_d");
# 
#   scalar(@dNextL_LtoH_state_to_d) == scalar(@dNextL_LtoH_state_table) or die;
#   scalar(@dNextL_LtoH_state_to_name) == scalar(@dNextL_LtoH_state_table) or die;
# }

# {
#   foreach my $i (0 .. $#dNextLsub1_state_to_name) {
#     print "% GP-DEFINE  dNextLsub1_$dNextLsub1_state_to_name[$i] = ",$i+1,";\n";
#   }
#   printwrap_indent("% GP-DEFINE  ");
#   printwrap_gp_matrix([map{ [map {$_+1} @$_] } @dNextLsub1_state_table],
#                       "dNextLsub1_state_table");
# 
#   printwrap_gp_vector(\@dNextLsub1_state_to_d, "dNextLsub1_state_to_d");
# 
#   scalar(@dNextLsub1_state_to_d) == scalar(@dNextLsub1_state_table) or die;
#   scalar(@dNextLsub1_state_to_name) == scalar(@dNextLsub1_state_table) or die;
# 
#   my @is_final;
#   my @non_lefts;
#   my $next_Lpred = TurnLpred_flat()->MyFLAT::digits_increment(add=>-1)
#     ->MyFLAT::optional_leading_0s;
#   foreach my $state (0 .. $#dNextLsub1_state_table) {
#     my $f = MyFLAT::aref_to_FLAT_DFA
#       (\@dNextLsub1_state_table,
#        name => "dNextLsub1 state $state",
#        starting => 0,
#        accepting => $state);
#     $is_final[$state] = $f->intersect($next_Lpred)->is_empty ? 0 : 1;
#   }
#   printwrap_gp_vector(\@is_final, "dNextLsub1_is_final");
# }

# {
#   foreach my $i (0 .. $#dNextLsub1_M_state_to_name) {
#     print "% GP-DEFINE  dNextLsub1_M_$dNextLsub1_M_state_to_name[$i] = ",$i+1,";\n";
#   }
#   printwrap_indent("% GP-DEFINE  ");
#   printwrap_gp_matrix([map{ [map {$_+1} @$_] } @dNextLsub1_M_state_table],
#                       "dNextLsub1_M_state_table");
# 
#   printwrap_gp_vector([map {defined ? $_ : "'undef"} @dNextLsub1_M_state_to_d],
#                       "dNextLsub1_M_state_to_d");
# 
#   scalar(@dNextLsub1_M_state_to_d) == scalar(@dNextLsub1_M_state_table) or die;
#   scalar(@dNextLsub1_M_state_to_name) == scalar(@dNextLsub1_M_state_table) or die;
# 
#   my @is_final;
#   my $next_Lpred = TurnLpred_flat()->MyFLAT::digits_increment(add=>-1)
#     ->MyFLAT::optional_leading_0s;
#   foreach my $state (0 .. $#dNextLsub1_M_state_table) {
#     my $f = MyFLAT::aref_to_FLAT_DFA
#       (\@dNextLsub1_M_state_table,
#        name => "dNextLsub1 state $state",
#        starting => 0,
#        accepting => $state);
#     $is_final[$state] = $f->intersect($next_Lpred)->is_empty ? 0 : 1;
#   }
#   printwrap_gp_vector(\@is_final, "dNextLsub1_M_is_final");
# }

# {
#   foreach my $i (0 .. $#dNextR_state_to_name) {
#     print "% GP-DEFINE  dNextR_$dNextR_state_to_name[$i] = ",$i+1,";\n";
#   }
#   printwrap_indent("% GP-DEFINE  ");
#   printwrap_gp_matrix([map{ [map {$_+1} @$_] } @dNextR_state_table],
#                       "dNextR_state_table");
# 
#   printwrap_gp_vector(\@dNextR_state_to_d, "dNextR_state_to_d");
# 
#   scalar(@dNextR_state_to_d) == scalar(@dNextR_state_table) or die;
#   scalar(@dNextR_state_to_name) == scalar(@dNextR_state_table) or die;
# 
#   my @is_final;
#   foreach my $state (0 .. $#dNextR_state_table) {
#     my $f = MyFLAT::aref_to_FLAT_DFA
#       (\@dNextR_state_table,
#        name => "dNextR state $state",
#        starting => $dNextR_starting,
#        accepting => $state);
#     $is_final[$state] = $f->intersect(TurnRpred_flat())->is_empty ? 0 : 1;
#   }
#   print "% GP-DEFINE  dNextR_starting = ",$dNextR_starting+1,";\n";
#   printwrap_gp_vector(\@is_final, "dNextR_is_final");
# }
