#!/usr/bin/perl -w

# Copyright 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use FindBin;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon '@dir4_to_dx','@dir4_to_dy',
  'xy_rotate_minus90','xy_rotate_plus90','chamfer','turn','BlobN',
  'Epred','EpredR','EpredL';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

my $n_lo = 701;
my $n_hi = 709;
my $dash_n_start = BlobN(10) - 2;

my $path = Math::PlanePath::DragonCurve->new;

$Printwrap = 2;

my ($x_lo,$y_lo) = $path->n_to_xy($n_lo);
print "  \\coordinate (start) at ($x_lo,$y_lo);\n";
{
  my ($x_lo,$y_lo) = $path->n_to_xy($dash_n_start);
  print "  \\coordinate (dash-start) at ($x_lo,$y_lo);\n";
}

my @EpredL;
my @EpredR;
{
  # grey squares

  $Printwrap = 2;
  printwrap("\\fill[my squares]");

  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    Epred($n) or die "oops, N=$n not Epred";
    push @EpredL, EpredL($n) ? 1 : 0;
    push @EpredR, EpredR($n) ? 1 : 0;

    my $rotate = (EpredL($n) ? \&xy_rotate_plus90 : \&xy_rotate_minus90);

    my ($x2,$y2) = ($x+$dx, $y+$dy);        # segment end
    my ($rdx,$rdy) = $rotate->($dx,$dy);    # rotated

    my ($xr,$yr)   = ($x +$rdx, $y +$rdy);
    my ($xr2,$yr2) = ($x2+$rdx, $y2+$rdy);

    $x .= chamfer($dx,'\MySquare');
    $y .= chamfer($dy,'\MySquare');
    $x2 .= chamfer(-$dx,'\MySquare');
    $y2 .= chamfer(-$dy,'\MySquare');

    $xr .= chamfer($dx-2*$rdx,'\MySquare');
    $yr .= chamfer($dy-2*$rdy,'\MySquare');
    $xr2 .= chamfer(-$dx-2*$rdx,'\MySquare');
    $yr2 .= chamfer(-$dy-2*$rdy,'\MySquare');

    printwrap_newline();
    $Printwrap = 4;
    printwrap("($x,$y)");
    $Printwrap = 6;
    printwrap("--($x2,$y2)");
    printwrap("--($xr2,$yr2)");
    printwrap("--($xr,$yr)");
    printwrap("--cycle");
  }
  printwrap(";\n");
}

my $start_from;
{
  # segments which enclose, thick arrows

  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my ($x2,$y2) = ($x+$dx, $y+$dy);
    $segments->add($x,$y, $x2,$y2);

    my $cx = chamfer($dx);
    my $cy = chamfer($dy);
    my $cx2 = chamfer(-$dx);
    my $cy2 = chamfer(-$dy);
    my $from = "$x$cx,$y$cy";
    my $to = "$x2$cx2,$y2$cy2";

    printwrap_newline();
    $Printwrap = 2;
    printwrap("\\draw[->,my segment]($from)");
    $Printwrap = 4;
    printwrap("--($to);");
    $start_from //= $from;
  }
  printwrap_newline();
}
{
  # chamfered connections between segments which enclose

  $Printwrap = 2;
  printwrap("\\draw[my segment join]");
  $Printwrap = 4;
  foreach my $n ($n_lo+1 .. $n_hi+1) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my ($prev_dx,$prev_dy) = $path->n_to_dxdy($n-1);

    my $cx = chamfer($dx);
    my $cy = chamfer($dy);
    my $prev_cx = chamfer(-$prev_dx);
    my $prev_cy = chamfer(-$prev_dy);
    printwrap(" ($x$prev_cx,$y$prev_cy)");
    printwrap("--($x$cx,$y$cy)");

    if ($n == $n_hi+1) {
      printwrap("-- +($dx*\\MySegmentEnd,$dy*\\MySegmentEnd)");
      printwrap(" coordinate[name=end]");
    }
  }
  printwrap(";\n");
}

# within $n_hi
sub path_is_traversed {
  my ($x1,$y1, $x2,$y2) = @_;
  my $n = $path->xyxy_to_n_either ($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_hi;
}

{
  my $segments = MyLineSegments->new;
  foreach my $n ($dash_n_start .. $n_lo-1) {
    $segments->add ($path->n_to_xy($n), $path->n_to_xy($n+1));
  }
  my ($x_end,$y_end) = $path->n_to_xy($n_lo);
  $Printwrap = 2;
  printwrap("\\draw[dashed] ");
  $Printwrap +=2;
  $segments->printwrap (stringize => sub {
                          my ($x,$y) = @_;
                          if ($x==$x_end && $y==$y_end) {
                            return "($x+\\MyChamfer,$y)";
                          } else {
                            return undef;
                          }
                        });
  printwrap(";\n");
}

print "% GP-Test  vector(",scalar(@EpredL),",n,n+=",$n_lo-1,
  ";EpredL(n)) == [",join(',',@EpredL),"]\n";
print "% GP-Test  vector(",scalar(@EpredR),",n,n+=",$n_lo-1,
  ";EpredR(n)) == [",join(',',@EpredR),"]\n";

print "% GP-Test  Epred(",$n_lo-1,") == 0\n";
print "% GP-Test  Epred($n_lo) == 1\n";
print "% GP-Test  Epred($n_hi) == 1\n";
print "% GP-Test  Epred(",$n_hi+1,") == 0\n";
print "  % end generated\n";
exit 0;
