#!/usr/bin/perl -w

# Copyright 2019, 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'ffixed_u','ffixed_uDepth',
  'ffixed_du','ffixed_du_parent','ffixed_row_w';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

my $max_depth = 8;

print "  % generated by tools/$FindBin::Script\n";

my $segments = MyLineSegments->new;

sub depth_to_y {
  my ($depth) = @_;
  return -$depth;
}

my %du_location;
foreach my $depth (1 .. $max_depth) {
  my $m_start = ffixed_row_w($depth-1);
  my $m_end   = ffixed_row_w($depth);
  my $y = depth_to_y($depth);
  my $factor = 2**($max_depth - $depth);
  my $u_start = ffixed_u($m_start);
  ### row: "depth=$depth, m = $m_start to $m_end, factor $factor"

  foreach my $m ($m_start .. $m_end) {
    my $u = ffixed_u($m);
    my $x = ($u - $u_start) * $factor;
    ### segment: "m=$m u=$u at $x,$y"
    $segments->add($x,$y, $x,$y+1);

    if ($m < $m_end) {
      my $next_u = ffixed_u($m+1);
      my $next_x = ($next_u - $u_start) * $factor;
      my $du_x = ($x + $next_x)/2;
      my $du_y = $y + .5;
      ### du: "next u $next_u  $du_x,$du_y"
      $du_location{$m} = "$du_x,$du_y";

      if (! defined(ffixed_du_parent($m))) {
        $segments->add($x,$y+1, $next_x,$y+1);
      }
    }
  }
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
my @coords;
my $last_row_m = ffixed_row_w($max_depth-1);
foreach my $m (1 .. ffixed_row_w($max_depth)-1) {
  my $node = "du$m";
  my $du = ffixed_du($m);
  if (0 && $du == 4 && ffixed_uDepth($m) == $max_depth) {
    # coordinate only at the smallest bits
    push @coords, "\\coordinate ($node) at ($du_location{$m});";
  } else {
    my $contents = "$du";
    if ($m==1) { $contents = "$du \\mathrlap{{}=\\mathit{du}_1}"; }
    if ($m == $last_row_m+1) {
      printwrap_newline();
      print "  \\begin{scope}[font=\\tiny]\n";
    }
    if ($m >= $last_row_m+1 && ($du==12 || $du==16)) {
      $du-=10;
      $contents = "1 \\mkern-2mu $du \\mkern1.5mu";
    }
    printwrap(" \\node ($node) at ($du_location{$m}) {\$$contents\$};");
  }
}
printwrap_newline();
foreach my $str (@coords) {
  print "  $str\n";
}
printwrap_newline();

print "  \\end{scope}\n";
print "  % end generated\n";
exit 0;
