#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min','max';

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'bitlength';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $target_k = 50;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyK}{$target_k}\n";

my @w = (1);
while ($#w < $target_k) {
  push @w, (3*$w[-1] + ($w[-1] & 1)) / 2;
}
$#w == $target_k or die;
print "% GP-Test  vector(",scalar(@w),",k,k--; ffixed_row_w(k)) == \\\n";
print "% GP-Test    [",join(',',@w),"]\n";

# bit length of w[k]
my @w_num_bits = map {bitlength($_)} @w;
print "% GP-Test  vector(",scalar(@w_num_bits),
  ",k,k--; #binary(ffixed_row_w(k))) == \\\n";
print "% GP-Test    [",join(',',@w_num_bits),"]\n";

# limit k-j+1
my @limit_bits = map { $target_k - $_ + 1 } 0 .. $target_k;
$limit_bits[$target_k] == 1 or die;

# actual bits needed with limit applied
my @actual_bits = map {min($w_num_bits[$_],$limit_bits[$_])} 0 .. $target_k;
my $peak_bits = max(@actual_bits);
### @actual_bits

# those k where w[k] equals $peak_bits
my @peak_k_list;
foreach my $k (0 .. $target_k) {
  if ($w_num_bits[$k] == $peak_bits) { push @peak_k_list, $k; }
}
### @peak_k_list

print "  % peak bits $peak_bits at k = ",join(',',@peak_k_list),
  "   k proportion ", $peak_k_list[-1]/$target_k, "\n";

my $grey = MyLineSegments->new;
{
  # full calculation
  my $prev = 0;
  foreach my $k (0 .. $target_k) {
    my $this = $w_num_bits[$k];
    $grey->add ($k,$prev, $k,$this);    # vertical, if necessary
    $grey->add ($k,$this, $k+1,$this);  # horizontal
    $prev = $this;
  }
}
{
  # limit stair step
  # stop at about midway between the actual peak and the full w
  my $top_limit_bits = int(.5 * ($w_num_bits[-1] + $peak_bits));
  ### stair step: "peak $peak_bits full $w_num_bits[-1] midway $top_limit_bits"
  my $prev = 0;
  foreach my $k (reverse 0 .. $target_k) {
    my $this = $limit_bits[$k];
    last if $this > $top_limit_bits;
    $grey->add ($k+1,$prev, $k+1,$this);    # vertical, if necessary
    $grey->add ($k+1,$this, $k,$this);  # horizontal
    $prev = $this;
  }
}

my $segments = MyLineSegments->new;
$segments->add(0,0, $target_k+5,0);    # axis line
{
  # actual calculation with limit applied
  my $prev = 0;
  foreach my $k (0 .. $target_k) {
    my $this = $actual_bits[$k];
    $segments->add ($k,$prev, $k,$this);    # vertical, if necessary
    $segments->add ($k,$this, $k+1,$this);  # horizontal
    $prev = $this;
  }
  $prev == 1 or die;
  $segments->add ($target_k+1,$prev, $target_k+1,0);  # final down to 0
}

$grey->remove_segments($segments);

$Printwrap = 2;
printwrap("\\draw[my grey] ");
$Printwrap += 2;
$grey->printwrap;
printwrap(";\n");

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

print "  \\coordinate (top) at (",$target_k+1,",$w_num_bits[$target_k]);\n";

print "  % end generated\n";
exit 0;
