#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use FindBin;
use Math::BigRat;

BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyDragon;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


print "% generated by tools/$FindBin::Script\n";
my $n_max = 10;

sub bigrat_to_tex {
  my ($f) = @_;
  if ($f == int($f)) {
    return "\$$f\$";
  } else {
    return ('$'
            . ($f < 0 ? '-' : '')
            . '\\tfrac' . abs($f->numerator) . $f->denominator
            . '$');
  }
}

my @half3on2_low = (Math::BigRat->new('0'),
                    Math::BigRat->new('-1/2'),
                    Math::BigRat->new('1/2'));

sub half3on2_digits {
  my ($n) = @_;
  $n = Math::BigRat->new($n);
  my @ret;
  while ($n > 1) {
    ### n: "$n"
    $n == int($n) or die;
    my $a = $half3on2_low[$n % 3];
    ### a: "$a"
    push @ret, $a;
    $n = ($n - $a) * Math::BigRat->new('2/3');
  }
  ### final n: "$n"
  if ($n) {
    push @ret, $n;
  }
  ### @ret
  return reverse @ret;   # high to low
}

my @half3on2_low_to_str = ('0', '\\tfrac12', '-\\tfrac12');

my $width = do {
  my @digits = half3on2_digits($n_max);
  scalar(@digits);
};
### $width

print "\\hspace{2em} % bias to the right\n";

foreach my $n (1 .. $n_max) {
  my @digits = half3on2_digits($n);
  ### $n
  ### digits: map {"$_"} @digits

  $Printwrap = 2;
  printwrap("\$$n = {} \\;\$");
  $Printwrap += 2;

  printwrap(' &' x ($width - scalar(@digits)));
  foreach my $i (0 .. $#digits) {
    my $str = bigrat_to_tex($digits[$i]);
    my $comma = $i < $#digits ? ',' : '\\phantom{,}';
    printwrap(" & $str$comma");
  }

  if ($n == 2) {
    printwrap(" & base \$\\tfrac32\$ \\kern.5em");
  } elsif ($n == 3) {
    printwrap(" & digits \$0, -\\tfrac12, +\\tfrac12\$");
  } elsif ($n == 4) {
    printwrap(" & and high \$1\$ \\kern.1em");
  }

  if ($n < $n_max) {
    my $nosplit = ($n < 5 || $n > $n_max-2 ? '*' : '');
    printwrap(" \\\\${nosplit}[.6ex]");
  }
  printwrap_newline();

  print "% GP-Test  half3on2_digits($n) == [",
    join(',', @digits), "]\n";
}

print "% end generated\n";
exit 0;
