#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyPrintwrap ':all';


my $level = 5;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
my $path = Math::PlanePath::DragonCurve->new;

$Printwrap = 2;
printwrap("\\begin{scope}[shift=(pos-$level)]\n");
$Printwrap += 2;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

$n_lo = 1*$n_hi;
my $n_join = 2*$n_hi;
$n_hi = 3*$n_hi;
my ($x_join,$y_join) = $path->n_to_xy($n_join);

my $is_touch = sub {
  my ($x,$y) = @_;
  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_ >= $n_lo && $_ < $n_hi} @n_list;
  return @n_list == 2 && $n_list[0] < $n_join && $n_list[1] >= $n_join;
};

my $rotate = sub {
  my ($x,$y) = @_;
  foreach (1 .. 2) {
    ($x,$y) = xy_rotate_minus90($x,$y);
  }
  return ($x,$y);
};

my $transform = sub {
  my ($x,$y) = @_;
  $x -= $x_join;
  $y -= $y_join;
  return $rotate->($x,$y);
};

printwrap("\\node (join-$level) at (0,0) [my dot];\n");
printwrap("\\draw ");
$Printwrap += 2;
my $join = '';
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  my $t = $is_touch->($x,$y);
  ($x,$y) = $transform->($x,$y);
  if ($t) {
    my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
    my ($tdx,$tdy) = $path->n_to_dxdy($n);
    ($sdx,$sdy) = $rotate->($sdx,$sdy);
    ($tdx,$tdy) = $rotate->($tdx,$tdy);
    my $sx = $x . chamfer(-$sdx);
    my $sy = $y . chamfer(-$sdy);
    my $tx = $x . chamfer($tdx);
    my $ty = $y . chamfer($tdy);
    printwrap("$join($sx,$sy)");
    printwrap("--($tx,$ty)");
  } else {
    printwrap("$join($x,$y)");
  }
  $join = '--';
}
printwrap(";\n");
$Printwrap = 2;
printwrap("\\end{scope}\n");

print "  % end generated\n";
exit 0;
