#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'transform_to_horizontal','GS_xy','div';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 5;
my $option = '';
if (@ARGV) { $option = $ARGV[0]; }

print "  % generated by tools/$FindBin::Script",
  $option ? " $option" : '',
  "\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
### n: "$n_lo to $n_hi"

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal(2*$x,2*$y, $level);
}

# scale to unit length, shift so centroid at origin 0,0 outside the {scope}
my $scale;
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  ($x,$y) = transform($x,$y);
  my $norm = $x*$x + $y*$y;
  my $abs = sqrt($norm);
  $scale = "scale={1/$abs}";
}
my $shift;
{
  my ($gx,$gy) = GS_xy($level);
  my ($sx,$sy) = (-$gx, -$gy);
  ($sx,$sy) = transform($sx,$sy);
  $shift = "shift={($sx,$sy)}";
}

print "  \\begin{scope}[$scale,$shift]\n";
print "    \\coordinate (start) at (0,0);\n";
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  ($x,$y) = transform($x,$y);
  print "    \\coordinate (end) at ($x,$y);\n";
}

my @dots;
my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $segments->add($x,$y, $x2,$y2);

  my $sx = ($x+$x2)/2;  # segment midpoint
  my $sy = ($y+$y2)/2;
  ($sx,$sy) = transform($sx,$sy);
  push @dots, "($sx,$sy),";
}

# circles in middle of each segment
unless ($option eq 'no-dots') {
  $Printwrap = 4;
  printwrap("\\fill foreach \\p in {");
  $Printwrap += 2;
  foreach my $dot (@dots) {
    printwrap($dot);
  }
  printwrap("}");
  printwrap(" { \\p circle[] };\n");
}

$Printwrap = 4;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);
printwrap(";\n");

print "  \\end{scope}\n";
print "  % end generated\n";
exit 0;
