#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Drawing downwards from the diameter, sub-trees directed left and down to
# make room for caption at right.

use 5.004;
use strict;
use Graph;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','xy_rotate_minus90';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 7;
if (@ARGV) { $max_level = shift @ARGV; }
$| = 1;

print "% generated by tools/$FindBin::Script $max_level\n";
my $path = Math::PlanePath::DragonCurve->new;

sub path_n_is_right_boundary {
  my ($path, $n, $n_limit) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}
sub path_n_is_left_boundary {
  my ($path, $n, $n_limit) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  ### consider: "$x1,$y1 to $x2,$y2   and $x1s,$y1s to $x2s,$y2s"
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}
sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}

print "% GP-DEFINE  BoundaryPathLen_by_diameter_samples = [";
my $join = '';
foreach my $level (0 .. $max_level) {
  my $graph = Graph->new (undirected => 1);

  my ($n_lo,$n_hi) = $path->level_to_n_range($level);
  foreach my $n ($n_lo .. $n_hi-1) {
    path_n_is_left_boundary($path, $n, $n_hi)
      or path_n_is_right_boundary($path, $n, $n_hi)
      or next;

    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my ($x2,$y2) = ($x+$dx, $y+$dy);

    $graph->add_edge("$x,$y", "$x2,$y2");
  }

  my $diameter = $graph->diameter;
  print "$join$diameter";
  $join = ',';
}
print "];\n";
print "% end generated\n";
exit 0;
