#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'max';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'mul','chamfer';
use MyPrintwrap ':all';


my $level = 3;

print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 2;

sub sum_move {
  my ($sx,$sy, $ex,$ey) = @_;
  return abs($sx-$ex) + abs($sy-$ey);
}
sub max_move {
  my ($sx,$sy, $ex,$ey) = @_;
  return max(abs($sx-$ex), abs($sy-$ey));
}

my $sum_move = 0;
my $max_move = 0;
my $prev = 0;
my ($pen_x,$pen_y) = (0,0);
my @draw_n;
my @move_norms;

sub draw {
  my ($k, $sx,$sy, $ex,$ey) = @_;
  if ($k > 0) {
    my $mx = ($sx+$ex)/2 + ($ey-$sy)/2;
    my $my = ($sy+$ey)/2 - ($ex-$sx)/2;
    draw($k-1, $sx,$sy, $mx,$my);
    draw($k-1, $ex,$ey, $mx,$my);

  } else {
    my $n = $path->xyxy_to_n_either($sx,$sy, $ex,$ey);
    push @draw_n, $n;

    my $dx = $ex - $sx;
    my $dy = $ey - $sy;
    my $cdx = chamfer($dx,"\\MyS");
    my $cdy = chamfer($dy,"\\MyS");
    my $ecdx = chamfer(-$dx,"\\MyE");
    my $ecdy = chamfer(-$dy,"\\MyE");
    printwrap(" \\coordinate (start) at ($sx$cdx,$sy$cdy);");

    if ($sx != $pen_x || $sy != $pen_y) {
      if ($sx == $pen_x || $sy == $pen_y) {
        printwrap(" \\draw[my arc move] (end) to (start);");
      } elsif ($sx == -2 && $sy == 2) {
        # special case left for manual drawing
      } else {
        printwrap(" \\draw[my move] (end)--(start);");
      }
    }
    $sum_move += sum_move($pen_x,$pen_y, $sx,$sy);
    $max_move += max_move($pen_x,$pen_y, $sx,$sy);
    push @move_norms, ($pen_x-$sx)**2 + ($pen_y-$sy)**2;

    printwrap(" \\coordinate (end) at ($ex$ecdx,$ey$ecdy);");
    printwrap(" \\draw[my line] (start)--(end);");

    ($pen_x,$pen_y) = ($ex,$ey);
  }
}

my ($ex,$ey) = (1,0);
foreach (1 .. $level) { ($ex,$ey) = mul($ex,$ey); }
draw($level, 0,0, $ex,$ey);
printwrap_newline();

# print "  \\newcommand{\\MyLevel}{$level}\n";
# print "  \\newcommand{\\MyMoveMax}{$max_move}\n";
# print "  \\newcommand{\\MyDrawEndX}{$pen_x}\n";
# print "  \\newcommand{\\MyDrawEndY}{$pen_y}\n";
print "  \\coordinate (draw-end) at ($pen_x,$pen_y);\n";
print "  \\coordinate (curve-end) at ($x_hi,$y_hi);\n";

print "% GP-Test-Last  End($level)==$pen_x+($pen_y)*I\n";
print "% GP-Test-Last  Move($level)==$max_move\n";
print "% GP-DEFINE  gen_plotter_k = $level;\n";
print "% GP-DEFINE  gen_plotter_middle_n_sequence = [",join(',',@draw_n),"];\n";
print "% GP-DEFINE  gen_plotter_move_norms = [",join(',',@move_norms),"];\n";

print "  % end generated\n";
exit 0;
