#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'JA','RQ','xy_rotate_plus90','xy_rotate_minus90',
  'rect_corners_sort','@dir4_to_dx','@dir4_to_dy','chamfer';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 8;
if (@ARGV) { $level = shift @ARGV; }
print "  % generated by tools/$FindBin::Script\n";

sub path_n_is_right_boundary {
  my ($path, $n, $n_limit) = @_;
  ### path_n_is_right_boundary(): $n
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  ### coords: "$x1,$y1  $x2,$y2  side $x1s,$y1s  $x2s,$y2s"
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}
sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

{
  $Printwrap = 2;
  printwrap("\\path foreach \\p in {");
  $Printwrap = 4;
  foreach my $k (0 .. $level-1) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x,$y) = $path->n_to_xy($n_hi);
    printwrap("($x,$y),");
  }
  printwrap("}");
  printwrap(" { \\p node[my dot] };\n");
}

my $squares;
my %seen;
my $jk = 1;
my $jdir = 0;
my $j = 0;
my @fill_lists;
my @draw_list;
my @join_strs;

{
  $Printwrap = 2;
  # printwrap("\\draw[my line] ");
  my $sep = '';
  $Printwrap = 4;
  my @fill_list;

  foreach my $n ($n_lo .. $n_hi - 1) {
    next unless path_n_is_right_boundary($path,$n, $n_hi);
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    my $x2 = $x1 + $dx;
    my $y2 = $y1 + $dy;
    ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
    $x2 += $dx;
    $y2 += $dy;  # opposite corner

    ($x1,$y1, $x2,$y2) = rect_corners_sort ($x1,$y1, $x2,$y2);
    ### corners: "$x1,$y1  $x2,$y2"
    next if $seen{"$x1,$y1"}++;

    if ($j <= 0) {
      $jdir ^= 1;
      if (! $jdir) { $jk++; }
      $j = JA($jk);
      push @join_strs, $j;
      if ($jdir) {
        if ($n <= 2) {
          push @draw_list, "$x1,$y1";
        } else {
          push @fill_lists, [ @fill_list ];
          @fill_list = ();
          push @fill_list, "$x1,$y1";
        }
        $j--;
      } else {
        push @fill_lists, [ @fill_list ];
        @fill_list = ();
        push @draw_list, "$x1,$y1";
      }
    } else {
      push @fill_list, "$x1,$y1";
      $j--;
    }
    $squares++;
    $x1 += .5;
    $y1 += .5;
    # printwrap("$sep($x1,$y1)");
    $sep = '--';
  }
  # printwrap(";\n");
  push @fill_lists, \@fill_list;
}

my $RQ = RQ($level);
$squares == $RQ
  or die "squares $squares RQ $RQ";

pop @fill_lists;
pop @draw_list;

{
  $Printwrap = 2;
  printwrap("\\draw foreach \\p in {");
  $Printwrap = 4;
  foreach my $p (@draw_list) { printwrap("($p),"); }
  printwrap("}\n");
  print "    { \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n";
}

# {
#   $Printwrap = 2;
#   printwrap("\\fill foreach \\p in {");
#   $Printwrap = 4;
#   foreach my $p (@fill_list) { printwrap("($p),"); }
#   printwrap("}\n");
#   print "    { \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n";
# }

foreach my $fill_list (@fill_lists) {
  ### fill_list: join('  ',@$fill_list)
  next unless @$fill_list;

  $Printwrap = 2;
  printwrap("\\fill ");
  $Printwrap = 4;

  my @points;
  my @points_dir;
  my @points_gap;
  my @prev_corners;

 P: foreach my $p (@$fill_list) {
    ### $p
    my ($x,$y) = split /,/, $p;
    my @corners = ($x   .','. $y     ,
                   ($x+1).','. $y    ,
                   ($x+1).','.($y+1) ,
                   $x   .','.($y+1)  ,
                  );
    foreach my $corners_pos (0 .. $#corners) {
      foreach my $prev_pos (0 .. $#prev_corners) {
        if ($corners[$corners_pos] eq $prev_corners[$prev_pos]) {
          ### $corners_pos
          ### $prev_pos

          foreach my $points_pos (reverse 0 .. $#points) {
            if ($points[$points_pos] eq $corners[$corners_pos]) {
              ### shuffle points ...
              unshift @points,     (splice @points,     $points_pos);
              unshift @points_dir, (splice @points_dir, $points_pos);
              unshift @points_gap, (splice @points_gap, $points_pos);
              push @points,     $points[0];
              push @points_dir, $points_dir[0];
              push @points_gap, ($prev_pos+1)%4;
              $points_gap[0] == -1 or die;
              $points_gap[0] = $prev_pos;
              last;
            }
          }
          foreach my $i (0 .. $#corners + 1) {
            my $pos = ($corners_pos + $i) % scalar(@corners);
            push @points,     $corners[$pos];
            push @points_dir, $pos;
          }
          push @points_gap, $corners_pos, -1,-1,-1, ($corners_pos+1)%4;
          @prev_corners = @corners;
          next P;
        }
      }
    }

    push @points,     @corners;
    push @points_dir, 0 .. 3;
    push @points_gap, -1,-1,-1,-1;
    @prev_corners = @corners;
  }
  ### points: scalar(@points)."  ".join('  ',@points)
  ### dirs: scalar(@points_dir)."  ".join(', ',@points_dir)
  ### gaps: scalar(@points_gap)."  ".join(', ',@points_gap)

  foreach my $i (0 .. $#points) {
    my $p = $points[$i];
    my $dir = $points_dir[$i];
    my $gap = $points_gap[$i];
    ### $p
    ### $dir
    ### $gap

    my ($x,$y) = split /,/, $p;
    my $dx = $dir4_to_dx[$dir];
    my $dy = $dir4_to_dy[$dir];
    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);  # to left
    $dx += $lx;
    $dy += $ly;

    $x .= chamfer($dx, "\\MyMargin");
    $y .= chamfer($dy, "\\MyMargin");

    if ($gap >= 0) {
      my $gx = $dir4_to_dx[$gap];
      my $gy = $dir4_to_dy[$gap];
      $x .= chamfer($gx, "\\MyGap");
      $y .= chamfer($gy, "\\MyGap");
    }
    printwrap("($x, $y)--");
  }
  printwrap("cycle;\n");
}

print "  % fill lists: ",join(', ', map {scalar(@$_)} @fill_lists),"\n";
print "  % joins:   ",join(', ', @join_strs),"\n";

print "  % end generated\n";
exit 0;
