#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $width = 20;
my $height = 8;
print "  % generated by tools/$FindBin::Script\n";

my $segments = MyLineSegments->new;

sub xy_rotate_dir4 {
  my ($x,$y, $dir4) = @_;
  $dir4 %= 4;
  foreach (1 .. $dir4) {
    ($x,$y) = xy_rotate_plus90($x,$y);
  }
  return ($x,$y);
}

{
  $Printwrap = 2;
  printwrap("\\foreach \\p in {");
  $Printwrap = 4;
  my @rect_dx = (0, 2, 2, 0);
  my @rect_dy = (0, 0, 1, 1);
  for (my $x = 0; $x <= $width; $x += 2) {
    for (my $y = 0; $y <= $height; $y += 2) {
      ### xy: "$x, $y"
      my $m = $x/2 + $y/2;

      if ($x%4==0 && $y%4==0) {
        printwrap("($x,$y),");
        $Printwrap = 2;
        $Printwrap = 4;
      }

      my $even = (($x/2+$y/2) % 2 == 0);

      foreach my $dir4 (0 .. 3) {
        foreach my $i (0 .. $#rect_dx) {
          my $dx1 = $rect_dx[$i-1];
          my $dy1 = $rect_dy[$i-1];
          my $dx2 = $rect_dx[$i];
          my $dy2 = $rect_dy[$i];
          ### dxdy: "$dx1,$dy1  $dx2,$dy2"

          if (! $even) {
            $dy1 = - $dy1;
            $dy2 = - $dy2;
          }

          ($dx1,$dy1) = xy_rotate_dir4($dx1,$dy1, $dir4);
          ($dx2,$dy2) = xy_rotate_dir4($dx2,$dy2, $dir4);
          ### seg: $x+$dx1, $y+$dy1,   $x+$dx2, $y+$dy2
          $segments->add($x+$dx1, $y+$dy1,   $x+$dx2, $y+$dy2);
        }
      }
    }
  }
  printwrap("}");
  printwrap(" { \\path \\p pic[transform shape] {my pinwheel}; }\n");
}

{
  $Printwrap = 2;
  printwrap("\\path foreach \\p in {");
  $Printwrap = 4;
  for (my $x = 0; $x <= $width; $x += 2) {
    for (my $y = 0; $y <= $height; $y += 2) {
      printwrap("($x,$y),");
    }
  }
  printwrap("}");
  printwrap(" { \\p node[my dot] };\n");
}

{
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

print "  % end generated\n";
exit 0;
