#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::Complex;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'points_str','hull_points_sort_by_angle',
  'P1_xy','P2_xy','P3_xy','P4_xy', 'xy_rotate_minus90';
use MyLineSegments;
use MyPrintwrap ':all';

# 6 and 7
my $level = 6;
if (@ARGV) { $level = $ARGV[0]; }

my $path = Math::PlanePath::DragonCurve->new;

print "  % generated by tools/$FindBin::Script $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $transform = sub {
  my ($x,$y) = @_;
  foreach (1 .. int($level/2)+1) {
    ($x,$y) = xy_rotate_minus90($x,$y);
  }
  return ($x,$y);
};

# curve
{
  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    foreach (1, 2) {
      $segments->add($x1,$y1, $x2,$y2);
      $x1 = $x_hi - $x1;
      $y1 = $y_hi - $y1;
      $x2 = $x_hi - $x2;
      $y2 = $y_hi - $y2;
    }
  }

  $Printwrap = 2;
  printwrap("\\draw[my dragon line] ");
  $Printwrap += 2;
  $segments->printwrap (transform => $transform);
  printwrap(";\n");

  my ($x_min,$y_min, $x_max,$y_max) = $segments->bbox;
  print "  \\coordinate (top-right) at ($x_max,$y_max);\n";
}

{
  my ($x,$y) = $transform->(0,0);
  print "  \\node (start) at ($x,$y) [my small dot];\n";
}
{
  my ($x,$y) = $transform->($x_hi,$y_hi);
  print "  \\node (end) at ($x,$y) [my small dot];\n";
}
{
  my ($x,$y) = $transform->($x_hi/2,$y_hi/2);
  print "  \\node (middle) at ($x,$y) [my middle dot];\n";
}

{
  my @hull_elems = ([ 'P3', P3_xy($level+1) ],
                    [ 'P3o', P3_xy($level+1) ]);
  foreach my $elem (@hull_elems) {
    my ($name, $x,$y) = @$elem;
    if ($name =~ /o/) { ($x,$y) = ($x_hi - $x, $y_hi - $y); }
    my ($tx,$ty) = $transform->($x,$y);
    print "  \\node ($name) at ($tx,$ty) [my hull dot];\n";
  }
  foreach my $elem (@hull_elems) {
    my ($name, $x,$y) = @$elem;
    if ($name =~ /o/) { ($x,$y) = ($x_hi - $x, $y_hi - $y); }
    print "% GP-Test  T_$name($level) == $x+I*$y\n";
  }
}

print "  % end generated\n";
exit 0;
