\\ Copyright 2019, 2020, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


OEIS_samples(anum) =
{
  extern(Str("my-oeis-samples.pl "anum));

  \\ my(filename=Str("/tmp/OEIS-samples-",random(1000000)));
  \\ my(vec=read(filename));
  \\ system(Str("rm "filename));
  \\ vec;
}
{
addhelp(OEIS_samples,
"vec = OEIS_samples(anum)
Return a vector of the sample values of anum.
anum can be a symbol like 'A123456 or a string like \"A123456\".
The values are read from the OEIS stripped file which is expected to be
at ~/OEIS/stripped.

If anum does not exist, or the stripped file is not available, then throw an
error().");
}

\\ v = OEIS_bfile_samples(anum)
OEIS_bfile_samples(anum) =
{
  \\ v is like [ 6, [100,101,102] ] with n of first term in the vector.
  extern(Str("my-oeis-samples.pl "anum" bfile"))[2];
}

\\ func = OEIS_samples_func(anum)
OEIS_bfile_func(anum) =
{
  \\ v is like [ 6, [100,101,102] ] with n of first term in the vector.
  my(v=extern(Str("my-oeis-samples.pl "anum" bfile")));
  (n) -> n>=v[1] || error(anum," is for n>=",v[1]);
         v[2][n-v[1]+1];
}

\\ ~/OEIS/b238837.txt
\\ my-oeis-samples.pl /tmp/x A238837 bfile_gf
OEIS_bfile_gf(anum) =
{
  \\ v is like [ 6, [100,101,102] ] for 100*x^6 + 100*x^7 + 100*x^8
  \\ A single Polrev is done rather than say elements [i,value] since circa
  \\ GP 2.11 polys are dense and it seems adding term by term ends up O(n^2)
  \\ time.
  my(v=extern(Str("my-oeis-samples.pl "anum" bfile")));
  Polrev(v[2])*'x^v[1];

  \\ v elements [i, value]
  \\ sum(i=1,#v, v[i][2] * 'x^v[i][1]);
  \\ my(filename=Str("/tmp/OEIS-samples-",random(1000000)));
  \\ my(vec=read(filename));
  \\ system(Str("rm "filename));
  \\ vec;

  \\ Polrev(v[2])*'x^v[1];
  \\ extern("echo 123")
  \\ g = OEIS_bfile_gf("A079954")
  \\ extern("my-oeis-samples.pl A079954 bfile_gf")
  \\ extern("cat /tmp/x.gp")
  \\ OEIS_samples_gf__INTERNAL

}
{
addhelp(OEIS_bfile_gf,
"poly = OEIS_bfile_gf(anum)
Return a polynomial whose coefficients are the terms from the b-file of
anum.  anum can be a symbol like 'A123456 or a string like \"A123456\".
The b-file is expected to be in ~/OEIS/b123456.txt.
If the b-file does not exist then an internal format download
~/OEIS/A123456.internal.txt is used.
If neither of these exist then throw an error().");
}

OEIS_print_STU(vec) =
{
  my(STU=['S,'T,'U],
    upto=1);
  for(i=1,#STU,
     my(join=Str("%",STU[i]," "),
        str="");
     while(1,
           if(upto>#vec,break(2));
           my(s=Str(join,vec[upto]));
           if(length(str)+length(s)>70, break);
           str=concat(str,s);
           upto++;
           join=",");
     if(i!=#STU,str=concat(str,","));
     print(str));
}

\\ OEIS_bfile_gf('A999999)
\\ OEIS_samples('A888888)


\\ OEIS_samples(anum) =
\\ {
\\   extern(Str("perl -MMath::OEIS::Stripped -e 'print q{[},Math::OEIS::Stripped->anum_to_values_str($ARGV[0]),q{]}' ", anum));
\\ }

OEIS_recurrence_guess(vec) =
{
  recurrence_guess(vec);
  my(f=recurrence_guess_values_to_factors(vec));
  print(f);
  while(#f&&f[1]==0,f=f[^1]);
  f=Vecrev(f);
  print(f);
  printf("%%H <a href=\"/index/Rec#order_%02d\">Index entries for linear recurrences with constant coefficients</a>, signature (", #f);
  for(i=1,#f, print1(f[i]); if(i!=#f,print1(",")));
  print(").");

  print1("a(n) = ");
  my(join=0);
  for(i=1,#f,
     f[i] || next;
     my(c=f[i]);
     if(join,
        print1(if(c<0," - "," + "));
        c=abs(c));
     if(c!=1, print1(c,"*"));
     print1("a(n-",i,")");
     join=1);
  print(".");
}