#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon 'chamfer','transform_to_horizontal','mul','xy_rotate_plus90';
use MyPrintwrap ':all';
use MyLineSegments;


my $level = 3;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

sub transform {
  my ($x,$y) = @_;
  ($x,$y) = transform_to_horizontal($x,$y, $level);
  return ($x, $y);
}

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

{
  $Printwrap = 2;
  my @between;
  my ($prev_x,$prev_y);
  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    ($x1,$y1)=transform($x1,$y1);
    ($x2,$y2)=transform($x2,$y2);
    ($dx,$dy)=transform($dx,$dy);

    $x1 = $x1 . chamfer($dx);
    $y1 = $y1 . chamfer($dy);
    $x2 = $x2 . chamfer(-$dx);
    $y2 = $y2 . chamfer(-$dy);

    my $arrow = ($n % 2 ? '<-' : '->');
    printwrap("\\draw[$arrow,my m] ($x1,$y1)--($x2,$y2);\n");

    if (defined $prev_x) {
      push @between, " ($prev_x,$prev_y)--($x1,$y1)";
    }
    ($prev_x,$prev_y) = ($x2,$y2);
  }
  printwrap_newline();
  printwrap("\\draw");
  $Printwrap += 2;
  foreach my $between (@between) {
    printwrap($between);
  }
  printwrap(";\n");
}

{
  my $segments = MyLineSegments->new;

  foreach my $elem ([0,0, 1],
                    [0,0, 2],
                    [0,0, 3],
                    [2,0, 2],
                    [1,1, 3],
                    [1,-1, 1],
                    [1,-1, 2]) {
    my ($ox,$oy, $dir) = @$elem;
    foreach (1 .. $level) { ($ox,$oy)=mul($ox,$oy); }

    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      foreach (1 .. $dir) {
        ($x1,$y1) = xy_rotate_plus90($x1,$y1);
        ($x2,$y2) = xy_rotate_plus90($x2,$y2);
      }
      $x1 += $ox;
      $y1 += $oy;
      $x2 += $ox;
      $y2 += $oy;
      $segments->add($x1,$y1, $x2,$y2);
    }
  }

  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap (transform => \&transform);
  printwrap(";\n");
}

print "  % end generated\n";
exit 0;
