#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use FindBin;
use Math::BigInt try => 'GMP';
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyDragon;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::DragonCurve->new;
my $x_bits = '1101000011111101000000000000110100000000000000000000000011010000';
# $x_bits = '0000';
my $x_segment  = Math::BigInt->new("0b$x_bits");
my $k = length($x_bits);
my ($n_lo, $n_hi) = $path->level_to_n_range(2*$k);

sub transform {
  my ($x,$y) = @_;
  $x -= $x_segment;
  return ($x,$y);
}

print "  \\node (x) at (0,0) [my dot];\n";

# within $n_hi
sub is_traversed {
  my ($x1,$y1, $x2,$y2) = @_;
  my $n = $path->xyxy_to_n_either ($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_hi;
}

my $segments = MyLineSegments->new;
my $x_lo = $x_segment - 4;
my $x_hi = $x_segment + 4;
my $y_lo = -2;
my $y_hi = 2;
for (my $x = $x_lo; $x <= $x_hi; $x++) {
  for (my $y = Math::BigInt->new($y_lo); $y <= $y_hi; $y++) {
    {
      my $x2 = $x+1;
      my $y2 = $y;
      if ($x2 <= $x_hi) {
        ### consider: "$x,$y to $x2,$y2"
        if (is_traversed($x,$y, $x2,$y2)) {
          ### yes ...
          $segments->add (transform($x,$y),
                          transform($x2,$y2));
        }
      }
    }
    {
      my $x2 = $x;
      my $y2 = $y+1;
      if ($y2 <= $y_hi) {
        ### consider: "$x,$y to $x2,$y2"
        if (is_traversed($x,$y, $x2,$y2)) {
          ### yes ...
          $segments->add (transform($x,$y),
                          transform($x2,$y2));
        }
      }
    }
  }
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap +=2;
$segments->printwrap;
printwrap(";\n");

print "  % end generated\n";
exit 0;
