#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Gray filled fractal, rotated and scaled 0 to 1.

use 5.010;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon 'xy_rotate_plus90','xy_rotate_minus90','transform_to_horizontal',
  'B',;
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


print "  % generated by tools/$FindBin::Script\n";
my $level = 14;
if (@ARGV) { $level = shift @ARGV; }

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my $scale = 2**($level/2);
print "  \\begin{scope}[scale=1/$scale]\n";

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

sub path_n_is_right_boundary {
  my ($path, $n, $n_limit) = @_;
  ### path_n_is_right_boundary(): $n
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}
sub path_n_is_left_boundary {
  my ($path, $n, $n_limit) = @_;
  ### path_n_is_left_boundary(): $n
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}
sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}

my $B = 0;
{
  my @right;
  my @left;
  my @fill;
  my $segments = MyLineSegments->new;

  foreach my $n ($n_lo .. $n_hi-1) {
    my $left = path_n_is_left_boundary($path,$n,$n_hi);
    my $right = path_n_is_right_boundary($path,$n,$n_hi);
    if ($right && ! $left) {
      my ($x,$y) = $path->n_to_xy($n);
      push @right, [$x,$y];
      $B++;
    }
    if ($left && ! $right) {
      my ($x2,$y2) = $path->n_to_xy($n+1);
      push @left, [$x2,$y2];
      $B++;
    }
    if ($left && $right) {
      push @fill, [ @right, reverse(@left) ];
      @left = ();
      @right = ();
      $segments->add($path->n_to_xy($n),
                     $path->n_to_xy($n+1));
      $B += 2;
    }
  }
  $B == B($level) or die;

  $Printwrap = 4;
  printwrap("\\fill[my gray] ");
  $Printwrap += 2;
  my $newline = '';
  foreach my $fill (@fill) {
    next unless @$fill;
    printwrap($newline);

    my $join = '';
    foreach my $p (@$fill) {
      my ($x,$y) = @$p;
      ($x,$y) = transform($x,$y);
      printwrap("$join($x,$y)");
      $join = '--';
    }
    printwrap("--cycle");
    $newline = "\n";
  }
  printwrap(";\n");

  $Printwrap = 4;
  printwrap("\\draw[my gray] ");
  $Printwrap += 2;
  $segments->printwrap (transform => \&transform);
  printwrap(";\n");
}

print "  \\end{scope}\n";
print "% GP-Test  B($level) == $B\n";

print "  % end generated\n";
exit 0;
