#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::DragonCurve->new;
foreach my $level (7,8) {
  my $letter = substr('ZABCDEFG',$level,1);
  my ($n_lo, $n_mid) = $path->level_to_n_range($level);
  my ($xm, $ym) = $path->n_to_xy($n_mid);
  my $n_hi = 2*$n_mid;

  my $xy_is_join = sub {
    my ($x,$y) = @_;
    my @n_list = $path->xy_to_n_list($x,$y);
    return (@n_list > 1
            && $n_list[0] < $n_mid
            && $n_list[1] > $n_mid);
  };

  printwrap_indent("");
  printwrap("  \\begin{scope}[shift=(pos-$level),rotate={-atan2($ym,$xm)},scale=\\MyScale$letter,\n");
  printwrap("                every circle/.style={radius=\\MyCircleRadius/\\MyScale$letter}]\n");
  printwrap("    % $n_lo to $n_mid to $n_hi\n");

  printwrap_indent("   ");
  foreach my $n ($n_lo, $n_mid, $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    printwrap(" \\fill ($x,$y) circle;");
  }

  printwrap(" \\draw");
  printwrap_indent("      ");
  my $join = '';
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my @n_list = $path->xy_to_n_list($x,$y);
    if ($xy_is_join->($x,$y)) {
      my ($dx,$dy) = $path->n_to_dxdy($n-1);
      my ($tdx,$tdy) = $path->n_to_dxdy($n);
      printwrap("$join($x-\\MyChamfer*($dx,$y-\\MyChamfer*$dy)--($x+\\MyChamfer*$tdx,$y+\\MyChamfer*$tdy)");
    } else {
      printwrap("$join($x,$y)");
    }
    $join = '--';
  }
  printwrap(";\n");

  printwrap_indent("");
  printwrap("  \\end{scope}\n");
}
print "  % end generated\n";
exit 0;
