#!/usr/bin/perl -w

# Copyright 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print ASCII-art picture of complex base i-1, rotated, and their
# intersection which gives MinusNegA.

use 5.010;
use strict;
use File::Spec;
use Math::PlanePath::ComplexMinus 117; # v.117 for level_to_n_range()

use FindBin;
use lib File::Spec->catdir($FindBin::Bin, File::Spec->updir, 'tools');
use MyPlanar;

my $level = 3;
if (@ARGV) { $level = shift @ARGV; }

my $path = Math::PlanePath::ComplexMinus->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $points = [map {[$path->n_to_xy($_)]} $n_lo .. $n_hi];
my $other_points = MyPlanar::points_rotate_plus90($points, 2);  # 180

my ($xmin,$ymin, $xmax,$ymax) = MyPlanar::points_bbox_xyxy([@$points,@$other_points]);

print "plain\n";
foreach my $y (reverse $ymin .. $ymax) {
  foreach my $x ($xmin .. $xmax) {
    my $n = $path->xy_to_n($x,$y);
    print ' ', $n > $n_hi ? ' '
      : $x==0 && $y==0 ? 'o'
      : '*';
  }
  print "\n";
}

print "rotated\n";
foreach my $y (reverse $ymin .. $ymax) {
  foreach my $x ($xmin .. $xmax) {
    my $n2 = $path->xy_to_n(-$x,-$y);
    print ' ', $n2 > $n_hi ? ' '
      : $x==0 && $y==0 ? 'o'
      : '*';
  }
  print "\n";
}

print "intersection\n";
foreach my $y (reverse $ymin .. $ymax) {
  foreach my $x ($xmin .. $xmax) {
    my $n = $path->xy_to_n($x,$y);
    my $n2 = $path->xy_to_n(-$x,-$y);
    print ' ', $n > $n_hi || $n2 > $n_hi ? ' '
      : $x==0 && $y==0 ? 'o'
      : '*';
  }
  print "\n";
}
exit 0;
